@@ -24,6 +24,8 @@
 #include <QWidget>
 #include <QLabel>
 #include <QDialogButtonBox>
+#include <QMultiMap>
+
 #include "qgis_gui.h"
 
 
@@ -199,6 +201,12 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
      */
     void setExtraContextScope( QgsExpressionContextScope *extraScope SIP_TRANSFER );
 
+    /**
+     * Returns TRUE if any of the form widgets need feature geometry
+     * \since QGIS 3.20
+     */
+    bool needsGeometry() const;
+
   signals:
 
     /**
@@ -274,7 +282,6 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
      */
     void openFilteredFeaturesAttributeTable( const QString &filter );
 
-
   public slots:
 
     /**
@@ -286,6 +293,13 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
      */
     void changeAttribute( const QString &field, const QVariant &value, const QString &hintText = QString() );
 
+    /**
+     * Changes the \a geometry of the feature attached to the form.
+     *
+     * \since QGIS 3.30
+     */
+    void changeGeometry( const QgsGeometry &geometry );
+
     /**
      * Update all editors to correspond to a different feature.
      *
@@ -338,12 +352,6 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
      */
     void parentFormValueChanged( const QString &attribute, const QVariant &newValue );
 
-    /**
-     * Returns TRUE if any of the form widgets need feature geometry
-     * \since QGIS 3.20
-     */
-    bool needsGeometry() const;
-
   private slots:
     void onAttributeChanged( const QVariant &value, const QVariantList &additionalFieldValues );
     void onAttributeAdded( int idx );
@@ -406,8 +414,6 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
 
     WidgetInfo createWidgetFromDef( const QgsAttributeEditorElement *widgetDef, QWidget *parent, QgsVectorLayer *vl, QgsAttributeEditorContext &context );
 
-    void addWidgetWrapper( QgsEditorWidgetWrapper *eww );
-
     /**
      * Creates widget wrappers for all suitable widgets found.
      * Called once maximally.
@@ -442,6 +448,7 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
     void updateContainersVisibility();
     void updateConstraint( const QgsFeature &ft, QgsEditorWidgetWrapper *eww );
     void updateLabels();
+    void updateEditableState();
     bool currentFormValuesFeature( QgsFeature &feature );
     bool currentFormValidConstraints( QStringList &invalidFields, QStringList &descriptions ) const;
     bool currentFormValidHardConstraints( QStringList &invalidFields, QStringList &descriptions ) const;
@@ -463,10 +470,11 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
     QDialogButtonBox *mButtonBox = nullptr;
     QWidget *mSearchButtonBox = nullptr;
     QList<QgsAttributeFormInterface *> mInterfaces;
-    QMap< int, QgsAttributeFormEditorWidget * > mFormEditorWidgets;
+    QMultiMap< int, QgsAttributeFormEditorWidget * > mFormEditorWidgets;
     QList< QgsAttributeFormWidget *> mFormWidgets;
     QMap<const QgsVectorLayerJoinInfo *, QgsFeature> mJoinedFeatures;
     QMap<QLabel *, QgsProperty> mLabelDataDefinedProperties;
+    QMap<QWidget *, QgsProperty> mEditableDataDefinedProperties;
     bool mValuesInitialized = false;
     bool mDirty = false;
     bool mIsSettingFeature = false;
@@ -544,13 +552,13 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
      * Dependency map for default values. Attribute index -> widget wrapper.
      * Attribute indexes will be added multiple times if more than one widget depends on them.
      */
-    QMap<int, QgsWidgetWrapper *> mDefaultValueDependencies;
+    QMultiMap<int, QgsWidgetWrapper *> mDefaultValueDependencies;
 
     /**
      * Dependency map for values from virtual fields. Attribute index -> widget wrapper.
      * Attribute indexes will be added multiple times if more than one widget depends on them.
      */
-    QMap<int, QgsWidgetWrapper *> mVirtualFieldsDependencies;
+    QMultiMap<int, QgsWidgetWrapper *> mVirtualFieldsDependencies;
 
     /**
      * Dependency list for values depending on related layers.
