@@ -18,19 +18,16 @@
 
 #include "qgis_3d.h"
 
-#include <QPointer>
-#include <QRect>
 #include <Qt3DCore/QEntity>
 #include <Qt3DInput/QMouseEvent>
 #include <QImage>
 
+#ifndef SIP_RUN
 namespace Qt3DInput
 {
   class QKeyEvent;
-  class QKeyboardDevice;
   class QKeyboardHandler;
   class QMouseEvent;
-  class QMouseDevice;
   class QMouseHandler;
   class QWheelEvent;
 }
@@ -38,19 +35,19 @@ namespace Qt3DInput
 namespace Qt3DRender
 {
   class QCamera;
-  class QPickEvent;
 }
 
+#endif
+
 #include "qgscamerapose.h"
 
 class QDomDocument;
 class QDomElement;
 
 class QgsCameraPose;
-class QgsTerrainEntity;
 class QgsVector3D;
-
-#define SIP_NO_FILE
+class QgsWindow3DEngine;
+class Qgs3DMapScene;
 
 /**
  * \ingroup 3d
@@ -58,44 +55,35 @@ class QgsVector3D;
  * \note Not available in Python bindings
  * \since QGIS 3.0
  */
+#ifndef SIP_RUN
 class _3D_EXPORT QgsCameraController : public Qt3DCore::QEntity
 {
+#else
+class _3D_EXPORT QgsCameraController : public QObject
+{
+#endif
+
     Q_OBJECT
-    Q_PROPERTY( Qt3DRender::QCamera *camera READ camera WRITE setCamera NOTIFY cameraChanged )
-    Q_PROPERTY( QRect viewport READ viewport WRITE setViewport NOTIFY viewportChanged )
   public:
 
-    //! The navigation mode used by the camera
-    enum NavigationMode
-    {
-      TerrainBasedNavigation, //!< The default navigation based on the terrain
-      WalkNavigation //!< Uses WASD keys or arrows to navigate in walking (first person) manner
-    };
-    Q_ENUM( NavigationMode )
-
-    //! Vertical axis inversion options
-    enum VerticalAxisInversion
-    {
-      Never, //!< Never invert vertical axis movements
-      WhenDragging, //!< Invert vertical axis movements when dragging in first person modes
-      Always, //!< Always invert vertical axis movements
-    };
-    Q_ENUM( VerticalAxisInversion )
-
-  public:
     //! Constructs the camera controller with optional parent node that will take ownership
-    QgsCameraController( Qt3DCore::QNode *parent = nullptr );
+    QgsCameraController( Qgs3DMapScene *scene ) SIP_SKIP;
+    ~QgsCameraController() override;
 
-    //! Returns camera that is being controlled
+#ifndef SIP_RUN
+
+    /**
+     * Returns camera that is being controlled
+     * \note Not available in Python bindings
+     */
     Qt3DRender::QCamera *camera() const { return mCamera; }
-    //! Returns viewport rectangle
-    QRect viewport() const { return mViewport; }
+#endif
 
     /**
      * Returns the navigation mode used by the camera controller.
      * \since QGIS 3.18
      */
-    QgsCameraController::NavigationMode cameraNavigationMode() const { return mCameraNavigationMode; }
+    Qgis::NavigationMode cameraNavigationMode() const { return mCameraNavigationMode; }
 
     /**
      * Returns the camera movement speed
@@ -113,25 +101,14 @@ class _3D_EXPORT QgsCameraController : public Qt3DCore::QEntity
      * Returns the vertical axis inversion behavior.
      * \since QGIS 3.18
      */
-    QgsCameraController::VerticalAxisInversion verticalAxisInversion() const { return mVerticalAxisInversion; }
+    Qgis::VerticalAxisInversion verticalAxisInversion() const { return mVerticalAxisInversion; }
 
     /**
      * Sets the vertical axis \a inversion behavior.
      * \since QGIS 3.18
      */
-    void setVerticalAxisInversion( QgsCameraController::VerticalAxisInversion inversion );
+    void setVerticalAxisInversion( Qgis::VerticalAxisInversion inversion );
 
-    /**
-     * Connects to object picker attached to terrain entity. Called internally from 3D scene.
-     * This allows camera controller understand how far from the camera is the terrain under mouse cursor.
-     * Also it allows adjustment of camera's view center to a point on terrain.
-     */
-    void setTerrainEntity( QgsTerrainEntity *te );
-
-    //! Assigns camera that should be controlled by this class. Called internally from 3D scene.
-    void setCamera( Qt3DRender::QCamera *camera );
-    //! Sets viewport rectangle. Called internally from 3D canvas. Allows conversion of mouse coordinates.
-    void setViewport( QRect viewport );
     //! Called internally from 3D scene when a new frame is generated. Updates camera according to keyboard/mouse input
     void frameTriggered( float dt );
 
@@ -201,6 +178,20 @@ class _3D_EXPORT QgsCameraController : public Qt3DCore::QEntity
     //! Move the map by \a tx and \a ty
     void moveView( float tx, float ty );
 
+    /**
+     * Walks into the map by \a tx, \a ty, and \a tz
+     * \since QGIS 3.30
+     */
+    void walkView( double tx, double ty, double tz );
+
+    /**
+     * Rotates the camera on itself.
+     * \param diffPitch the pitch difference
+     * \param diffYaw the yaw difference
+     * \since QGIS 3.30
+     */
+    void rotateCamera( float diffPitch, float diffYaw );
+
     /**
      * Returns TRUE if the camera controller will handle the specified key \a event,
      * preventing it from being instead handled by parents of the 3D window before
@@ -214,7 +205,7 @@ class _3D_EXPORT QgsCameraController : public Qt3DCore::QEntity
      * Sets the navigation mode used by the camera controller.
      * \since QGIS 3.18
      */
-    void setCameraNavigationMode( QgsCameraController::NavigationMode navigationMode );
+    void setCameraNavigationMode( Qgis::NavigationMode navigationMode );
 
     /**
      * Sets the depth buffer image used by the camera controller to calculate world position from a pixel's coordinates and depth
@@ -223,17 +214,22 @@ class _3D_EXPORT QgsCameraController : public Qt3DCore::QEntity
     void depthBufferCaptured( const QImage &depthImage );
 
   private:
-    void rotateCamera( float diffPitch, float diffYaw );
+#ifdef SIP_RUN
+    QgsCameraController();
+    QgsCameraController( const QgsCameraController &other );
+#endif
+
     void updateCameraFromPose();
     void moveCameraPositionBy( const QVector3D &posDiff );
+    //! Returns a pointer to the scene's engine's window or nullptr if engine is QgsOffscreen3DEngine
+    QWindow *window() const;
 
   signals:
     //! Emitted when camera has been updated
     void cameraChanged();
-    //! Emitted when viewport rectangle has been updated
-    void viewportChanged();
+
     //! Emitted when the navigation mode is changed using the hotkey ctrl + ~
-    void navigationModeChanged( QgsCameraController::NavigationMode mode );
+    void navigationModeChanged( Qgis::NavigationMode mode );
 
     /**
      * Emitted whenever the camera movement speed is changed by the controller.
@@ -265,7 +261,6 @@ class _3D_EXPORT QgsCameraController : public Qt3DCore::QEntity
     void onMouseReleased( Qt3DInput::QMouseEvent *mouse );
     void onKeyPressed( Qt3DInput::QKeyEvent *event );
     void onKeyReleased( Qt3DInput::QKeyEvent *event );
-    void onPickerMousePressed( Qt3DRender::QPickEvent *pick );
     void applyFlyModeKeyMovements();
 
   private:
@@ -282,15 +277,14 @@ class _3D_EXPORT QgsCameraController : public Qt3DCore::QEntity
      */
     double sampleDepthBuffer( const QImage &buffer, int px, int py );
 
+    bool screenPointToWorldPos( QPoint position, Qt3DRender::QCamera *cameraBefore, double &depth, QVector3D &worldPosition );
+
   private:
+    //! The 3d scene the controller uses
+    Qgs3DMapScene *mScene = nullptr;
+
     //! Camera that is being controlled
     Qt3DRender::QCamera *mCamera = nullptr;
-    //! used for computation of translation when dragging mouse
-    QRect mViewport;
-    //! height of terrain when mouse button was last pressed - for camera control
-    float mLastPressedHeight = 0;
-
-    QPointer<QgsTerrainEntity> mTerrainEntity;
 
     //! Keeps definition of the camera's position and towards where it is looking
     QgsCameraPose mCameraPose;
@@ -322,14 +316,10 @@ class _3D_EXPORT QgsCameraController : public Qt3DCore::QEntity
     bool mZoomPointCalculated = false;
     QVector3D mZoomPoint;
 
-    //! Delegates mouse events to the attached MouseHandler objects
-    Qt3DInput::QMouseDevice *mMouseDevice = nullptr;
-    Qt3DInput::QKeyboardDevice *mKeyboardDevice = nullptr;
-
     Qt3DInput::QMouseHandler *mMouseHandler = nullptr;
     Qt3DInput::QKeyboardHandler *mKeyboardHandler = nullptr;
-    NavigationMode mCameraNavigationMode = NavigationMode::TerrainBasedNavigation;
-    VerticalAxisInversion mVerticalAxisInversion = WhenDragging;
+    Qgis::NavigationMode mCameraNavigationMode = Qgis::NavigationMode::TerrainBased;
+    Qgis::VerticalAxisInversion mVerticalAxisInversion = Qgis::VerticalAxisInversion::WhenDragging;
     double mCameraMovementSpeed = 5.0;
 
     QSet< int > mDepressedKeys;
