@@ -34,13 +34,15 @@ typedef QList<int> QgsAttributeList SIP_SKIP;
 #include "qgseditorwidgetsetup.h"
 #include "qgsfieldconstraints.h"
 #include "qgsdefaultvalue.h"
+#include "qgis.h"
 
 class QgsFieldPrivate;
 
 /**
  * \class QgsField
   * \ingroup core
-  * Encapsulate a field in an attribute table or data source.
+  * \brief Encapsulate a field in an attribute table or data source.
+  *
   * QgsField stores metadata about an attribute field, including name, type
   * length, and if applicable, precision.
   * \note QgsField objects are implicitly shared.
@@ -60,9 +62,33 @@ class CORE_EXPORT QgsField
     Q_PROPERTY( QString alias READ alias WRITE setAlias )
     Q_PROPERTY( QgsDefaultValue defaultValueDefinition READ defaultValueDefinition WRITE setDefaultValueDefinition )
     Q_PROPERTY( QgsFieldConstraints constraints READ constraints WRITE setConstraints )
+    Q_PROPERTY( ConfigurationFlags configurationFlags READ configurationFlags WRITE setConfigurationFlags )
+    Q_PROPERTY( bool isReadOnly READ isReadOnly WRITE setReadOnly )
+
 
   public:
 
+#ifndef SIP_RUN
+
+    /**
+       * Configuration flags for fields
+       * These flags are meant to be user-configurable
+       * and are not describing any information from the data provider.
+       * \note Flags are expressed in the negative forms so that default flags is None.
+       * \since QGIS 3.16
+       */
+    enum class ConfigurationFlag : int
+    {
+      None = 0, //!< No flag is defined
+      NotSearchable = 1 << 1, //!< Defines if the field is searchable (used in the locator search for instance)
+      HideFromWms = 1 << 2, //!< Field is not available if layer is served as WMS from QGIS server
+      HideFromWfs = 1 << 3, //!< Field is not available if layer is served as WFS from QGIS server
+    };
+    Q_ENUM( ConfigurationFlag )
+    Q_DECLARE_FLAGS( ConfigurationFlags, ConfigurationFlag )
+    Q_FLAG( ConfigurationFlags )
+#endif
+
     /**
      * Constructor. Constructs a new QgsField object.
      * \param name Field name
@@ -130,17 +156,30 @@ class CORE_EXPORT QgsField
      */
     QString displayNameWithAlias() const;
 
-
     /**
      * Returns the type to use when displaying this field, including the length and precision of the datatype if applicable.
      *
      * This will be used when the full datatype with details has to displayed to the user.
      *
      * \see type()
+     * \see friendlyTypeString()
+     *
      * \since QGIS 3.14
      */
     QString displayType( bool showConstraints = false ) const;
 
+    /**
+     * Returns a user friendly, translated representation of the field type.
+     *
+     * Unlike displayType(), this method only returns strings representing the field type and does not include
+     * length, precision or constraint information.
+     *
+     * \see type()
+     * \see displayType()
+     * \since QGIS 3.14
+     */
+    QString friendlyTypeString() const;
+
     //! Gets variant type of the field as it will be retrieved from data source
     QVariant::Type type() const;
 
@@ -282,18 +321,48 @@ class CORE_EXPORT QgsField
      */
     void setAlias( const QString &alias );
 
+    /**
+     * Returns the Flags for the field (searchable, …)
+     * \since QGIS 3.16
+     */
+    QgsField::ConfigurationFlags configurationFlags() const SIP_SKIP;
+
+    /**
+     * Sets the Flags for the field (searchable, …)
+     * \since QGIS 3.16
+     */
+    void setConfigurationFlags( QgsField::ConfigurationFlags configurationFlags ) SIP_SKIP;
+
     //! Formats string for display
     QString displayString( const QVariant &v ) const;
 
+    /**
+     * Returns the readable and translated value of the configuration flag
+     * \since QGIS 3.16
+     */
+    static QString readableConfigurationFlag( QgsField::ConfigurationFlag flag ) SIP_SKIP;
+
+#ifndef SIP_RUN
+
+    /**
+     * Converts the provided variant to a compatible format
+     *
+     * \param v  The value to convert
+     * \param errorMessage if specified, will be set to a descriptive error when a conversion failure occurs
+     *
+     * \returns   TRUE if the conversion was successful
+     */
+    bool convertCompatible( QVariant &v, QString *errorMessage = nullptr ) const;
+#else
+
     /**
      * Converts the provided variant to a compatible format
      *
      * \param v  The value to convert
      *
-     * \returns   TRUE if the conversion was successful
+     * \throws ValueError if the value could not be converted to a compatible format
      */
     bool convertCompatible( QVariant &v ) const;
-#ifdef SIP_RUN
     % MethodCode
     PyObject *sipParseErr = NULL;
 
@@ -305,11 +374,12 @@ class CORE_EXPORT QgsField
       if ( sipParseArgs( &sipParseErr, sipArgs, "BJ1", &sipSelf, sipType_QgsField, &sipCpp, sipType_QVariant, &a0, &a0State ) )
       {
         bool sipRes;
+        QString errorMessage;
 
         Py_BEGIN_ALLOW_THREADS
         try
         {
-          sipRes = sipCpp->convertCompatible( *a0 );
+          sipRes = sipCpp->convertCompatible( *a0, &errorMessage );
         }
         catch ( ... )
         {
@@ -322,24 +392,28 @@ class CORE_EXPORT QgsField
 
         Py_END_ALLOW_THREADS
 
-        PyObject *res = sipConvertFromType( a0, sipType_QVariant, NULL );
-        sipReleaseType( a0, sipType_QVariant, a0State );
-
         if ( !sipRes )
         {
           PyErr_SetString( PyExc_ValueError,
-                           QString( "Value %1 (%2) could not be converted to field type %3." ).arg( a0->toString(), a0->typeName() ).arg( sipCpp->type() ).toUtf8().constData() );
-          sipError = sipErrorFail;
+                           QString( "Value could not be converted to field type %1: %2" ).arg( QMetaType::typeName( sipCpp->type() ), errorMessage ).toUtf8().constData() );
+          sipIsErr = 1;
         }
+        else
+        {
+          PyObject *res = sipConvertFromType( a0, sipType_QVariant, NULL );
+          sipReleaseType( a0, sipType_QVariant, a0State );
+          return res;
+        }
+      }
+      else
+      {
+        // Raise an exception if the arguments couldn't be parsed.
+        sipNoMethod( sipParseErr, sipName_QgsField, sipName_convertCompatible, doc_QgsField_convertCompatible );
 
-        return res;
+        return nullptr;
       }
     }
 
-    // Raise an exception if the arguments couldn't be parsed.
-    sipNoMethod( sipParseErr, sipName_QgsField, sipName_convertCompatible, doc_QgsField_convertCompatible );
-
-    return nullptr;
     % End
 #endif
 
@@ -366,6 +440,40 @@ class CORE_EXPORT QgsField
      */
     QgsEditorWidgetSetup editorWidgetSetup() const;
 
+    /**
+     * Make field read-only if \a readOnly is set to true. This is the case for
+     * providers which support generated fields for instance.
+     * \since QGIS 3.18
+     */
+    void setReadOnly( bool readOnly );
+
+    /**
+     * Returns TRUE if this field is a read-only field. This is the case for
+     * providers which support generated fields for instance.
+     * \since QGIS 3.18
+     */
+    bool isReadOnly() const;
+
+    /**
+     * Returns the field's split policy, which indicates how field values should
+     * be handled during a split operation.
+     *
+     * \see setSplitPolicy()
+     *
+     * \since QGIS 3.30
+     */
+    Qgis::FieldDomainSplitPolicy splitPolicy() const;
+
+    /**
+     * Sets the field's split \a policy, which indicates how field values should
+     * be handled during a split operation.
+     *
+     * \see splitPolicy()
+     *
+     * \since QGIS 3.30
+     */
+    void setSplitPolicy( Qgis::FieldDomainSplitPolicy policy );
+
 #ifdef SIP_RUN
     SIP_PYOBJECT __repr__();
     % MethodCode
@@ -374,6 +482,10 @@ class CORE_EXPORT QgsField
     % End
 #endif
 
+#ifndef SIP_RUN
+    static constexpr int MAX_WKT_LENGTH = 999;
+#endif
+
   private:
 
     QSharedDataPointer<QgsFieldPrivate> d;
@@ -383,6 +495,8 @@ class CORE_EXPORT QgsField
 
 Q_DECLARE_METATYPE( QgsField )
 
+Q_DECLARE_OPERATORS_FOR_FLAGS( QgsField::ConfigurationFlags ) SIP_SKIP
+
 //! Writes the field to stream out. QGIS version compatibility is not guaranteed.
 CORE_EXPORT QDataStream &operator<<( QDataStream &out, const QgsField &field );
 //! Reads a field from stream in into field. QGIS version compatibility is not guaranteed.
