@@ -21,16 +21,20 @@
 #include "qgis_core.h"
 #include "qgsmapsettings.h"
 
-#include "qgspallabeling.h"
-#include "qgslabelingenginesettings.h"
 #include "qgslabeling.h"
+#include "qgsfeedback.h"
+#include "qgslabelobstaclesettings.h"
 
 class QgsLabelingEngine;
 class QgsLabelingResults;
+class QgsLabelFeature;
+class QgsLabelingEngineSettings;
 
 namespace pal
 {
   class Problem;
+  class Pal;
+  class LabelPosition;
 }
 
 /**
@@ -60,7 +64,6 @@ class CORE_EXPORT QgsAbstractLabelProvider
     enum Flag
     {
       DrawLabels              = 1 << 1,  //!< Whether the labels should be rendered
-      DrawAllLabels           = 1 << 2,  //!< Whether all features will be labelled even though overlaps occur
       MergeConnectedLines     = 1 << 3,  //!< Whether adjacent lines (with the same label text) should be merged
       CentroidMustBeInside    = 1 << 4,  //!< Whether location of centroid must be inside of polygons
     };
@@ -83,7 +86,7 @@ class CORE_EXPORT QgsAbstractLabelProvider
      *
      * The default behavior is to draw nothing for these labels.
      *
-     * \note This method is only used if the QgsLabelingEngineSettings::DrawUnplacedLabels flag
+     * \note This method is only used if the Qgis::Qgis::LabelingFlag::DrawUnplacedLabels flag
      * is set on the labeling engine.
      *
      * \since QGIS 3.10
@@ -127,7 +130,11 @@ class CORE_EXPORT QgsAbstractLabelProvider
     //! Returns ID of associated layer, or empty string if no layer is associated with the provider.
     QString layerId() const { return mLayerId; }
 
-    //! Returns the associated layer, or NULLPTR if no layer is associated with the provider.
+    /**
+     * Returns the associated layer, or NULLPTR if no layer is associated with the provider.
+     *
+     * \warning Accessing the layer is not thread safe, and this should never be called while the labeling engine is running from a background thread!
+     */
     QgsMapLayer *layer() const { return mLayer.data(); }
 
     /**
@@ -141,7 +148,7 @@ class CORE_EXPORT QgsAbstractLabelProvider
     Flags flags() const { return mFlags; }
 
     //! What placement strategy to use for the labels
-    QgsPalLayerSettings::Placement placement() const { return mPlacement; }
+    Qgis::LabelPlacement placement() const { return mPlacement; }
 
     //! Default priority of labels (may be overridden by individual labels)
     double priority() const { return mPriority; }
@@ -150,7 +157,21 @@ class CORE_EXPORT QgsAbstractLabelProvider
     QgsLabelObstacleSettings::ObstacleType obstacleType() const { return mObstacleType; }
 
     //! How to handle labels that would be upside down
-    QgsPalLayerSettings::UpsideDownLabels upsidedownLabels() const { return mUpsidedownLabels; }
+    Qgis::UpsideDownLabelHandling upsidedownLabels() const { return mUpsidedownLabels; }
+
+    /**
+     * Returns the expression context scope created from the layer associated with this provider.
+     *
+     * \since QGIS 3.22
+     */
+    QgsExpressionContextScope *layerExpressionContextScope() const;
+
+    /**
+     * Returns the symbology reference scale of the layer associated with this provider.
+     *
+     * \since QGIS 3.22
+     */
+    double layerReferenceScale() const { return mLayerReferenceScale; }
 
   protected:
     //! Associated labeling engine
@@ -165,20 +186,127 @@ class CORE_EXPORT QgsAbstractLabelProvider
     //! Associated provider ID (one layer may have multiple providers, e.g. in rule-based labeling)
     QString mProviderId;
     //! Flags altering drawing and registration of features
-    Flags mFlags;
+    Flags mFlags = DrawLabels;
     //! Placement strategy
-    QgsPalLayerSettings::Placement mPlacement;
+    Qgis::LabelPlacement mPlacement = Qgis::LabelPlacement::AroundPoint;
     //! Default priority of labels
-    double mPriority;
+    double mPriority = 0.5;
     //! Type of the obstacle of feature geometries
     QgsLabelObstacleSettings::ObstacleType mObstacleType = QgsLabelObstacleSettings::PolygonBoundary;
     //! How to handle labels that would be upside down
-    QgsPalLayerSettings::UpsideDownLabels mUpsidedownLabels;
+    Qgis::UpsideDownLabelHandling mUpsidedownLabels = Qgis::UpsideDownLabelHandling::FlipUpsideDownLabels;
+
+  private:
+
+    std::unique_ptr< QgsExpressionContextScope > mLayerExpressionContextScope;
+    double mLayerReferenceScale = -1;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS( QgsAbstractLabelProvider::Flags )
 
 
+/**
+ * \ingroup core
+ * \brief QgsFeedback subclass for granular reporting of labeling engine progress.
+ * \note not available in Python bindings
+ * \since QGIS 3.24
+ */
+class CORE_EXPORT QgsLabelingEngineFeedback : public QgsFeedback
+{
+    Q_OBJECT
+
+  public:
+
+    /**
+     * Constructor for QgsLabelingEngineFeedback, with the specified \a parent object.
+     */
+    QgsLabelingEngineFeedback( QObject *parent SIP_TRANSFERTHIS = nullptr )
+      : QgsFeedback( parent )
+    {}
+
+  signals:
+
+    /**
+     * Emitted when the label registration is about to begin.
+     */
+    void labelRegistrationAboutToBegin();
+
+    /**
+     * Emitted when the label registration has completed for all providers.
+     */
+    void labelRegistrationFinished();
+
+    /**
+     * Emitted when the label registration is about to begin for a \a provider.
+     */
+    void providerRegistrationAboutToBegin( QgsAbstractLabelProvider *provider );
+
+    /**
+     * Emitted when the label registration has completed for a \a provider.
+     */
+    void providerRegistrationFinished( QgsAbstractLabelProvider *provider );
+
+    /**
+     * Emitted when the label candidate creation is about to begin for a \a provider.
+     */
+    void candidateCreationAboutToBegin( QgsAbstractLabelProvider *provider );
+
+    /**
+     * Emitted when the label candidate creation has completed for a \a provider.
+     */
+    void candidateCreationFinished( QgsAbstractLabelProvider *provider );
+
+    /**
+     * Emitted when the obstacle costing is about to begin.
+     */
+    void obstacleCostingAboutToBegin();
+
+    /**
+     * Emitted when the obstacle costing has completed.
+     */
+    void obstacleCostingFinished();
+
+    /**
+     * Emitted when the conflict handling step is about to begin.
+     */
+    void calculatingConflictsAboutToBegin();
+
+    /**
+     * Emitted when the conflict handling step has completed.
+     */
+    void calculatingConflictsFinished();
+
+    /**
+     * Emitted when the label candidates are about to be finalized.
+     */
+    void finalizingCandidatesAboutToBegin();
+
+    /**
+     * Emitted when the label candidates are finalized.
+     */
+    void finalizingCandidatesFinished();
+
+    /**
+     * Emitted when the candidate reduction step is about to begin.
+     */
+    void reductionAboutToBegin();
+
+    /**
+     * Emitted when the candidate reduction step is finished.
+     */
+    void reductionFinished();
+
+    /**
+     * Emitted when the problem solving step is about to begin.
+     */
+    void solvingPlacementAboutToBegin();
+
+    /**
+     * Emitted when the problem solving step is finished.
+     */
+    void solvingPlacementFinished();
+};
+
 /**
  * \ingroup core
  * \brief The QgsLabelingEngine class provides map labeling functionality.
@@ -275,7 +403,6 @@ class CORE_EXPORT QgsLabelingEngine
      * Runs the label registration step.
      *
      * Must be called by subclasses prior to solve() and drawLabels()
-     *
      * \since QGIS 3.10
      */
     void registerLabels( QgsRenderContext &context );
@@ -323,6 +450,10 @@ class CORE_EXPORT QgsLabelingEngine
     QList<pal::LabelPosition *> mUnlabeled;
     QList<pal::LabelPosition *> mLabels;
 
+  private:
+
+    QStringList mLayerRenderingOrderIds;
+
 };
 
 /**
@@ -405,7 +536,7 @@ class CORE_EXPORT QgsLabelingUtils
      * \returns list encoded to string
      * \see decodePredefinedPositionOrder()
      */
-    static QString encodePredefinedPositionOrder( const QVector< QgsPalLayerSettings::PredefinedPointPosition > &positions );
+    static QString encodePredefinedPositionOrder( const QVector< Qgis::LabelPredefinedPointPosition > &positions );
 
     /**
      * Decodes a string to an ordered list of predefined point label positions.
@@ -413,7 +544,7 @@ class CORE_EXPORT QgsLabelingUtils
      * \returns decoded list
      * \see encodePredefinedPositionOrder()
      */
-    static QVector< QgsPalLayerSettings::PredefinedPointPosition > decodePredefinedPositionOrder( const QString &positionString );
+    static QVector< Qgis::LabelPredefinedPointPosition > decodePredefinedPositionOrder( const QString &positionString );
 
     /**
      * Encodes line placement \a flags to a string.
