@@ -35,6 +35,9 @@ class QgsLegendSettings;
 class QgsMapSettings;
 class QgsSymbol;
 class QgsRenderContext;
+class QgsTextFormat;
+class QgsTextDocument;
+class QgsTextDocumentMetrics;
 
 /**
  * \ingroup core
@@ -232,6 +235,21 @@ class CORE_EXPORT QgsLayerTreeModelLegendNode : public QObject
        * \since QGIS 3.14
        */
       QSizeF patchSize;
+
+      /**
+       * Optional text document
+       *
+       * \since QGIS 3.30
+       */
+      const QgsTextDocument *textDocument = nullptr;
+
+      /**
+       * Optional text document metrics.
+       *
+       * \since QGIS 3.30
+       */
+      const QgsTextDocumentMetrics *textDocumentMetrics = nullptr;
+
     };
 
     struct ItemMetrics
@@ -371,6 +389,7 @@ class CORE_EXPORT QgsSymbolLegendNode : public QgsLayerTreeModelLegendNode
      * \param parent attach a parent QObject to the legend node.
      */
     QgsSymbolLegendNode( QgsLayerTreeLayer *nodeLayer, const QgsLegendSymbolItem &item, QObject *parent SIP_TRANSFERTHIS = nullptr );
+    ~QgsSymbolLegendNode();
 
     Qt::ItemFlags flags() const override;
     QVariant data( int role ) const override;
@@ -730,6 +749,7 @@ class CORE_EXPORT QgsDataDefinedSizeLegendNode : public QgsLayerTreeModelLegendN
  */
 class CORE_EXPORT QgsVectorLabelLegendNode : public QgsLayerTreeModelLegendNode
 {
+    Q_OBJECT
   public:
 
     /**
