@@ -29,10 +29,64 @@ class QgsLayerTreeViewDefaultActions;
 class QgsLayerTreeViewIndicator;
 class QgsLayerTreeViewMenuProvider;
 class QgsMapLayer;
+class QgsMessageBar;
+class QgsLayerTreeFilterProxyModel;
+
+
+#include <QSortFilterProxyModel>
 
 /**
  * \ingroup gui
- * The QgsLayerTreeView class extends QTreeView and provides some additional functionality
+ *
+ * \brief The QgsLayerTreeProxyModel class is a proxy model for QgsLayerTreeModel, supports
+ * private layers and text filtering.
+ *
+ * \since QGIS 3.18
+ */
+class GUI_EXPORT QgsLayerTreeProxyModel : public QSortFilterProxyModel
+{
+    Q_OBJECT
+
+  public:
+
+    /**
+     * Constructs QgsLayerTreeProxyModel with source model \a treeModel and a \a parent
+     */
+    QgsLayerTreeProxyModel( QgsLayerTreeModel *treeModel, QObject *parent );
+
+    /**
+     * Sets filter to \a filterText.
+     */
+    void setFilterText( const QString &filterText = QString() );
+
+    /**
+     * Returns if private layers are shown.
+     */
+    bool showPrivateLayers() const;
+
+    /**
+     * Determines if private layers are shown.
+     */
+    void setShowPrivateLayers( bool showPrivate );
+
+  protected:
+
+    bool filterAcceptsRow( int sourceRow, const QModelIndex &sourceParent ) const override;
+
+  private:
+
+    bool nodeShown( QgsLayerTreeNode *node ) const;
+
+    QgsLayerTreeModel *mLayerTreeModel = nullptr;
+    QString mFilterText;
+    bool mShowPrivateLayers = false;
+
+};
+
+
+/**
+ * \ingroup gui
+ * \brief The QgsLayerTreeView class extends QTreeView and provides some additional functionality
  * when working with a layer tree.
  *
  * The view updates expanded state of layer tree nodes and also listens to changes
@@ -72,6 +126,74 @@ class GUI_EXPORT QgsLayerTreeView : public QTreeView
     //! Gets access to the model casted to QgsLayerTreeModel
     QgsLayerTreeModel *layerTreeModel() const;
 
+    /**
+     * Returns the proxy model used by the view.
+     *
+     * This can be used to set filters controlling which layers are shown in the view.
+     *
+     * \since QGIS 3.18
+     */
+    QgsLayerTreeProxyModel *proxyModel() const;
+
+    /**
+     * Returns layer tree node for given proxy model tree \a index. Returns root node for invalid index.
+     * Returns NULLPTR if index does not refer to a layer tree node (e.g. it is a legend node)
+     *
+     * Unlike QgsLayerTreeModel::index2Node(), calling this method correctly accounts
+     * for mapping the view indexes through the view's proxy model to the source model.
+     *
+     * \since QGIS 3.18
+     */
+    QgsLayerTreeNode *index2node( const QModelIndex &index ) const;
+
+    /**
+     * Returns proxy model index for a given node. If the node does not belong to the layer tree, the result is undefined
+     *
+     * Unlike QgsLayerTreeModel::node2index(), calling this method correctly accounts
+     * for mapping the view indexes through the view's proxy model to the source model.
+     *
+     * \since QGIS 3.18
+     */
+    QModelIndex node2index( QgsLayerTreeNode *node ) const;
+
+
+    /**
+     * Returns source model index for a given node. If the node does not belong to the layer tree, the result is undefined
+     *
+     * \since QGIS 3.18
+     */
+    QModelIndex node2sourceIndex( QgsLayerTreeNode *node ) const;
+
+
+    /**
+     * Returns legend node for given proxy model tree \a index. Returns NULLPTR for invalid index
+     *
+     * Unlike QgsLayerTreeModel::index2legendNode(), calling this method correctly accounts
+     * for mapping the view indexes through the view's proxy model to the source model.
+     *
+     * \since QGIS 3.18
+     */
+    QgsLayerTreeModelLegendNode *index2legendNode( const QModelIndex &index ) const;
+
+    /**
+     * Returns proxy model index for a given legend node. If the legend node does not belong to the layer tree, the result is undefined.
+     * If the legend node is belongs to the tree but it is filtered out, invalid model index is returned.
+     *
+     * Unlike QgsLayerTreeModel::legendNode2index(), calling this method correctly accounts
+     * for mapping the view indexes through the view's proxy model to the source model.
+     *
+     * \since QGIS 3.18
+     */
+    QModelIndex legendNode2index( QgsLayerTreeModelLegendNode *legendNode );
+
+    /**
+     * Returns index for a given legend node. If the legend node does not belong to the layer tree, the result is undefined.
+     * If the legend node is belongs to the tree but it is filtered out, invalid model index is returned.
+     *
+     * \since QGIS 3.18
+     */
+    QModelIndex legendNode2sourceIndex( QgsLayerTreeModelLegendNode *legendNode );
+
     //! Gets access to the default actions that may be used with the tree view
     QgsLayerTreeViewDefaultActions *defaultActions();
 
@@ -87,6 +209,14 @@ class GUI_EXPORT QgsLayerTreeView : public QTreeView
      */
     QgsMapLayer *currentLayer() const;
 
+    /**
+     * Convenience methods which sets the visible state of the specified map \a layer.
+     *
+     * \see QgsLayerTreeNode::setItemVisibilityChecked()
+     * \since QGIS 3.10
+     */
+    void setLayerVisible( QgsMapLayer *layer, bool visible );
+
     /**
      * Sets the currently selected \a layer.
      *
@@ -154,6 +284,13 @@ class GUI_EXPORT QgsLayerTreeView : public QTreeView
      */
     QList<QgsLayerTreeViewIndicator *> indicators( QgsLayerTreeNode *node ) const;
 
+    /**
+     * Returns width of contextual menu mark, at right of layer node items.
+     * \see setLayerMarkWidth
+     * \since QGIS 3.8
+     */
+    int layerMarkWidth() const { return mLayerMarkWidth; }
+
 ///@cond PRIVATE
 
     /**
@@ -166,6 +303,7 @@ class GUI_EXPORT QgsLayerTreeView : public QTreeView
      * \since QGIS 3.2
      */
     static QStringList viewOnlyCustomProperties() SIP_SKIP;
+
 ///@endcond
 
   public slots:
@@ -184,10 +322,38 @@ class GUI_EXPORT QgsLayerTreeView : public QTreeView
      */
     void collapseAllNodes();
 
+    /**
+     * Set width of contextual menu mark, at right of layer node items.
+     * \see layerMarkWidth
+     * \since QGIS 3.8
+     */
+    void setLayerMarkWidth( int width ) { mLayerMarkWidth = width; }
+
+    /**
+     * Set the message bar to display messages from the layer tree
+     * \since QGIS 3.14
+     */
+    void setMessageBar( QgsMessageBar *messageBar );
+
+    /**
+     * Set the show private layers to \a showPrivate
+     * \since QGIS 3.18
+     */
+    void setShowPrivateLayers( bool showPrivate );
+
+    /**
+     * Returns the show private layers status
+     * \since QGIS 3.18
+     */
+    bool showPrivateLayers( );
+
   signals:
     //! Emitted when a current layer is changed
     void currentLayerChanged( QgsMapLayer *layer );
 
+    //! Emitted when datasets are dropped onto the layer tree view
+    void datasetsDropped( QDropEvent *event );
+
   protected:
     void contextMenuEvent( QContextMenuEvent *event ) override;
 
@@ -198,8 +364,12 @@ class GUI_EXPORT QgsLayerTreeView : public QTreeView
     void mouseReleaseEvent( QMouseEvent *event ) override;
     void keyPressEvent( QKeyEvent *event ) override;
 
+    void dragEnterEvent( QDragEnterEvent *event ) override;
+    void dragMoveEvent( QDragMoveEvent *event ) override;
     void dropEvent( QDropEvent *event ) override;
 
+    void resizeEvent( QResizeEvent *event ) override;
+
   protected slots:
 
     void modelRowsInserted( const QModelIndex &index, int start, int end );
@@ -213,6 +383,10 @@ class GUI_EXPORT QgsLayerTreeView : public QTreeView
 
   private slots:
     void onCustomPropertyChanged( QgsLayerTreeNode *node, const QString &key );
+    //! Handles updating the viewport to avoid flicker
+    void onHorizontalScroll( int value );
+
+    void onDataChanged( const QModelIndex &topLeft, const QModelIndex &bottomRight, const QVector<int> &roles );
 
   protected:
     //! helper class with default actions. Lazily initialized.
@@ -226,6 +400,19 @@ class GUI_EXPORT QgsLayerTreeView : public QTreeView
     //! Used by the item delegate for identification of which indicator has been clicked
     QPoint mLastReleaseMousePos;
 
+    //! Width of contextual menu mark for layer nodes
+    int mLayerMarkWidth;
+
+  private:
+    QgsLayerTreeProxyModel *mProxyModel = nullptr;
+
+    QgsMessageBar *mMessageBar = nullptr;
+
+    bool mShowPrivateLayers = false;
+
+    // For model  debugging
+    // void checkModel( );
+
     // friend so it can access viewOptions() method and mLastReleaseMousePos without making them public
     friend class QgsLayerTreeViewItemDelegate;
 };
@@ -233,7 +420,7 @@ class GUI_EXPORT QgsLayerTreeView : public QTreeView
 
 /**
  * \ingroup gui
- * Implementation of this interface can be implemented to allow QgsLayerTreeView
+ * \brief Implementation of this interface can be implemented to allow QgsLayerTreeView
  * instance to provide custom context menus (opened upon right-click).
  *
  * \see QgsLayerTreeView
