@@ -34,7 +34,7 @@ class QgsLayoutItemLegend;
 
 /**
  * \ingroup core
- * Item model implementation based on layer tree model for layout legend.
+ * \brief Item model implementation based on layer tree model for layout legend.
  *
  * Overrides some functionality of QgsLayerTreeModel to better fit the needs of layout legends.
  *
@@ -55,6 +55,22 @@ class CORE_EXPORT QgsLegendModel : public QgsLayerTreeModel
 
     Qt::ItemFlags flags( const QModelIndex &index ) const override;
 
+    /**
+     * Returns filtered list of active legend nodes attached to a particular layer node
+     * (by default it returns also legend node embedded in parent layer node (if any) unless skipNodeEmbeddedInParent is TRUE)
+     * \note Parameter skipNodeEmbeddedInParent added in QGIS 2.18
+     * \note Not available in Python bindings
+     * \see layerOriginalLegendNodes()
+     * \since QGIS 3.10
+     */
+    QList<QgsLayerTreeModelLegendNode *> layerLegendNodes( QgsLayerTreeLayer *nodeLayer, bool skipNodeEmbeddedInParent = false ) const SIP_SKIP;
+
+    /**
+     * Clears any previously cached data for the specified \a node.
+     * \since QGIS 3.14
+     */
+    void clearCachedData( QgsLayerTreeNode *node ) const;
+
   signals:
 
     /**
@@ -73,28 +89,25 @@ class CORE_EXPORT QgsLegendModel : public QgsLayerTreeModel
 
   private:
 
-    /**
-     * Returns filtered list of active legend nodes attached to a particular layer node
-     * (by default it returns also legend node embedded in parent layer node (if any) unless skipNodeEmbeddedInParent is true)
-     * \note Parameter skipNodeEmbeddedInParent added in QGIS 2.18
-     * \see layerOriginalLegendNodes()
-     * \since QGIS 3.10
-     */
-    QList<QgsLayerTreeModelLegendNode *> layerLegendNodes( QgsLayerTreeLayer *nodeLayer, bool skipNodeEmbeddedInParent = false ) const;
-
     /**
      * Pointer to the QgsLayoutItemLegend class that made the model.
      * \since QGIS 3.10
      */
     QgsLayoutItemLegend *mLayoutLegend = nullptr;
 
+    /**
+     * Evaluate the expression or symbol expressions of a given layer node.
+     * \since QGIS 3.14
+     */
+    QString evaluateLayerExpressions( QgsLayerTreeLayer *nodeLayer ) const;
+
 };
 
 
 
 /**
  * \ingroup core
- * A layout item subclass for map legends.
+ * \brief A layout item subclass for map legends.
  * \since QGIS 3.0
  */
 class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
@@ -226,14 +239,18 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
     /**
      * Returns the font settings for a legend \a component.
      * \see setStyleFont()
+     *
+     * \deprecated use QgsLegendStyle::textFormat() from style() instead.
      */
-    QFont styleFont( QgsLegendStyle::Style component ) const;
+    Q_DECL_DEPRECATED QFont styleFont( QgsLegendStyle::Style component ) const SIP_DEPRECATED;
 
     /**
      * Sets the style \a font for a legend \a component.
      * \see styleFont()
+     *
+     * \deprecated use QgsLegendStyle::setTextFormat() from style() instead.
      */
-    void setStyleFont( QgsLegendStyle::Style component, const QFont &font );
+    Q_DECL_DEPRECATED void setStyleFont( QgsLegendStyle::Style component, const QFont &font ) SIP_DEPRECATED;
 
     /**
      * Set the \a margin for a legend \a component.
@@ -248,14 +265,18 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
     /**
      * Returns the spacing in-between lines in layout units.
      * \see setLineSpacing()
+     *
+     * \deprecated use QgsLegendStyle::textFormat() from style() instead.
      */
-    double lineSpacing() const;
+    Q_DECL_DEPRECATED double lineSpacing() const SIP_DEPRECATED;
 
     /**
      * Sets the \a spacing in-between multiple lines.
      * \see lineSpacing()
+     *
+     * \deprecated use QgsLegendStyle::setTextFormat() from style() instead.
      */
-    void setLineSpacing( double spacing );
+    Q_DECL_DEPRECATED void setLineSpacing( double spacing ) SIP_DEPRECATED;
 
     /**
      * Returns the legend box space.
@@ -284,14 +305,18 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
     /**
      * Returns the legend font color.
      * \see setFontColor()
+     *
+     * \deprecated use QgsLegendStyle::setTextFormat() from style() instead.
      */
-    QColor fontColor() const;
+    Q_DECL_DEPRECATED QColor fontColor() const SIP_DEPRECATED;
 
     /**
      * Sets the legend font \a color.
      * \see fontColor()
+     *
+     * \deprecated use QgsLegendStyle::setTextFormat() from style() instead.
      */
-    void setFontColor( const QColor &color );
+    Q_DECL_DEPRECATED void setFontColor( const QColor &color ) SIP_DEPRECATED;
 
     /**
      * Returns the legend symbol width.
@@ -305,6 +330,42 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
      */
     void setSymbolWidth( double width );
 
+    /**
+     * Returns the maximum symbol size (in mm). 0.0 means there is no maximum set.
+     *
+     * \see setMaximumSymbolSize()
+     * \since QGIS 3.16
+     */
+    double maximumSymbolSize() const;
+
+    /**
+     * Set the maximum symbol \a size for symbol (in millimeters).
+     *
+     * A symbol size of 0.0 indicates no maximum is set.
+     *
+     * \see maximumSymbolSize()
+     * \since QGIS 3.16
+     */
+    void setMaximumSymbolSize( double size );
+
+    /**
+     * Returns the minimum symbol size (in mm). A value 0.0 means there is no minimum set.
+     *
+     * \see setMinimumSymbolSize
+     * \since QGIS 3.16
+     */
+    double minimumSymbolSize() const;
+
+    /**
+     * Set the minimum symbol \a size for symbol (in millimeters).
+     *
+     * A symbol size of 0.0 indicates no minimum is set.
+     *
+     * \see minimumSymbolSize()
+     * \since QGIS 3.16
+     */
+    void setMinimumSymbolSize( double size );
+
     /**
      * Sets the \a alignment for placement of legend symbols.
      *
@@ -506,6 +567,7 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
 
     QgsExpressionContext createExpressionContext() const override;
     ExportLayerBehavior exportLayerBehavior() const override;
+    bool accept( QgsStyleEntityVisitorInterface *visitor ) const override;
 
   public slots:
 
@@ -536,6 +598,8 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
 
     void nodeCustomPropertyChanged( QgsLayerTreeNode *node, const QString &key );
 
+    //! Clears any data cached for the legend model
+    void clearLegendCachedData();
 
   private:
     QgsLayoutItemLegend() = delete;
