@@ -19,91 +19,100 @@
 #ifndef QGSMAPOVERVIEWCANVAS_H
 #define QGSMAPOVERVIEWCANVAS_H
 
-
-#include <QMouseEvent>
-#include <QWheelEvent>
 #include <QWidget>
-#include <QStringList>
 #include <QPixmap>
 
+class QMouseEvent;
 class QgsMapCanvas;
 class QgsPanningWidget; // defined in .cpp
 class QgsRectangle;
 
 class QgsMapRendererQImageJob;
 #include "qgsmapsettings.h"
+#include "qgis_gui.h"
 
-/** \ingroup gui
- * A widget that displays an overview map.
+/**
+ * \ingroup gui
+ * \brief A widget that displays an overview map.
  */
 class GUI_EXPORT QgsMapOverviewCanvas : public QWidget
 {
     Q_OBJECT
 
   public:
-    QgsMapOverviewCanvas( QWidget * parent = nullptr, QgsMapCanvas* mapCanvas = nullptr );
-
-    ~QgsMapOverviewCanvas();
+    QgsMapOverviewCanvas( QWidget *parent SIP_TRANSFERTHIS = nullptr, QgsMapCanvas *mapCanvas = nullptr );
 
     //! renders overview and updates panning widget
     void refresh();
 
     //! changes background color
-    void setBackgroundColor( const QColor& color );
+    void setBackgroundColor( const QColor &color );
 
     //! updates layer set for overview
-    void setLayerSet( const QStringList& layerSet );
+    void setLayers( const QList<QgsMapLayer *> &layers );
 
-    QStringList layerSet() const;
+    //! Returns list of layers visible in the overview
+    QList<QgsMapLayer *> layers() const;
 
-    void enableAntiAliasing( bool flag ) { mSettings.setFlag( QgsMapSettings::Antialiasing, flag ); }
+    void enableAntiAliasing( bool flag )
+    {
+      mSettings.setFlag( Qgis::MapSettingsFlag::Antialiasing, flag );
+      mSettings.setFlag( Qgis::MapSettingsFlag::HighQualityImageTransforms, flag );
+    }
 
     void updateFullExtent();
 
-  public slots:
-
-    // ### QGIS 3: make protected
-    //! used for overview canvas to reflect changed extent in main map canvas
-    void drawExtentRect();
-
-    // ### QGIS 3: rename so it does not look like getter, make protected
-    void hasCrsTransformEnabled( bool flag );
-
-    // ### QGIS 3: rename Srs to Crs, make protected
-    void destinationSrsChanged();
-
   protected slots:
     void mapRenderingFinished();
-    void layerRepaintRequested();
+
+    /**
+     * Triggered when a layer in the overview requests a repaint.
+     */
+    void layerRepaintRequested( bool deferred = false );
 
   protected:
 
+    //! used for overview canvas to reflect changed extent in main map canvas
+    void drawExtentRect();
+
+    //! Should be called when the canvas destination CRS is changed
+    void destinationCrsChanged();
+
+    //! Called when the canvas transform context is changed
+    void transformContextChanged();
+
     //! Overridden paint event
-    void paintEvent( QPaintEvent * pe ) override;
+    void paintEvent( QPaintEvent *pe ) override;
+
+    //! Overridden show event
+    void showEvent( QShowEvent *e ) override;
 
     //! Overridden resize event
-    void resizeEvent( QResizeEvent * e ) override;
+    void resizeEvent( QResizeEvent *e ) override;
 
     //! Overridden mouse move event
-    void mouseMoveEvent( QMouseEvent * e ) override;
+    void mouseMoveEvent( QMouseEvent *e ) override;
 
     //! Overridden mouse press event
-    void mousePressEvent( QMouseEvent * e ) override;
+    void mousePressEvent( QMouseEvent *e ) override;
 
     //! Overridden mouse release event
-    void mouseReleaseEvent( QMouseEvent * e ) override;
+    void mouseReleaseEvent( QMouseEvent *e ) override;
+
+    //! Overridden mouse release event
+    void wheelEvent( QWheelEvent *e ) override;
 
     //! called when panning to reflect mouse movement
     void updatePanningWidget( QPoint pos );
 
     //! widget for panning map in overview
-    QgsPanningWidget* mPanningWidget;
+    QgsPanningWidget *mPanningWidget = nullptr;
 
     //! position of cursor inside panning widget
     QPoint mPanningCursorOffset;
 
     //! main map canvas - used to get/set extent
-    QgsMapCanvas* mMapCanvas;
+    QgsMapCanvas *mMapCanvas = nullptr;
 
     //! pixmap where the map is stored
     QPixmap mPixmap;
@@ -112,10 +121,12 @@ class GUI_EXPORT QgsMapOverviewCanvas : public QWidget
     QgsMapSettings mSettings;
 
     //! for rendering overview
-    QgsMapRendererQImageJob* mJob;
+    QgsMapRendererQImageJob *mJob = nullptr;
 };
 
 
+#ifndef SIP_RUN
+
 /// @cond PRIVATE
 // Widget that serves as rectangle showing current extent in overview
 class QgsPanningWidget : public QWidget
@@ -125,13 +136,15 @@ class QgsPanningWidget : public QWidget
     QPolygon mPoly;
 
   public:
-    explicit QgsPanningWidget( QWidget* parent );
+    explicit QgsPanningWidget( QWidget *parent );
 
-    void setPolygon( const QPolygon& p );
+    void setPolygon( const QPolygon &p );
 
-    void paintEvent( QPaintEvent* pe ) override;
+    void paintEvent( QPaintEvent *pe ) override;
 
 };
 ///@endcond
 
 #endif
+
+#endif
