@@ -22,8 +22,6 @@
 #include "qgscompoundcurve.h"
 #include "qgsgeometry.h"
 #include "qobjectuniqueptr.h"
-#include "qgssnappingutils.h"
-#include "qgsgeometryrubberband.h"
 
 #include <QPoint>
 #include <QList>
@@ -35,89 +33,17 @@ class QgsVertexMarker;
 class QgsMapLayer;
 class QgsGeometryValidator;
 class QgsMapToolCaptureRubberBand;
+class QgsCurvePolygon;
+class QgsMapToolShapeAbstract;
+class QgsMapToolShapeMetadata;
 
 
-#ifndef SIP_RUN
-
-///@cond PRIVATE
-
-/**
- * Class that reprensents a rubber band that can be linear or circular.
- *
- * \since QGIS 3.16
- */
-class QgsMapToolCaptureRubberBand: public QgsGeometryRubberBand
-{
-  public:
-    //! Constructor
-    QgsMapToolCaptureRubberBand( QgsMapCanvas *mapCanvas, QgsWkbTypes::GeometryType geomType = QgsWkbTypes::LineGeometry );
-
-    //! Returns the curve defined by the rubber band, the caller has to take the ownership, nullptr if no curve is defined.
-    QgsCurve *curve();
-
-    /**
-     * Returns if the curve defined by the rubber band is complete :
-     * has more than 2 points for circular string and more than 1 point for linear string
-     */
-    bool curveIsComplete() const;
-
-    /**
-     * Resets the rubber band with the specified geometry type
-     * that must be line geometry or polygon geometry.
-     * \a firstPolygonPoint is the first point that will be used to render the polygon rubber band (if \a geomType is PolygonGeometry)
-     */
-    void reset( QgsWkbTypes::GeometryType geomType = QgsWkbTypes::LineGeometry, QgsWkbTypes::Type stringType = QgsWkbTypes::LineString, const QgsPoint &firstPolygonPoint = QgsPoint() );
-
-    //! Sets the geometry type of the rubberband without removing already existing points
-    void setRubberBandGeometryType( QgsWkbTypes::GeometryType geomType );
-
-    //! Adds point to the rubber band
-    void addPoint( const QgsPoint &point, bool doUpdate = true );
-
-    //! Moves the last point to the \a point position
-    void movePoint( const QgsPoint &point );
-
-    //! Moves the point with \a index to the \a point position
-    void movePoint( int index, const QgsPoint &point );
-
-    //! Returns the points count in the rubber band (except the first point if polygon)
-    int pointsCount();
-
-    //! Returns the type of the curve (linear string or circular string)
-    QgsWkbTypes::Type stringType() const;
-
-    //! Sets the type of the curve (linear string or circular string)
-    void setStringType( const QgsWkbTypes::Type &type );
-
-    //! Returns the last point of the rubber band
-    QgsPoint lastPoint() const;
-
-    //! Returns the point of the rubber band at position from end
-    QgsPoint pointFromEnd( int posFromEnd ) const;
-
-    //! Removes the last point of the rrubber band
-    void removeLastPoint();
-
-  private:
-    QgsWkbTypes::Type mStringType = QgsWkbTypes::LineString;
-
-    void setGeometry( QgsAbstractGeometry *geom ) override;
-    void updateCurve();
-
-    QgsCurve *createLinearString();
-    QgsCurve *createCircularString();
-
-    QgsPointSequence mPoints;
-    QgsPoint mFirstPolygonPoint;
-};
-
-/// @endcond
-
-#endif //SIP_RUN
-
 /**
  * \ingroup gui
- * \class QgsMapToolCapture
+ * QgsMapToolCapture is a base class capable of capturing point, lines and polygons.
+ * The tool supports different techniques: straight segments, curves, streaming and shapes
+ * Once the the geometry is captured the virtual private handler geometryCaptured is called
+ * as well as a more specific handler (pointCaptured, lineCaptured or polygonCaptured)
  */
 class GUI_EXPORT QgsMapToolCapture : public QgsMapToolAdvancedDigitizing
 {
@@ -134,23 +60,12 @@ class GUI_EXPORT QgsMapToolCapture : public QgsMapToolAdvancedDigitizing
       CapturePolygon  //!< Capture polygons
     };
 
-    /**
-     * Capture technique.
-     *
-     * \since QGIS 3.20
-     */
-    enum CaptureTechnique
-    {
-      StraightSegments, //!< Default capture mode - capture occurs with straight line segments
-      CircularString, //!< Capture in circular strings
-      Streaming, //!< Streaming points digitizing mode (points are automatically added as the mouse cursor moves). Since QGIS 3.20.
-    };
-
     //! Specific capabilities of the tool
     enum Capability
     {
-      NoCapabilities = 0,       //!< No specific capabilities
-      SupportsCurves = 1,       //!< Supports curved geometries input
+      NoCapabilities = 1 << 0, //!< No specific capabilities
+      SupportsCurves = 1 << 1, //!< Supports curved geometries input
+      ValidateGeometries = 1 << 2, //!< Tool supports geometry validation (since QGIS 3.22)
     };
 
     Q_DECLARE_FLAGS( Capabilities, Capability )
@@ -170,7 +85,20 @@ class GUI_EXPORT QgsMapToolCapture : public QgsMapToolAdvancedDigitizing
      *
      * \since QGIS 3.20
      */
-    virtual bool supportsTechnique( CaptureTechnique technique ) const;
+    virtual bool supportsTechnique( Qgis::CaptureTechnique technique ) const;
+
+    /**
+     * Sets the current capture if it is supported by the map tool
+     * \since QGIS 3.26
+     */
+    void setCurrentCaptureTechnique( Qgis::CaptureTechnique technique );
+
+    /**
+     * Sets the current shape tool
+     * \see QgsMapToolShapeRegistry
+     * \since QGIS 3.26
+     */
+    void setCurrentShapeMapTool( const QgsMapToolShapeMetadata *shapeMapToolMetadata ) SIP_SKIP;
 
     void activate() override;
     void deactivate() override;
@@ -207,6 +135,7 @@ class GUI_EXPORT QgsMapToolCapture : public QgsMapToolAdvancedDigitizing
     QList<QgsPointLocator::Match> snappingMatches() const;
 
     void cadCanvasMoveEvent( QgsMapMouseEvent *e ) override;
+    void cadCanvasReleaseEvent( QgsMapMouseEvent *e ) override;
 
     /**
      * Intercept key events like Esc or Del to delete the last point
@@ -226,69 +155,16 @@ class GUI_EXPORT QgsMapToolCapture : public QgsMapToolAdvancedDigitizing
      * Returns the rubberBand currently owned by this map tool and
      * transfers ownership to the caller.
      *
+     * May be NULLPTR.
+     *
      * \since QGIS 3.8
      */
     QgsRubberBand *takeRubberBand() SIP_FACTORY;
 
-  public slots:
-    //! Enable the digitizing with curve
-    void setCircularDigitizingEnabled( bool enable );
-
-    /**
-     * Toggles the stream digitizing mode.
-     * \since QGIS 3.20
-     */
-    void setStreamDigitizingEnabled( bool enable );
-
-  private slots:
-    void addError( const QgsGeometry::Error &error );
-    void currentLayerChanged( QgsMapLayer *layer );
-    //! Update the extra snap layer, this should be called whenever the capturecurve changes
-    void updateExtraSnapLayer();
-
-  protected:
-
-    // TODO QGIS 4.0 returns an enum instead of a magic constant
-
-    /**
-     * Converts a map point to layer coordinates
-     *  \param mapPoint the point in map coordinates
-     *  \param[in,out] layerPoint the point in layer coordinates
-     *  \returns
-     *   0 in case of success
-     *   1 if the current layer is NULLPTR or not a vector layer
-     *   2 if the transformation failed
-     */
-    int nextPoint( const QgsPoint &mapPoint, QgsPoint &layerPoint );
-
-    // TODO QGIS 4.0 returns an enum instead of a magic constant
-
-    /**
-     * Converts a point to map coordinates and layer coordinates
-     * \param p the input point
-     * \param[in,out] layerPoint the point in layer coordinates
-     * \param[in,out] mapPoint the point in map coordinates
-     * \returns
-     *  0 in case of success
-     *  1 if the current layer is NULLPTR or not a vector layer
-     *  2 if the transformation failed
-     */
-    int nextPoint( QPoint p, QgsPoint &layerPoint, QgsPoint &mapPoint );
-
-    // TODO QGIS 4.0 returns an enum instead of a magic constant
-
-    /**
-     * Fetches the original point from the source layer if it has the same
-     * CRS as the current layer.
-     * \returns 0 in case of success, 1 if not applicable (CRS mismatch), 2 in case of failure
-     * \since QGIS 2.14
-     */
-    int fetchLayerPoint( const QgsPointLocator::Match &match, QgsPoint &layerPoint );
-
     /**
      * Creates a QgsPoint with ZM support if necessary (according to the
      * WkbType of the current layer). If the point is snapped, then the Z
-     * value is took from the snapped point.
+     * value is derived from the snapped point.
      *
      * \param e A mouse event
      *
@@ -312,9 +188,74 @@ class GUI_EXPORT QgsMapToolCapture : public QgsMapToolAdvancedDigitizing
 
     // TODO QGIS 4.0 returns an enum instead of a magic constant
 
+  public slots:
+
+    /**
+     * Enable the digitizing with curve
+     * \deprecated since QGIS 3.26 use setCurrentCaptureTechnique() instead
+     */
+    Q_DECL_DEPRECATED void setCircularDigitizingEnabled( bool enable ) SIP_DEPRECATED;
+
+    /**
+     * Toggles the stream digitizing mode.
+     * \since QGIS 3.20
+    * \deprecated since QGIS 3.26 use setCurrentCaptureTechnique() instead
+     */
+    Q_DECL_DEPRECATED void setStreamDigitizingEnabled( bool enable ) SIP_DEPRECATED;
+
+  private slots:
+    void addError( const QgsGeometry::Error &error );
+    void currentLayerChanged( QgsMapLayer *layer );
+    //! Update the extra snap layer, this should be called whenever the capturecurve changes
+    void updateExtraSnapLayer();
+
+  protected:
+
+    // TODO QGIS 4.0 returns an enum instead of a magic constant
+
+    /**
+     * Converts a map point to layer coordinates
+     *  \param mapPoint the point in map coordinates
+     *  \param[in,out] layerPoint the point in layer coordinates
+     *  \returns
+     *   0 in case of success
+     *   1 if the current layer is NULLPTR
+     *   2 if the transformation failed
+     */
+    int nextPoint( const QgsPoint &mapPoint, QgsPoint &layerPoint );
+
+    // TODO QGIS 4.0 returns an enum instead of a magic constant
+
+    /**
+     * Converts a point to map coordinates and layer coordinates
+     * \param p the input point
+     * \param[in,out] layerPoint the point in layer coordinates
+     * \param[in,out] mapPoint the point in map coordinates
+     * \returns
+     *  0 in case of success
+     *  1 if the current layer is NULLPTR or not a vector layer
+     *  2 if the transformation failed
+     */
+    int nextPoint( QPoint p, QgsPoint &layerPoint, QgsPoint &mapPoint );
+
+    // TODO QGIS 4.0 returns an enum instead of a magic constant
+
+    /**
+     * Fetches the original point from the source layer if it has the same
+     * CRS as the current layer.
+     * If topological editing is activated, the points are projected to the
+     * current layer CRS.
+     * \returns
+     *  0 in case of success
+     *  1 if not applicable (CRS mismatch / invalid layer)
+     *  2 in case of failure
+     * \since QGIS 2.14
+     */
+    int fetchLayerPoint( const QgsPointLocator::Match &match, QgsPoint &layerPoint );
+
     /**
      * Adds a point to the rubber band (in map coordinates) and to the capture list (in layer coordinates)
-     * \returns 0 in case of success, 1 if current layer is not a vector layer, 2 if coordinate transformation failed
+     * \returns 0 in case of success, 2 if coordinate transformation failed
      */
     int addVertex( const QgsPointXY &point );
 
@@ -398,6 +339,35 @@ class GUI_EXPORT QgsMapToolCapture : public QgsMapToolAdvancedDigitizing
     void stopCapturing();
 
   private:
+
+    /**
+     * Called when the geometry is captured
+     * A more specific handler is also called afterwards (pointCaptured, lineCaptured or polygonCaptured)
+     * \since QGIS 3.26
+     */
+    virtual void geometryCaptured( const QgsGeometry &geometry ) {Q_UNUSED( geometry )} SIP_FORCE
+
+    /**
+     * Called when a point is captured
+     * geometryCaptured is called just before
+     * \since QGIS 3.26
+     */
+    virtual void pointCaptured( const QgsPoint &point ) {Q_UNUSED( point )} SIP_FORCE
+
+    /**
+     * Called when a line is captured
+     * geometryCaptured is called just before
+     * \since QGIS 3.26
+     */
+    virtual void lineCaptured( const QgsCurve *line ) {Q_UNUSED( line )} SIP_FORCE
+
+    /**
+     * Called when a polygon is captured
+     * geometryCaptured is called just before
+     * \since QGIS 3.26
+     */
+    virtual void polygonCaptured( const QgsCurvePolygon *polygon ) {Q_UNUSED( polygon )} SIP_FORCE
+
     //! whether tracing has been requested by the user
     bool tracingEnabled();
     //! first point that will be used as a start of the trace
@@ -410,22 +380,16 @@ class GUI_EXPORT QgsMapToolCapture : public QgsMapToolAdvancedDigitizing
     //! create a curve rubber band
     QgsMapToolCaptureRubberBand *createCurveRubberBand() const;
 
-    //! Returns extemity point of the captured curve in map coordinates
-    QgsPoint firstCapturedMapPoint();
-    QgsPoint lastCapturedMapPoint();
-
     //! Reset the
     void resetRubberBand();
 
-  private:
     //! The capture mode in which this tool operates
     CaptureMode mCaptureMode;
 
-
-
     //! Flag to indicate a map canvas capture operation is taking place
     bool mCapturing = false;
 
+    //! extremity point of the captured curve in map coordinates
     QgsPoint mCaptureFirstPoint;
     QgsPoint mCaptureLastPoint;
 
@@ -433,7 +397,7 @@ class GUI_EXPORT QgsMapToolCapture : public QgsMapToolAdvancedDigitizing
     QObjectUniquePtr<QgsRubberBand> mRubberBand;
 
     //! Temporary rubber band for polylines and polygons. this connects the last added point to the mouse cursor position
-    std::unique_ptr<QgsMapToolCaptureRubberBand> mTempRubberBand;
+    QObjectParentUniquePtr<QgsMapToolCaptureRubberBand> mTempRubberBand;
 
     //! List to store the points of digitized lines and polygons (in layer coordinates)
     QgsCompoundCurve mCaptureCurve;
@@ -465,9 +429,12 @@ class GUI_EXPORT QgsMapToolCapture : public QgsMapToolAdvancedDigitizing
     QgsPointXY mTracingStartPoint;
 
     //! Used to store the state of digitizing type (linear or circular)
-    QgsWkbTypes::Type mDigitizingType = QgsWkbTypes::LineString;
+    Qgis::WkbType mLineDigitizingType = Qgis::WkbType::LineString;
+
+    Qgis::CaptureTechnique mCurrentCaptureTechnique = Qgis::CaptureTechnique::StraightSegments;
+
+    QObjectUniquePtr< QgsMapToolShapeAbstract > mCurrentShapeMapTool;
 
-    bool mStreamingEnabled = false;
     bool mAllowAddingStreamingPoints = false;
     int mStreamingToleranceInPixels = 1;
 
@@ -475,6 +442,9 @@ class GUI_EXPORT QgsMapToolCapture : public QgsMapToolAdvancedDigitizing
 
     bool mIgnoreSubsequentAutoRepeatUndo = false;
 
+    friend class TestQgsMapToolCapture;
+
+
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS( QgsMapToolCapture::Capabilities )
