@@ -21,7 +21,7 @@
 #include "qgis_sip.h"
 #include <QTransform>
 #include <vector>
-#include "qgsunittypes.h"
+#include "qgis.h"
 #include "qgspointxy.h"
 
 #include <cassert>
@@ -39,6 +39,13 @@ class CORE_EXPORT QgsMapToPixel
 {
   public:
 
+    /**
+     * Constructor for an invalid QgsMapToPixel.
+     *
+     * A manual call to setParameters() is required to initialize the object.
+     */
+    QgsMapToPixel();
+
     /**
      * Constructor
      * \param mapUnitsPerPixel Map units per pixel
@@ -65,17 +72,18 @@ class CORE_EXPORT QgsMapToPixel
      * \returns matching QgsMapToPixel
      * \since QGIS 3.0
      */
-    static QgsMapToPixel fromScale( double scale, QgsUnitTypes::DistanceUnit mapUnits, double dpi = 96 );
+    static QgsMapToPixel fromScale( double scale, Qgis::DistanceUnit mapUnits, double dpi = 96 );
 
     /**
-     * Constructor
+     * Returns TRUE if the object is valid (i.e. it has parameters set), or FALSE if the object is default constructed
+     * with no parameters set.
      *
-     * Use setParameters to fill
+     * \since QGIS 3.22
      */
-    QgsMapToPixel();
+    bool isValid() const { return mValid; }
 
     /**
-     * Transform the point \a p from map (world) coordinates to device coordinates.
+     * Transforms a point \a p from map (world) coordinates to device coordinates.
      * \param p Point to transform
      * \returns QgsPointXY in device coordinates
      */
@@ -88,7 +96,7 @@ class CORE_EXPORT QgsMapToPixel
     }
 
     /**
-     * Transform the point \a p from map (world) coordinates to device coordinates in place.
+     * Transforms a point \a p from map (world) coordinates to device coordinates in place.
      */
     void transform( QgsPointXY *p ) const
     {
@@ -99,9 +107,9 @@ class CORE_EXPORT QgsMapToPixel
     }
 
     /**
-     * Transform the point specified by x,y from map (world)
-     * coordinates to device coordinates
-     * \param x x coordinate o point to transform
+     * Transforms the point specified by x,y from map (world) coordinates to device coordinates.
+     *
+     * \param x x coordinate of point to transform
      * \param y y coordinate of point to transform
      * \returns QgsPointXY in device coordinates
      */
@@ -112,8 +120,9 @@ class CORE_EXPORT QgsMapToPixel
     }
 
     /**
-     * Transforms device coordinates to map coordinates. Modifies the
-     * given coordinates in place. Intended as a fast way to do the
+     * Transforms device coordinates to map coordinates.
+     *
+     * This method modifies the given coordinates in place. It is intended as a fast way to do the
      * transform.
      */
     void transformInPlace( double &x, double &y ) const
@@ -125,9 +134,11 @@ class CORE_EXPORT QgsMapToPixel
     }
 
     /**
-     * Transforms device coordinates to map coordinates. Modifies the
-     * given coordinates in place. Intended as a fast way to do the
+     * Transforms device coordinates to map coordinates.
+     *
+     * This method modifies the given coordinates in place. It is intended as a fast way to do the
      * transform.
+     *
      * \note Not available in Python bindings
      */
     void transformInPlace( float &x, float &y ) const SIP_SKIP
@@ -141,8 +152,9 @@ class CORE_EXPORT QgsMapToPixel
 #ifndef SIP_RUN
 
     /**
-     * Transform device coordinates to map coordinates. Modifies the
-     * given coordinates in place. Intended as a fast way to do the
+     * Transforms device coordinates to map coordinates.
+     *
+     * This method modifies the given coordinates in place. It is intended as a fast way to do the
      * transform.
      * \note not available in Python bindings
      */
@@ -155,17 +167,21 @@ class CORE_EXPORT QgsMapToPixel
     }
 #endif
 
-    //! Transform device coordinates to map (world) coordinates
+    /**
+     * Transforms device coordinates to map (world) coordinates.
+     */
     QgsPointXY toMapCoordinates( int x, int y ) const
     {
       return toMapCoordinates( static_cast<double>( x ), static_cast<double>( y ) );
     }
 
-    //! Transform device coordinates to map (world) coordinates
+    /**
+     * Transforms device coordinates to map (world) coordinates.
+     */
     QgsPointXY toMapCoordinates( double x, double y ) const SIP_PYNAME( toMapCoordinatesF )
     {
       bool invertible;
-      QTransform matrix = mMatrix.inverted( &invertible );
+      const QTransform matrix = mMatrix.inverted( &invertible );
       assert( invertible );
       qreal mx, my;
       matrix.map( static_cast< qreal >( x ), static_cast< qreal >( y ), &mx, &my );
@@ -173,18 +189,20 @@ class CORE_EXPORT QgsMapToPixel
     }
 
     /**
-     * Transform device coordinates to map (world) coordinates
+     * Transforms device coordinates to map (world) coordinates.
+     *
      * \param p Point to be converted to map cooordinates
      * \returns QgsPointXY in map coorndiates
      */
     QgsPointXY toMapCoordinates( QPoint p ) const
     {
-      QgsPointXY mapPt = toMapCoordinates( static_cast<double>( p.x() ), static_cast<double>( p.y() ) );
+      const QgsPointXY mapPt = toMapCoordinates( static_cast<double>( p.x() ), static_cast<double>( p.y() ) );
       return QgsPointXY( mapPt );
     }
 
     /**
-     * Transform device coordinates to map (world) coordinates
+     * Transforms device coordinates to map (world) coordinates.
+     *
      * \deprecated since QGIS 3.4 use toMapCoordinates instead
      */
     Q_DECL_DEPRECATED QgsPointXY toMapPoint( double x, double y ) const SIP_DEPRECATED
@@ -193,55 +211,100 @@ class CORE_EXPORT QgsMapToPixel
     }
 
     /**
-     * Set map units per pixel
+     * Sets the map units per pixel.
+     *
+     * Calling this method will automatically set the object as valid.
+     *
      * \param mapUnitsPerPixel Map units per pixel
+     *
+     * \see mapUnitsPerPixel()
      */
     void setMapUnitsPerPixel( double mapUnitsPerPixel );
 
-    //! Returns current map units per pixel
+    /**
+     * Returns the current map units per pixel.
+     *
+     * \see setMapUnitsPerPixel()
+     */
     double mapUnitsPerPixel() const { return mMapUnitsPerPixel; }
 
     /**
-     * Returns current map width in pixels
-     * The information is only known if setRotation was used
+     * Returns the current map width in pixels.
+     *
+     * The information is only known if setRotation was used.
+     *
+     * \see mapHeight()
      * \since QGIS 2.8
      */
     int mapWidth() const { return mWidth; }
 
     /**
      * Returns current map height in pixels
+     *
+     * \see mapWidth()
      * \since QGIS 2.8
      */
     int mapHeight() const { return mHeight; }
 
     /**
-     * Set map rotation in degrees (clockwise)
+     * Sets map rotation in \a degrees (clockwise).
+     *
+     * Calling this method will automatically set the object as valid.
+     *
      * \param degrees clockwise rotation in degrees
      * \param cx X ordinate of map center in geographical units
      * \param cy Y ordinate of map center in geographical units
+     *
+     * \see mapRotation()
      * \since QGIS 2.8
      */
     void setMapRotation( double degrees, double cx, double cy );
 
     /**
-     * Returns current map rotation in degrees (clockwise)
+     * Returns the current map rotation in degrees (clockwise).
+     *
+     * \see setMapRotation()
      * \since QGIS 2.8
      */
     double mapRotation() const { return mRotation; }
 
     /**
-     * Set parameters for use in transforming coordinates
+     * Sets parameters for use in transforming coordinates.
+     *
+     * Calling this method will automatically set the object as valid.
+     *
      * \param mapUnitsPerPixel Map units per pixel
      * \param centerX X coordinate of map center, in geographical units
      * \param centerY Y coordinate of map center, in geographical units
      * \param widthPixels Output width, in pixels
      * \param heightPixels Output height, in pixels
      * \param rotation clockwise rotation in degrees
+     *
+     * \note if the specified parameters result in an invalid transform then no changes will be applied to the object
      * \since QGIS 2.8
      */
     void setParameters( double mapUnitsPerPixel, double centerX, double centerY, int widthPixels, int heightPixels, double rotation );
 
-    //! String representation of the parameters used in the transform
+    /**
+     * Sets parameters for use in transforming coordinates.
+     *
+     * Calling this method will automatically set the object as valid.
+     *
+     * \param mapUnitsPerPixel Map units per pixel
+     * \param centerX X coordinate of map center, in geographical units
+     * \param centerY Y coordinate of map center, in geographical units
+     * \param widthPixels Output width, in pixels
+     * \param heightPixels Output height, in pixels
+     * \param rotation clockwise rotation in degrees
+     * \param ok will be set to TRUE if the specified parameters result in a valid transform, otherwise the changes are ignored and ok will be set to FALSE.
+     *
+     * \since QGIS 3.20
+     */
+    void setParameters( double mapUnitsPerPixel, double centerX, double centerY, int widthPixels, int heightPixels, double rotation, bool *ok ) SIP_SKIP;
+
+    /**
+     * Returns a string representation of the parameters used in the transform.
+     */
     QString showParameters() const;
 
     /**
@@ -265,7 +328,8 @@ class CORE_EXPORT QgsMapToPixel
 
     bool operator==( const QgsMapToPixel &other ) const
     {
-      return mMapUnitsPerPixel == other.mMapUnitsPerPixel
+      return mValid == other.mValid
+             && mMapUnitsPerPixel == other.mMapUnitsPerPixel
              && mWidth == other.mWidth
              && mHeight == other.mHeight
              && mRotation == other.mRotation
@@ -281,6 +345,7 @@ class CORE_EXPORT QgsMapToPixel
     }
 
   private:
+    bool mValid = false;
     double mMapUnitsPerPixel = 1;
     int mWidth = 1;
     int mHeight = 1;
