@@ -25,7 +25,6 @@
 #include "qgis_core.h"
 #include "qgis.h"
 #include "qgscolorrampshader.h"
-#include "qgsmeshdataprovider.h"
 #include "qgsmesh3daveraging.h"
 #include "qgsinterpolatedlinerenderer.h"
 
@@ -61,14 +60,14 @@ class CORE_EXPORT QgsMeshRendererMeshSettings
      *
      * \since QGIS 3.14
      */
-    QgsUnitTypes::RenderUnit lineWidthUnit() const;
+    Qgis::RenderUnit lineWidthUnit() const;
 
     /**
      * Sets units of the width of the mesh frame
      *
      * \since QGIS 3.14
      */
-    void setLineWidthUnit( const QgsUnitTypes::RenderUnit &lineWidthUnit );
+    void setLineWidthUnit( Qgis::RenderUnit lineWidthUnit );
 
     //! Writes configuration to a new DOM element
     QDomElement writeXml( QDomDocument &doc ) const;
@@ -78,7 +77,7 @@ class CORE_EXPORT QgsMeshRendererMeshSettings
   private:
     bool mEnabled = false;
     double mLineWidth = DEFAULT_LINE_WIDTH;
-    QgsUnitTypes::RenderUnit mLineWidthUnit = QgsUnitTypes::RenderMillimeters;
+    Qgis::RenderUnit mLineWidthUnit = Qgis::RenderUnit::Millimeters;
     QColor mColor = Qt::black;
 };
 
@@ -168,14 +167,14 @@ class CORE_EXPORT QgsMeshRendererScalarSettings
     *
     * \since QGIS 3.14
     */
-    QgsUnitTypes::RenderUnit edgeStrokeWidthUnit() const;
+    Qgis::RenderUnit edgeStrokeWidthUnit() const;
 
     /**
      * Sets the stroke width unit used to render edges scalar dataset
      *
      * \since QGIS 3.14
      */
-    void setEdgeStrokeWidthUnit( const QgsUnitTypes::RenderUnit &edgeStrokeWidthUnit );
+    void setEdgeStrokeWidthUnit( Qgis::RenderUnit edgeStrokeWidthUnit );
 
     //! Writes configuration to a new DOM element
     QDomElement writeXml( QDomDocument &doc, const QgsReadWriteContext &context = QgsReadWriteContext() ) const;
@@ -190,7 +189,7 @@ class CORE_EXPORT QgsMeshRendererScalarSettings
     double mOpacity = 1;
 
     QgsInterpolatedLineWidth mEdgeStrokeWidth;
-    QgsUnitTypes::RenderUnit mEdgeStrokeWidthUnit = QgsUnitTypes::RenderMillimeters;
+    Qgis::RenderUnit mEdgeStrokeWidthUnit = Qgis::RenderUnit::Millimeters;
 };
 
 /**
@@ -380,21 +379,19 @@ class CORE_EXPORT QgsMeshRendererVectorTracesSettings
     //! Sets particles count
     void setParticlesCount( int value );
     //! Returns the maximum tail length unit
-    QgsUnitTypes::RenderUnit maximumTailLengthUnit() const;
+    Qgis::RenderUnit maximumTailLengthUnit() const;
     //! Sets the maximum tail length unit
-    void setMaximumTailLengthUnit( const QgsUnitTypes::RenderUnit &maximumTailLengthUnit );
+    void setMaximumTailLengthUnit( Qgis::RenderUnit maximumTailLengthUnit );
 
     //! Reads configuration from the given DOM element
     void readXml( const QDomElement &elem );
     //! Writes configuration to a new DOM element
     QDomElement writeXml( QDomDocument &doc ) const;
 
-
-
   private:
     int mParticlesCount = 1000;
     double mMaximumTailLength = 100;
-    QgsUnitTypes::RenderUnit mMaximumTailLengthUnit = QgsUnitTypes::RenderMillimeters;
+    Qgis::RenderUnit mMaximumTailLengthUnit = Qgis::RenderUnit::Millimeters;
 
 };
 
