@@ -32,6 +32,17 @@
 /**
  * \ingroup core
  * \brief Point geometry type, with support for z-dimension and m-values.
+ *
+ * A QgsPoint represents a 2, 3 or 4-dimensional position, with X and Y and optional
+ * Z or M coordinates. Since it supports these additional dimensions, QgsPoint is
+ * used as the low-level storage of geometry coordinates throughout QGIS.
+ *
+ * In some scenarios it is preferable to use the QgsPointXY class instead, which is
+ * lighter and has smaller memory requirements compared to QgsPoint. See the QgsPointXY
+ * documentation for examples of situations where it is appropriate to use QgsPointXY
+ * instead of QgsPoint.
+ *
+ * \see QgsPointXY
  * \since QGIS 3.0, (previously QgsPointV2 since QGIS 2.10)
  */
 class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
@@ -48,8 +59,8 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
     /**
      * Construct a point with the provided initial coordinate values.
      *
-     * If \a wkbType is set to `QgsWkbTypes::Point`, `QgsWkbTypes::PointZ`, `QgsWkbTypes::PointM` or `QgsWkbTypes::PointZM`
-     * the type will be set accordingly. If it is left to the default `QgsWkbTypes::Unknown`, the type will be set
+     * If \a wkbType is set to `Qgis::WkbType::Point`, `Qgis::WkbType::PointZ`, `Qgis::WkbType::PointM` or `Qgis::WkbType::PointZM`
+     * the type will be set accordingly. If it is left to the default `Qgis::WkbType::Unknown`, the type will be set
      * based on the following rules:
      *
      * - If only x and y are specified, the type will be a 2D point.
@@ -60,21 +71,21 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
      *   pt.asWkt() # Point(43.4 5.3)
      *
      *   pt_z = QgsPoint(120, 343, 77)
-     *   pt.asWkt() # PointZ(120 343 77)
+     *   pt_z.asWkt() # PointZ(120 343 77)
      *
      *   pt_m = QgsPoint(33, 88, m=5)
      *   pt_m.m() # 5
-     *   pt_m.wkbType() # QgsWkbTypes.PointM
+     *   pt_m.wkbType() # 2001 (QgsWkbTypes.PointM)
      *
      *   pt = QgsPoint(30, 40, wkbType=QgsWkbTypes.PointZ)
      *   pt.z() # nan
-     *   pt.wkbType() # QgsWkbTypes.PointZ
+     *   pt.wkbType() # 1001 (QgsWkbTypes.PointZ)
      * \endcode
      */
 #ifndef SIP_RUN
-    QgsPoint( double x = std::numeric_limits<double>::quiet_NaN(), double y = std::numeric_limits<double>::quiet_NaN(), double z = std::numeric_limits<double>::quiet_NaN(), double m = std::numeric_limits<double>::quiet_NaN(), QgsWkbTypes::Type wkbType = QgsWkbTypes::Unknown );
+    QgsPoint( double x = std::numeric_limits<double>::quiet_NaN(), double y = std::numeric_limits<double>::quiet_NaN(), double z = std::numeric_limits<double>::quiet_NaN(), double m = std::numeric_limits<double>::quiet_NaN(), Qgis::WkbType wkbType = Qgis::WkbType::Unknown );
 #else
-    QgsPoint( SIP_PYOBJECT x = Py_None, SIP_PYOBJECT y = Py_None, SIP_PYOBJECT z = Py_None, SIP_PYOBJECT m = Py_None, SIP_PYOBJECT wkbType = Py_None ) [( double x = 0.0, double y = 0.0, double z = 0.0, double m = 0.0, QgsWkbTypes::Type wkbType = QgsWkbTypes::Unknown )];
+    QgsPoint( SIP_PYOBJECT x SIP_TYPEHINT( Optional[Union[QgsPoint, QPointF, float]] ) = Py_None, SIP_PYOBJECT y SIP_TYPEHINT( Optional[float] ) = Py_None, SIP_PYOBJECT z SIP_TYPEHINT( Optional[float] ) = Py_None, SIP_PYOBJECT m SIP_TYPEHINT( Optional[float] ) = Py_None, SIP_PYOBJECT wkbType SIP_TYPEHINT( Optional[int] ) = Py_None ) [( double x = 0.0, double y = 0.0, double z = 0.0, double m = 0.0, Qgis::WkbType wkbType = Qgis::WkbType::Unknown )];
     % MethodCode
     if ( sipCanConvertToType( a0, sipType_QgsPointXY, SIP_NOT_NONE ) && a1 == Py_None && a2 == Py_None && a3 == Py_None && a4 == Py_None )
     {
@@ -110,14 +121,13 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
       ( a0 == Py_None || PyFloat_AsDouble( a0 ) != -1.0 || !PyErr_Occurred() ) &&
       ( a1 == Py_None || PyFloat_AsDouble( a1 ) != -1.0 || !PyErr_Occurred() ) &&
       ( a2 == Py_None || PyFloat_AsDouble( a2 ) != -1.0 || !PyErr_Occurred() ) &&
-      ( a3 == Py_None || PyFloat_AsDouble( a3 ) != -1.0 || !PyErr_Occurred() ) &&
-      ( a4 == Py_None || sipCanConvertToEnum( a4, sipType_QgsWkbTypes_Type ) ) )
+      ( a3 == Py_None || PyFloat_AsDouble( a3 ) != -1.0 || !PyErr_Occurred() ) )
     {
       double x = a0 == Py_None ? std::numeric_limits<double>::quiet_NaN() : PyFloat_AsDouble( a0 );
       double y = a1 == Py_None ? std::numeric_limits<double>::quiet_NaN() : PyFloat_AsDouble( a1 );
       double z = a2 == Py_None ? std::numeric_limits<double>::quiet_NaN() : PyFloat_AsDouble( a2 );
       double m = a3 == Py_None ? std::numeric_limits<double>::quiet_NaN() : PyFloat_AsDouble( a3 );
-      QgsWkbTypes::Type wkbType = a4 == Py_None ? QgsWkbTypes::Unknown : static_cast<QgsWkbTypes::Type>( sipConvertToEnum( a4, sipType_QgsWkbTypes_Type ) );
+      Qgis::WkbType wkbType = a4 == Py_None ? Qgis::WkbType::Unknown : static_cast<Qgis::WkbType>( sipConvertToEnum( a4, sipType_Qgis_WkbType ) );
       sipCpp = new sipQgsPoint( QgsPoint( x, y, z, m, wkbType ) );
     }
     else // Invalid ctor arguments
@@ -143,15 +153,15 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
      *
      * \note Not available in Python bindings
      */
-    explicit QgsPoint( QgsWkbTypes::Type wkbType, double x = std::numeric_limits<double>::quiet_NaN(), double y = std::numeric_limits<double>::quiet_NaN(), double z = std::numeric_limits<double>::quiet_NaN(), double m = std::numeric_limits<double>::quiet_NaN() ) SIP_SKIP;
+    explicit QgsPoint( Qgis::WkbType wkbType, double x = std::numeric_limits<double>::quiet_NaN(), double y = std::numeric_limits<double>::quiet_NaN(), double z = std::numeric_limits<double>::quiet_NaN(), double m = std::numeric_limits<double>::quiet_NaN() ) SIP_SKIP;
 
-    bool operator==( const QgsAbstractGeometry &other ) const override
+    bool operator==( const QgsAbstractGeometry &other ) const override SIP_HOLDGIL
     {
       const QgsPoint *pt = qgsgeometry_cast< const QgsPoint * >( &other );
       if ( !pt )
         return false;
 
-      const QgsWkbTypes::Type type = wkbType();
+      const Qgis::WkbType type = wkbType();
 
       if ( pt->wkbType() != type )
         return false;
@@ -193,7 +203,7 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
       return true;
     }
 
-    bool operator!=( const QgsAbstractGeometry &other ) const override
+    bool operator!=( const QgsAbstractGeometry &other ) const override SIP_HOLDGIL
     {
       return !operator==( other );
     }
@@ -203,28 +213,28 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
      * \see setX()
      * \see rx()
      */
-    double x() const { return mX; }
+    double x() const SIP_HOLDGIL { return mX; }
 
     /**
      * Returns the point's y-coordinate.
      * \see setY()
      * \see ry()
      */
-    double y() const { return mY; }
+    double y() const SIP_HOLDGIL { return mY; }
 
     /**
      * Returns the point's z-coordinate.
      * \see setZ()
      * \see rz()
      */
-    double z() const { return mZ; }
+    double z() const SIP_HOLDGIL { return mZ; }
 
     /**
      * Returns the point's m value.
      * \see setM()
      * \see rm()
      */
-    double m() const { return mM; }
+    double m() const SIP_HOLDGIL { return mM; }
 
     /**
      * Returns a reference to the x-coordinate of this point.
@@ -267,7 +277,7 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
      * \see x()
      * \see rx()
      */
-    void setX( double x )
+    void setX( double x ) SIP_HOLDGIL
     {
       clearCache();
       mX = x;
@@ -278,7 +288,7 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
      * \see y()
      * \see ry()
      */
-    void setY( double y )
+    void setY( double y ) SIP_HOLDGIL
     {
       clearCache();
       mY = y;
@@ -291,7 +301,7 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
      * \see z()
      * \see rz()
      */
-    void setZ( double z )
+    void setZ( double z ) SIP_HOLDGIL
     {
       if ( !is3D() )
         return;
@@ -306,7 +316,7 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
      * \see m()
      * \see rm()
      */
-    void setM( double m )
+    void setM( double m ) SIP_HOLDGIL
     {
       if ( !isMeasure() )
         return;
@@ -318,7 +328,7 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
      * Returns the point as a QPointF.
      * \since QGIS 2.14
      */
-    QPointF toQPointF() const
+    QPointF toQPointF() const SIP_HOLDGIL
     {
       return QPointF( mX, mY );
     }
@@ -330,7 +340,7 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
      * \see distanceSquared()
      * \since QGIS 3.0
     */
-    double distance( double x, double y ) const
+    double distance( double x, double y ) const SIP_HOLDGIL
     {
       return std::sqrt( ( mX - x ) * ( mX - x ) + ( mY - y ) * ( mY - y ) );
     }
@@ -339,9 +349,10 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
      * Returns the Cartesian 2D distance between this point and another point. In certain
      * cases it may be more appropriate to call the faster distanceSquared() method, e.g.,
      * when comparing distances.
+     * \see distanceSquared()
      * \since QGIS 3.0
     */
-    double distance( const QgsPoint &other ) const
+    double distance( const QgsPoint &other ) const SIP_HOLDGIL
     {
       return std::sqrt( ( mX - other.x() ) * ( mX - other.x() ) + ( mY - other.y() ) * ( mY - other.y() ) );
     }
@@ -353,7 +364,7 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
      * \see distance()
      * \since QGIS 3.0
     */
-    double distanceSquared( double x, double y ) const
+    double distanceSquared( double x, double y ) const SIP_HOLDGIL
     {
       return ( mX - x ) * ( mX - x ) + ( mY - y ) * ( mY - y );
     }
@@ -365,61 +376,62 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
      * \see distance()
      * \since QGIS 3.0
     */
-    double distanceSquared( const QgsPoint &other ) const
+    double distanceSquared( const QgsPoint &other ) const SIP_HOLDGIL
     {
       return ( mX - other.x() ) * ( mX - other.x() ) + ( mY - other.y() ) * ( mY - other.y() );
     }
 
     /**
      * Returns the Cartesian 3D distance between this point and a specified x, y, z coordinate. In certain
-     * cases it may be more appropriate to call the faster distanceSquared() method, e.g.,
+     * cases it may be more appropriate to call the faster distanceSquared3D() method, e.g.,
      * when comparing distances.
-     * \see distanceSquared()
+     * \see distanceSquared3D()
      * \since QGIS 3.0
     */
-    double distance3D( double x, double y, double z ) const;
+    double distance3D( double x, double y, double z ) const SIP_HOLDGIL;
 
     /**
      * Returns the Cartesian 3D distance between this point and another point. In certain
-     * cases it may be more appropriate to call the faster distanceSquared() method, e.g.,
+     * cases it may be more appropriate to call the faster distanceSquared3D() method, e.g.,
      * when comparing distances.
+     * \see distanceSquared3D()
      * \since QGIS 3.0
     */
-    double distance3D( const QgsPoint &other ) const;
+    double distance3D( const QgsPoint &other ) const SIP_HOLDGIL;
 
     /**
      * Returns the Cartesian 3D squared distance between this point and a specified x, y, z coordinate. Calling
-     * this is faster than calling distance(), and may be useful in use cases such as comparing
-     * distances where the extra expense of calling distance() is not required.
-     * \see distance()
+     * this is faster than calling distance3D(), and may be useful in use cases such as comparing
+     * distances where the extra expense of calling distance3D() is not required.
+     * \see distance3D()
      * \since QGIS 3.0
     */
-    double distanceSquared3D( double x, double y, double z ) const;
+    double distanceSquared3D( double x, double y, double z ) const SIP_HOLDGIL;
 
     /**
      * Returns the Cartesian 3D squared distance between this point and another point. Calling
-     * this is faster than calling distance(), and may be useful in use cases such as comparing
-     * distances where the extra expense of calling distance() is not required.
-     * \see distance()
+     * this is faster than calling distance3D(), and may be useful in use cases such as comparing
+     * distances where the extra expense of calling distance3D() is not required.
+     * \see distance3D()
      * \since QGIS 3.0
     */
-    double distanceSquared3D( const QgsPoint &other ) const;
+    double distanceSquared3D( const QgsPoint &other ) const SIP_HOLDGIL;
 
     /**
      * Calculates Cartesian azimuth between this point and other one (clockwise in degree, starting from north)
      * \since QGIS 3.0
      */
-    double azimuth( const QgsPoint &other ) const;
+    double azimuth( const QgsPoint &other ) const SIP_HOLDGIL;
 
     /**
      * Calculates Cartesian inclination between this point and other one (starting from zenith = 0 to nadir = 180. Horizon = 90)
      * Returns 90.0 if the distance between this point and other one is equal to 0 (same point).
      * \since QGIS 3.0
      */
-    double inclination( const QgsPoint &other ) const;
+    double inclination( const QgsPoint &other ) const SIP_HOLDGIL;
 
     /**
-     * Returns a new point which correspond to this point projected by a specified distance
+     * Returns a new point which corresponds to this point projected by a specified distance
      * with specified angles (azimuth and inclination), using Cartesian mathematics.
      * M value is preserved.
      * \param distance distance to project
@@ -437,8 +449,8 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
      *   pr = p.project ( 1, 0, 90 )
      *   # pr is a 2D point: 'Point (1 3)'
      *   pr = p.project (1, 0, 0 )
-     *   # pr is a 3D point: 'PointZ (1 2 1)'
-     *   p = QgsPoint( QgsWkbTypes.PointZ, 1, 2, 2 ) # 3D point
+     *   # pr is a 3D point: 'PointZ (1 2 nan)'
+     *   p = QgsPoint( 1, 2, 2, wkbType=QgsWkbTypes.PointZ ) # 3D point
      *   pr = p.project ( 1, 0 )
      *   # pr is a 3D point: 'PointZ (1 3 2)'
      *   pr = p.project ( 1, 0, 90 )
@@ -448,63 +460,66 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
      * \endcode
      * \since QGIS 3.0
      */
-    QgsPoint project( double distance, double azimuth, double inclination = 90.0 ) const;
+    QgsPoint project( double distance, double azimuth, double inclination = 90.0 ) const SIP_HOLDGIL;
 
     /**
      * Calculates the vector obtained by subtracting a point from this point.
      * \since QGIS 3.0
      */
-    QgsVector operator-( const QgsPoint &p ) const { return QgsVector( mX - p.mX, mY - p.mY ); }
+    QgsVector operator-( const QgsPoint &p ) const SIP_HOLDGIL { return QgsVector( mX - p.mX, mY - p.mY ); }
 
     /**
      * Adds a vector to this point in place.
      * \since QGIS 3.0
      */
-    QgsPoint &operator+=( QgsVector v ) { mX += v.x(); mY += v.y(); return *this; }
+    QgsPoint &operator+=( QgsVector v ) SIP_HOLDGIL { mX += v.x(); mY += v.y(); return *this; }
 
     /**
      * Subtracts a vector from this point in place.
      * \since QGIS 3.0
      */
-    QgsPoint &operator-=( QgsVector v ) { mX -= v.x(); mY -= v.y(); return *this; }
+    QgsPoint &operator-=( QgsVector v ) SIP_HOLDGIL { mX -= v.x(); mY -= v.y(); return *this; }
 
     /**
      * Adds a vector to this point.
      * \since QGIS 3.0
      */
-    QgsPoint operator+( QgsVector v ) const { QgsPoint r = *this; r.rx() += v.x(); r.ry() += v.y(); return r; }
+    QgsPoint operator+( QgsVector v ) const SIP_HOLDGIL { QgsPoint r = *this; r.rx() += v.x(); r.ry() += v.y(); return r; }
 
     /**
      * Subtracts a vector from this point.
      * \since QGIS 3.0
      */
-    QgsPoint operator-( QgsVector v ) const { QgsPoint r = *this; r.rx() -= v.x(); r.ry() -= v.y(); return r; }
+    QgsPoint operator-( QgsVector v ) const SIP_HOLDGIL { QgsPoint r = *this; r.rx() -= v.x(); r.ry() -= v.y(); return r; }
 
     //implementation of inherited methods
-    bool isEmpty() const override;
-    QgsRectangle boundingBox() const override;
-    QString geometryType() const override;
-    int dimension() const override;
+    void normalize() final SIP_HOLDGIL;
+    bool isEmpty() const override SIP_HOLDGIL;
+    QgsRectangle boundingBox() const override SIP_HOLDGIL;
+    QString geometryType() const override SIP_HOLDGIL;
+    int dimension() const override SIP_HOLDGIL;
     QgsPoint *clone() const override SIP_FACTORY;
     QgsPoint *snappedToGrid( double hSpacing, double vSpacing, double dSpacing = 0, double mSpacing = 0 ) const override SIP_FACTORY;
     bool removeDuplicateNodes( double epsilon = 4 * std::numeric_limits<double>::epsilon(), bool useZValues = false ) override;
     void clear() override;
     bool fromWkb( QgsConstWkbPtr &wkb ) override;
     bool fromWkt( const QString &wkt ) override;
-    QByteArray asWkb( QgsAbstractGeometry::WkbFlags = nullptr ) const override;
+    int wkbSize( QgsAbstractGeometry::WkbFlags flags = QgsAbstractGeometry::WkbFlags() ) const override;
+    QByteArray asWkb( QgsAbstractGeometry::WkbFlags = QgsAbstractGeometry::WkbFlags() ) const override;
     QString asWkt( int precision = 17 ) const override;
     QDomElement asGml2( QDomDocument &doc, int precision = 17, const QString &ns = "gml", QgsAbstractGeometry::AxisOrder axisOrder = QgsAbstractGeometry::AxisOrder::XY ) const override;
     QDomElement asGml3( QDomDocument &doc, int precision = 17, const QString &ns = "gml", QgsAbstractGeometry::AxisOrder axisOrder = QgsAbstractGeometry::AxisOrder::XY ) const override;
     json asJsonObject( int precision = 17 ) const override SIP_SKIP;
     QString asKml( int precision = 17 ) const override;
     void draw( QPainter &p ) const override;
-    void transform( const QgsCoordinateTransform &ct, QgsCoordinateTransform::TransformDirection d = QgsCoordinateTransform::ForwardTransform, bool transformZ = false ) override SIP_THROW( QgsCsException );
+    QPainterPath asQPainterPath() const override;
+    void transform( const QgsCoordinateTransform &ct, Qgis::TransformDirection d = Qgis::TransformDirection::Forward, bool transformZ = false ) override SIP_THROW( QgsCsException );
     void transform( const QTransform &t, double zTranslate = 0.0, double zScale = 1.0, double mTranslate = 0.0, double mScale = 1.0 ) override;
     QgsCoordinateSequence coordinateSequence() const override;
-    int nCoordinates() const override;
+    int nCoordinates() const override SIP_HOLDGIL;
     int vertexNumberFromVertexId( QgsVertexId id ) const override;
     QgsAbstractGeometry *boundary() const override SIP_FACTORY;
-    bool isValid( QString &error SIP_OUT, int flags = 0 ) const override;
+    bool isValid( QString &error SIP_OUT, Qgis::GeometryValidityFlags flags = Qgis::GeometryValidityFlags() ) const override SIP_HOLDGIL;
 
     //low-level editing
     bool insertVertex( QgsVertexId position, const QgsPoint &vertex ) override;
@@ -517,8 +532,9 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
 
     /**
      * Angle undefined. Always returns 0.0
-        \param vertex the vertex id
-        \returns 0.0*/
+     * \param vertex the vertex id
+     * \returns 0.0
+    */
     double vertexAngle( QgsVertexId vertex ) const override;
 
     int vertexCount( int /*part*/ = 0, int /*ring*/ = 0 ) const override;
@@ -527,13 +543,16 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
     QgsPoint vertexAt( QgsVertexId /*id*/ ) const override;
     QgsPoint *toCurveType() const override SIP_FACTORY;
     double segmentLength( QgsVertexId startVertex ) const override;
+    bool boundingBoxIntersects( const QgsRectangle &rectangle ) const override SIP_HOLDGIL;
 
     bool addZValue( double zValue = 0 ) override;
     bool addMValue( double mValue = 0 ) override;
     bool dropZValue() override;
     bool dropMValue() override;
     void swapXy() override;
-    bool convertTo( QgsWkbTypes::Type type ) override;
+    bool convertTo( Qgis::WkbType type ) override;
+
+    bool transform( QgsAbstractGeometryTransformer *transformer, QgsFeedback *feedback = nullptr ) override;
 
 #ifndef SIP_RUN
 
@@ -547,9 +566,9 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
      * \note Not available in Python. Objects will be automatically be converted to the appropriate target type.
      * \since QGIS 3.0
      */
-    inline const QgsPoint *cast( const QgsAbstractGeometry *geom ) const
+    inline static const QgsPoint *cast( const QgsAbstractGeometry *geom )
     {
-      if ( geom && QgsWkbTypes::flatType( geom->wkbType() ) == QgsWkbTypes::Point )
+      if ( geom && QgsWkbTypes::flatType( geom->wkbType() ) == Qgis::WkbType::Point )
         return static_cast<const QgsPoint *>( geom );
       return nullptr;
     }
@@ -567,6 +586,7 @@ class CORE_EXPORT QgsPoint: public QgsAbstractGeometry
 
   protected:
 
+    int compareToSameClass( const QgsAbstractGeometry *other ) const final;
     int childCount() const override;
     QgsPoint childPoint( int index ) const override;
 
