@@ -23,17 +23,19 @@
 #include "qgsproject.h"
 #include "qgsexpressioncontext.h"
 #include "qgsfeaturerequest.h"
-#include "qgsmaplayerlistutils.h"
-#include "qgsexception.h"
 #include "qgsprocessingfeedback.h"
 #include "qgsprocessingutils.h"
 
+
+#include <QThread>
+#include <QPointer>
+
 class QgsProcessingLayerPostProcessorInterface;
 
 /**
  * \class QgsProcessingContext
  * \ingroup core
- * Contains information about the context in which a processing algorithm is executed.
+ * \brief Contains information about the context in which a processing algorithm is executed.
  *
  * Contextual information includes settings such as the associated project, and
  * expression context.
@@ -47,10 +49,22 @@ class CORE_EXPORT QgsProcessingContext
     //! Flags that affect how processing algorithms are run
     enum Flag
     {
-      // UseSelectionIfPresent = 1 << 0,
+      // For future API flexibility only and to avoid sip issues, remove when real entries are added to flags.
+      Unused = 1 << 0, //!< Temporary unused entry
     };
     Q_DECLARE_FLAGS( Flags, Flag )
 
+    /**
+     * Logging level for algorithms to use when pushing feedback messages.
+     *
+     * \since QGIS 3.20
+     */
+    enum LogLevel
+    {
+      DefaultLevel = 0, //!< Default logging level
+      Verbose, //!< Verbose logging
+    };
+
     /**
      * Constructor for QgsProcessingContext.
      */
@@ -73,11 +87,19 @@ class CORE_EXPORT QgsProcessingContext
       mProject = other.mProject;
       mTransformContext = other.mTransformContext;
       mExpressionContext = other.mExpressionContext;
+      mExpressionContext.setLoadedLayerStore( &tempLayerStore );
       mInvalidGeometryCallback = other.mInvalidGeometryCallback;
+      mUseDefaultInvalidGeometryCallback = other.mUseDefaultInvalidGeometryCallback;
       mInvalidGeometryCheck = other.mInvalidGeometryCheck;
       mTransformErrorCallback = other.mTransformErrorCallback;
       mDefaultEncoding = other.mDefaultEncoding;
       mFeedback = other.mFeedback;
+      mPreferredVectorFormat = other.mPreferredVectorFormat;
+      mPreferredRasterFormat = other.mPreferredRasterFormat;
+      mEllipsoid = other.mEllipsoid;
+      mDistanceUnit = other.mDistanceUnit;
+      mAreaUnit = other.mAreaUnit;
+      mLogLevel = other.mLogLevel;
     }
 
     /**
@@ -101,8 +123,8 @@ class CORE_EXPORT QgsProcessingContext
     /**
      * Sets the \a project in which the algorithm will be executed.
      *
-     * This also automatically sets the transformContext() to match
-     * the project's transform context.
+     * This also automatically sets the transformContext(), ellipsoid(), distanceUnit() and
+     * areaUnit() to match the project's settings.
      *
      * \see project()
      */
@@ -110,7 +132,15 @@ class CORE_EXPORT QgsProcessingContext
     {
       mProject = project;
       if ( mProject )
+      {
         mTransformContext = mProject->transformContext();
+        if ( mEllipsoid.isEmpty() )
+          mEllipsoid = mProject->ellipsoid();
+        if ( mDistanceUnit == Qgis::DistanceUnit::Unknown )
+          mDistanceUnit = mProject->distanceUnits();
+        if ( mAreaUnit == Qgis::AreaUnit::Unknown )
+          mAreaUnit = mProject->areaUnits();
+      }
     }
 
     /**
@@ -126,7 +156,7 @@ class CORE_EXPORT QgsProcessingContext
     /**
      * Sets the expression \a context.
      */
-    void setExpressionContext( const QgsExpressionContext &context ) { mExpressionContext = context; }
+    void setExpressionContext( const QgsExpressionContext &context );
 
     /**
      * Returns the coordinate transform context.
@@ -144,6 +174,80 @@ class CORE_EXPORT QgsProcessingContext
      */
     void setTransformContext( const QgsCoordinateTransformContext &context ) { mTransformContext = context; }
 
+    /**
+     * Returns the ellipsoid to use for distance and area calculations.
+     *
+     * \see setEllipsoid()
+     * \since QGIS 3.16
+     */
+    QString ellipsoid() const;
+
+    /**
+     * Sets a specified \a ellipsoid to use for distance and area calculations.
+     *
+     * If not explicitly set, the ellipsoid will default to the project()'s ellipsoid setting.
+     *
+     * \see ellipsoid()
+     * \since QGIS 3.16
+     */
+    void setEllipsoid( const QString &ellipsoid );
+
+    /**
+     * Returns the distance unit to use for distance calculations.
+     *
+     * \see setDistanceUnit()
+     * \see areaUnit()
+     * \since QGIS 3.16
+     */
+    Qgis::DistanceUnit distanceUnit() const;
+
+    /**
+     * Sets the \a unit to use for distance calculations.
+     *
+     * If not explicitly set, the unit will default to the project()'s distance unit setting.
+     *
+     * \see distanceUnit()
+     * \see setAreaUnit()
+     * \since QGIS 3.16
+     */
+    void setDistanceUnit( Qgis::DistanceUnit unit );
+
+    /**
+     * Returns the area unit to use for area calculations.
+     *
+     * \see setAreaUnit()
+     * \see distanceUnit()
+     * \since QGIS 3.16
+     */
+    Qgis::AreaUnit areaUnit() const;
+
+    /**
+     * Sets the \a unit to use for area calculations.
+     *
+     * If not explicitly set, the unit will default to the project()'s area unit setting.
+     *
+     * \see areaUnit()
+     * \see setDistanceUnit()
+     * \since QGIS 3.16
+     */
+    void setAreaUnit( Qgis::AreaUnit areaUnit );
+
+    /**
+     * Returns the current time range to use for temporal operations.
+     *
+     * \see setCurrentTimeRange()
+     * \since QGIS 3.18
+     */
+    QgsDateTimeRange currentTimeRange() const;
+
+    /**
+     * Sets the \a current time range to use for temporal operations.
+     *
+     * \see currentTimeRange()
+     * \since QGIS 3.18
+     */
+    void setCurrentTimeRange( const QgsDateTimeRange &currentTimeRange );
+
     /**
      * Returns a reference to the layer store used for storing temporary layers during
      * algorithm execution.
@@ -151,7 +255,7 @@ class CORE_EXPORT QgsProcessingContext
     QgsMapLayerStore *temporaryLayerStore() { return &tempLayerStore; }
 
     /**
-     * Details for layers to load into projects.
+     * \brief Details for layers to load into projects.
      * \ingroup core
      * \since QGIS 3.0
      */
@@ -180,6 +284,13 @@ class CORE_EXPORT QgsProcessingContext
          */
         QString name;
 
+        /**
+         * Set to TRUE if LayerDetails::name should always be used as the loaded layer name, regardless
+         * of the user's local Processing settings.
+         * \since QGIS 3.16
+         */
+        bool forceName = false;
+
         /**
          * Associated output name from algorithm which generated the layer.
          */
@@ -311,7 +422,7 @@ class CORE_EXPORT QgsProcessingContext
      * \since QGIS 3.0
      */
 #ifndef SIP_RUN
-    void setInvalidGeometryCallback( const std::function< void( const QgsFeature & ) > &callback ) { mInvalidGeometryCallback = callback; }
+    void setInvalidGeometryCallback( const std::function< void( const QgsFeature & ) > &callback ) { mInvalidGeometryCallback = callback; mUseDefaultInvalidGeometryCallback = false; }
 #else
     void setInvalidGeometryCallback( SIP_PYCALLABLE / AllowNone / );
     % MethodCode
@@ -335,7 +446,14 @@ class CORE_EXPORT QgsProcessingContext
      * \see setInvalidGeometryCallback()
      * \since QGIS 3.0
      */
-    SIP_SKIP std::function< void( const QgsFeature & ) > invalidGeometryCallback() const { return mInvalidGeometryCallback; }
+    SIP_SKIP std::function< void( const QgsFeature & ) > invalidGeometryCallback( QgsFeatureSource *source = nullptr ) const;
+
+    /**
+     * Returns the default callback function to use for a particular invalid geometry \a check
+     * \note not available in Python bindings
+     * \since QGIS 3.14
+     */
+    SIP_SKIP std::function< void( const QgsFeature & ) > defaultInvalidGeometryCallbackForCheck( QgsFeatureRequest::InvalidGeometryCheck check, QgsFeatureSource *source = nullptr ) const;
 
     /**
      * Sets a callback function to use when encountering a transform error when iterating
@@ -413,6 +531,7 @@ class CORE_EXPORT QgsProcessingContext
      */
     void pushToThread( QThread *thread )
     {
+      // cppcheck-suppress assertWithSideEffect
       Q_ASSERT_X( QThread::currentThread() == QgsProcessingContext::thread(), "QgsProcessingContext::pushToThread", "Cannot push context to another thread unless the current thread matches the existing context thread affinity" );
       tempLayerStore.moveToThread( thread );
     }
@@ -522,16 +641,67 @@ class CORE_EXPORT QgsProcessingContext
      */
     void setPreferredRasterFormat( const QString &format ) { mPreferredRasterFormat = format; }
 
+    /**
+     * Returns the logging level for algorithms to use when pushing feedback messages to users.
+     *
+     * \see setLogLevel()
+     * \since QGIS 3.20
+     */
+    LogLevel logLevel() const;
+
+    /**
+     * Sets the logging \a level for algorithms to use when pushing feedback messages to users.
+     *
+     * \see logLevel()
+     * \since QGIS 3.20
+     */
+    void setLogLevel( LogLevel level );
+
+    /**
+     * Exports the context's settings to a variant map.
+     *
+     * \since QGIS 3.24
+     */
+    QVariantMap exportToMap() const;
+
+    /**
+     * Flags controlling the results given by asQgisProcessArguments().
+     *
+     * \since QGIS 3.24
+     */
+    enum class ProcessArgumentFlag : int
+    {
+      IncludeProjectPath = 1 << 0, //!< Include the associated project path argument
+    };
+    Q_DECLARE_FLAGS( ProcessArgumentFlags, ProcessArgumentFlag )
+
+    /**
+     * Returns list of the equivalent qgis_process arguments representing the settings from the context.
+     *
+     * \since QGIS 3.24
+     */
+    QStringList asQgisProcessArguments( QgsProcessingContext::ProcessArgumentFlags flags = QgsProcessingContext::ProcessArgumentFlags() ) const;
+
   private:
 
-    QgsProcessingContext::Flags mFlags = nullptr;
+    QgsProcessingContext::Flags mFlags = QgsProcessingContext::Flags();
     QPointer< QgsProject > mProject;
     QgsCoordinateTransformContext mTransformContext;
+
+    QString mEllipsoid;
+    Qgis::DistanceUnit mDistanceUnit = Qgis::DistanceUnit::Unknown;
+    Qgis::AreaUnit mAreaUnit = Qgis::AreaUnit::Unknown;
+
+    QgsDateTimeRange mCurrentTimeRange;
+
     //! Temporary project owned by the context, used for storing temporarily loaded map layers
     QgsMapLayerStore tempLayerStore;
     QgsExpressionContext mExpressionContext;
+
     QgsFeatureRequest::InvalidGeometryCheck mInvalidGeometryCheck = QgsFeatureRequest::GeometryNoCheck;
+    bool mUseDefaultInvalidGeometryCallback = true;
     std::function< void( const QgsFeature & ) > mInvalidGeometryCallback;
+
     std::function< void( const QgsFeature & ) > mTransformErrorCallback;
     QString mDefaultEncoding;
     QMap< QString, LayerDetails > mLayersToLoadOnCompletion;
@@ -541,16 +711,19 @@ class CORE_EXPORT QgsProcessingContext
     QString mPreferredVectorFormat;
     QString mPreferredRasterFormat;
 
+    LogLevel mLogLevel = DefaultLevel;
+
 #ifdef SIP_RUN
     QgsProcessingContext( const QgsProcessingContext &other );
 #endif
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS( QgsProcessingContext::Flags )
+Q_DECLARE_OPERATORS_FOR_FLAGS( QgsProcessingContext::ProcessArgumentFlags )
 
 
 /**
- * An interface for layer post-processing handlers for execution following a processing algorithm operation.
+ * \brief An interface for layer post-processing handlers for execution following a processing algorithm operation.
  *
  * Note that post-processing of a layer will ONLY occur if that layer is set to be loaded into a QGIS project
  * on algorithm completion. See QgsProcessingContext::layersToLoadOnCompletion().
