@@ -31,8 +31,8 @@
 #include "qgsdataprovider.h"
 #include "qgis_core.h"
 #include <functional>
-#include "qgsvectorlayerexporter.h"
 #include "qgsabstractproviderconnection.h"
+#include "qgsabstractlayermetadataprovider.h"
 #include "qgsfields.h"
 #include "qgsexception.h"
 
@@ -43,6 +43,9 @@ class QgsTransaction;
 class QgsRasterDataProvider;
 class QgsMeshDataProvider;
 class QgsAbstractDatabaseProviderConnection;
+class QgsLayerMetadata;
+class QgsProviderSublayerDetails;
+class QgsFeedback;
 
 struct QgsMesh;
 
@@ -83,11 +86,32 @@ class CORE_EXPORT QgsMeshDriverMetadata
      * \param description short description of the driver
      * \param capabilities driver's capabilities
      * \param writeDatasetOnFileSuffix suffix used to write datasets on file
+     *
+     * \deprecated QGIS 3.22
+     */
+    Q_DECL_DEPRECATED QgsMeshDriverMetadata( const QString &name,
+        const QString &description,
+        const MeshDriverCapabilities &capabilities,
+        const QString &writeDatasetOnFileSuffix ) SIP_DEPRECATED;
+
+    /**
+     * Constructs driver metadata with selected capabilities
+     *
+     * \param name name/key of the driver
+     * \param description short description of the driver
+     * \param capabilities driver's capabilities
+     * \param writeDatasetOnFileSuffix suffix used to write datasets on file
+     * \param writeMeshFrameOnFileSuffix suffix used to write mesh frame on file
+     * \param maxVerticesPerface maximum vertices count per face supported by the driver
+     *
+     * \since QGIS 3.22
      */
     QgsMeshDriverMetadata( const QString &name,
                            const QString &description,
                            const MeshDriverCapabilities &capabilities,
-                           const QString &writeDatasetOnFileSuffix );
+                           const QString &writeDatasetOnFileSuffix,
+                           const QString &writeMeshFrameOnFileSuffix,
+                           int maxVerticesPerface );
 
     /**
      * Returns the capabilities for this driver.
@@ -109,11 +133,27 @@ class CORE_EXPORT QgsMeshDriverMetadata
      */
     QString writeDatasetOnFileSuffix() const;
 
+    /**
+     * Returns the suffix used to write mesh on file
+     *
+     * \since QGIS 3.22
+     */
+    QString writeMeshFrameOnFileSuffix() const;
+
+    /**
+     * Returns the maximum number of vertices per face supported by the driver
+     *
+     * \since QGIS 3.22
+     */
+    int maximumVerticesCountPerFace() const;
+
   private:
     QString mName;
     QString mDescription;
     MeshDriverCapabilities mCapabilities;
     QString mWriteDatasetOnFileSuffix;
+    QString mWriteMeshFrameOnFileSuffix;
+    int mMaxVerticesPerFace = -1;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS( QgsMeshDriverMetadata::MeshDriverCapabilities )
@@ -150,6 +190,8 @@ class CORE_EXPORT QgsProviderMetadata : public QObject
     {
       PriorityForUri = 1 << 0, //!< Indicates that the metadata can calculate a priority for a URI
       LayerTypesForUri = 1 << 1, //!< Indicates that the metadata can determine valid layer types for a URI
+      QuerySublayers = 1 << 2, //!< Indicates that the metadata can query sublayers for a URI (since QGIS 3.22)
+      CreateDatabase = 1 << 3, //!< Indicates that the metadata can create new empty databases (since QGIS 3.28)
     };
     Q_DECLARE_FLAGS( ProviderMetadataCapabilities, ProviderMetadataCapability )
 
@@ -161,6 +203,7 @@ class CORE_EXPORT QgsProviderMetadata : public QObject
     enum ProviderCapability
     {
       FileBasedUris = 1 << 0, //!< Indicates that the provider can utilize URIs which are based on paths to files (as opposed to database or internet paths)
+      SaveLayerMetadata = 1 << 1, //!< Indicates that the provider supports saving native layer metadata (since QGIS 3.20)
     };
     Q_DECLARE_FLAGS( ProviderCapabilities, ProviderCapability )
 
@@ -203,6 +246,13 @@ class CORE_EXPORT QgsProviderMetadata : public QObject
      */
     QString description() const;
 
+    /**
+     * Returns an icon representing the provider.
+     *
+     * \since QGIS 3.26
+     */
+    virtual QIcon icon() const;
+
     /**
      * Returns the provider metadata capabilities.
      *
@@ -217,6 +267,51 @@ class CORE_EXPORT QgsProviderMetadata : public QObject
      */
     virtual QgsProviderMetadata::ProviderCapabilities providerCapabilities() const;
 
+    /**
+     * Returns a list of the map layer types supported by the provider.
+     *
+     * \since QGIS 3.26
+     */
+#ifndef SIP_RUN
+    virtual QList< Qgis::LayerType > supportedLayerTypes() const;
+#else
+    SIP_PYOBJECT supportedLayerTypes() const SIP_TYPEHINT( List[Qgis.LayerType] );
+    % MethodCode
+    // adapted from the qpymultimedia_qlist.sip file from the PyQt6 sources
+
+    const QList< Qgis::LayerType > cppRes = sipCpp->supportedLayerTypes();
+
+    PyObject *l = PyList_New( cppRes.size() );
+
+    if ( !l )
+      sipIsErr = 1;
+    else
+    {
+      for ( int i = 0; i < cppRes.size(); ++i )
+      {
+        PyObject *eobj = sipConvertFromEnum( static_cast<int>( cppRes.at( i ) ),
+                                             sipType_Qgis_LayerType );
+
+        if ( !eobj )
+        {
+          sipIsErr = 1;
+        }
+
+        PyList_SetItem( l, i, eobj );
+      }
+
+      if ( !sipIsErr )
+      {
+        sipRes = l;
+      }
+      else
+      {
+        Py_DECREF( l );
+      }
+    }
+    % End
+#endif
+
     /**
      * This returns the library file name
      *
@@ -303,7 +398,7 @@ class CORE_EXPORT QgsProviderMetadata : public QObject
      *
      * \since QGIS 3.18
      */
-    virtual QList< QgsMapLayerType > validLayerTypesForUri( const QString &uri ) const;
+    virtual QList< Qgis::LayerType > validLayerTypesForUri( const QString &uri ) const;
 
     /**
      * Returns TRUE if the specified \a uri is known by this provider to be something which should
@@ -323,6 +418,55 @@ class CORE_EXPORT QgsProviderMetadata : public QObject
      */
     virtual bool uriIsBlocklisted( const QString &uri ) const;
 
+    /**
+     * Given a \a uri, returns any sidecar files which are associated with the URI and this
+     * provider.
+     *
+     * In this context a sidecar file is defined as a file which shares the same base filename
+     * as a dataset, but which differs in file extension. It defines the list of additional
+     * files which must be renamed or deleted alongside the main file associated with the
+     * dataset in order to completely rename/delete the dataset.
+     *
+     * For instance, the OGR provider would return the corresponding .dbf, .idx, etc files for a
+     * uri pointing at a .shp file.
+     *
+     * Implementations should files any files which MAY exist for the URI, and it is up to the caller
+     * to filter these to only existing files if required.
+     *
+     * \note Some file formats consist of a set of static file names, such as ESRI aigrid datasets
+     * which consist of a folder with files with the names "hdr.adf", "prj.adf", etc. These statically
+     * named files are NOT considered as sidecar files.
+     *
+     * \since QGIS 3.22
+     */
+    virtual QStringList sidecarFilesForUri( const QString &uri ) const;
+
+    /**
+     * Queries the specified \a uri and returns a list of any valid sublayers found in the dataset which can be handled by this provider.
+     *
+     * The optional \a flags argument can be used to control the behavior of the query.
+     *
+     * The optional \a feedback argument can be used to provide cancellation support for long-running queries.
+     *
+     * \note Providers which implement this method should always return a list of sublayer details for any valid, even if the \a uri
+     * only relates to a single layer. Returning a non-empty list indicates that the provider is able to load at least one layer using the \a uri,
+     * and is used to collate a combined layer of all providers which support the URI (e.g. in the case that a URI may be readable by multiple
+     * different providers).
+     *
+     * \since QGIS 3.22
+    */
+    virtual QList< QgsProviderSublayerDetails > querySublayers( const QString &uri, Qgis::SublayerQueryFlags flags = Qgis::SublayerQueryFlags(), QgsFeedback *feedback = nullptr ) const;
+
+    /**
+     * Returns a name that can be used as a group name for sublayers retrieved from
+     * the specified \a uri.
+     *
+     * The default implementation returns an empty string.
+     *
+     * \since QGIS 3.30
+    */
+    virtual QString suggestGroupNameForUri( const QString &uri ) const;
+
     /**
      * Class factory to return a pointer to a newly created QgsDataProvider object
      *
@@ -360,9 +504,9 @@ class CORE_EXPORT QgsProviderMetadata : public QObject
      * \note not available in Python bindings
      * \since QGIS 3.10
      */
-    virtual QgsVectorLayerExporter::ExportError createEmptyLayer( const QString &uri,
+    virtual Qgis::VectorExportResult createEmptyLayer( const QString &uri,
         const QgsFields &fields,
-        QgsWkbTypes::Type wkbType,
+        Qgis::WkbType wkbType,
         const QgsCoordinateReferenceSystem &srs,
         bool overwrite,
         QMap<int, int> &oldToNewAttrIdxMap,
@@ -370,6 +514,23 @@ class CORE_EXPORT QgsProviderMetadata : public QObject
         const QMap<QString, QVariant> *options );
 #endif
 
+    /**
+     * Creates a new empty database at the specified \a uri.
+     *
+     * This method can be used for supported providers to construct a new empty database. For instance, the OGR provider
+     * metadata createDatabase() method can be used to create new empty GeoPackage or FileGeodatabase databases.
+     *
+     * \param uri destination URI for newly created database.
+     * \param errorMessage will be set to a descriptive error message if the database could not be successfully created.
+     *
+     * \returns TRUE if the database was successfully created
+     *
+     * \note This method is only supported by providers which return the QgsProviderMetadata::ProviderMetadataCapability::CreateDatabase capability.
+     *
+     * \since QGIS 3.28
+     */
+    virtual bool createDatabase( const QString &uri, QString &errorMessage SIP_OUT );
+
     /**
      * Creates a new instance of the raster data provider.
      * \since QGIS 3.10
@@ -386,15 +547,24 @@ class CORE_EXPORT QgsProviderMetadata : public QObject
       const QStringList &createOptions = QStringList() ) SIP_FACTORY;
 
     /**
-     * Creates mesh data source, that is the mesh frame stored in file, memory or with other way (depending of the provider)
+     * Creates mesh data source from a file name \a fileName and a driver \a driverName, that is the mesh frame stored in file, memory or with other way (depending of the provider)
      * \since QGIS 3.16
      */
     virtual bool createMeshData(
       const QgsMesh &mesh,
-      const QString uri,
+      const QString &fileName,
       const QString &driverName,
       const QgsCoordinateReferenceSystem &crs ) const;
 
+    /**
+     * Creates mesh data source from an \a uri, that is the mesh frame stored in file, memory or with other way (depending of the provider)
+     * \since QGIS 3.22
+     */
+    virtual bool createMeshData(
+      const QgsMesh &mesh,
+      const QString &uri,
+      const QgsCoordinateReferenceSystem &crs ) const;
+
     /**
      * Returns pyramid resampling methods available for provider
      * \since QGIS 3.10
@@ -430,6 +600,32 @@ class CORE_EXPORT QgsProviderMetadata : public QObject
      */
     virtual QString encodeUri( const QVariantMap &parts ) const;
 
+    /**
+     * Converts absolute path(s) to relative path(s) in the given provider-specific URI. and
+     * returns modified URI according to the context object's configuration.
+     * This is commonly used when writing project files.
+     * If a provider does not work with paths, unmodified URI will be returned.
+     * \returns modified URI with relative path(s)
+     * \note this function may not be supported by all providers. The default
+     *       implementation uses QgsPathResolver::writePath() on the whole URI.
+     * \see relativeToAbsoluteUri()
+     * \since QGIS 3.30
+     */
+    virtual QString absoluteToRelativeUri( const QString &uri, const QgsReadWriteContext &context ) const;
+
+    /**
+     * Converts relative path(s) to absolute path(s) in the given provider-specific URI. and
+     * returns modified URI according to the context object's configuration.
+     * This is commonly used when reading project files.
+     * If a provider does not work with paths, unmodified URI will be returned.
+     * \returns modified URI with absolute path(s)
+     * \note this function may not be supported by all providers. The default
+     *       implementation uses QgsPathResolver::readPath() on the whole URI.
+     * \see absoluteToRelativeUri()
+     * \since QGIS 3.30
+     */
+    virtual QString relativeToAbsoluteUri( const QString &uri, const QgsReadWriteContext &context ) const;
+
     /**
      * Returns data item providers. Caller is responsible for ownership of the item providers
      * \see QgsProviderGuiMetadata::dataItemGuiProviders()
@@ -446,20 +642,43 @@ class CORE_EXPORT QgsProviderMetadata : public QObject
     virtual int listStyles( const QString &uri, QStringList &ids, QStringList &names,
                             QStringList &descriptions, QString &errCause );
 
+    /**
+     * Returns TRUE if a layer style with the specified \a styleId exists in the provider defined by \a uri.
+     *
+     * \param uri provider URI
+     * \param styleId style ID to test for
+     * \param errorCause will be set to a descriptive error message, if an error occurs while checking if the style exists
+     * \returns TRUE if the layer style already exists
+     *
+     * \see getStyleById()
+     * \since QGIS 3.24
+     */
+    virtual bool styleExists( const QString &uri, const QString &styleId, QString &errorCause SIP_OUT );
+
     /**
      * Gets a layer style defined by \a uri
+     *
+     * \see styleExists()
+     *
      * \since QGIS 3.10
      */
-    virtual QString getStyleById( const QString &uri, QString styleId, QString &errCause );
+    virtual QString getStyleById( const QString &uri, const QString &styleId, QString &errCause );
 
     /**
      * Deletes a layer style defined by \a styleId
      * \since QGIS 3.10
      */
-    virtual bool deleteStyleById( const QString &uri, QString styleId, QString &errCause );
+    virtual bool deleteStyleById( const QString &uri, const QString &styleId, QString &errCause );
 
     /**
-     * Saves a layer style to provider
+     * Saves a layer style to provider.
+     *
+     * \note Prior to QGIS 3.24, this method would show a message box warning when a
+     * style with the same \a styleName already existed to confirm replacing the style with the user.
+     * Since 3.24, calling this method will ALWAYS overwrite any existing style with the same name.
+     * Use styleExists() to test in advance if a style already exists and handle this appropriately
+     * in your client code.
+     *
      * \since QGIS 3.10
      */
     virtual bool saveStyle( const QString &uri, const QString &qmlStyle, const QString &sldStyle,
@@ -472,6 +691,32 @@ class CORE_EXPORT QgsProviderMetadata : public QObject
      */
     virtual QString loadStyle( const QString &uri, QString &errCause );
 
+    /**
+     * Loads a layer style from the provider storage, reporting its name.
+     * \param uri data source uri
+     * \param styleName the name of the style if available, empty otherwise
+     * \param errCause report errors
+     * \returns the style QML (XML)
+     * \since QGIS 3.30
+     */
+    virtual QString loadStoredStyle( const QString &uri, QString &styleName, QString &errCause );
+
+    /**
+     * Saves \a metadata to the layer corresponding to the specified \a uri.
+     *
+     * \param uri uri of layer to store metadata for
+     * \param metadata layer metadata
+     * \param errorMessage descriptive string of error if encountered
+     *
+     * \returns TRUE if the metadata was successfully saved.
+     *
+     * \throws QgsNotSupportedException if the provider does not support saving layer metadata for the
+     * specified \a uri.
+     *
+     * \since QGIS 3.20
+     */
+    virtual bool saveLayerMetadata( const QString &uri, const QgsLayerMetadata &metadata, QString &errorMessage SIP_OUT ) SIP_THROW( QgsNotSupportedException );
+
     /**
      * Creates database by the provider on the path
      * \since QGIS 3.10
@@ -659,6 +904,6 @@ class CORE_EXPORT QgsProviderMetadata : public QObject
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS( QgsProviderMetadata::ProviderMetadataCapabilities )
-
+Q_DECLARE_OPERATORS_FOR_FLAGS( QgsProviderMetadata::ProviderCapabilities )
 
 #endif //QGSPROVIDERMETADATA_H
