@@ -18,6 +18,7 @@
 #include "qgsmapcanvasitem.h"
 #include "qgis_sip.h"
 #include "qgsgeometry.h"
+#include "qgscoordinatereferencesystem.h"
 
 #include <QBrush>
 #include <QVector>
@@ -30,6 +31,7 @@
 
 class QgsVectorLayer;
 class QPaintEvent;
+class QgsSymbol;
 
 #ifdef SIP_RUN
 % ModuleHeaderCode
@@ -57,7 +59,13 @@ class GUI_EXPORT QgsRubberBand : public QgsMapCanvasItem
 #ifdef SIP_RUN
     SIP_CONVERT_TO_SUBCLASS_CODE
     if ( dynamic_cast<QgsRubberBand *>( sipCpp ) )
+    {
       sipType = sipType_QgsRubberBand;
+      // We need to tweak the pointer as sip believes it is single inheritance
+      // from QgsMapCanvasItem, but the raw address of QgsRubberBand (sipCpp)
+      // is actually a QObject
+      *sipCppRet = dynamic_cast<QgsRubberBand *>( sipCpp );
+    }
     else
       sipType = nullptr;
     SIP_END
@@ -131,7 +139,8 @@ class GUI_EXPORT QgsRubberBand : public QgsMapCanvasItem
      *  \param geometryType Defines how the data should be drawn onto the screen.
      *         QgsWkbTypes::LineGeometry, QgsWkbTypes::PolygonGeometry or QgsWkbTypes::PointGeometry
      */
-    QgsRubberBand( QgsMapCanvas *mapCanvas SIP_TRANSFERTHIS, QgsWkbTypes::GeometryType geometryType = QgsWkbTypes::LineGeometry );
+    QgsRubberBand( QgsMapCanvas *mapCanvas SIP_TRANSFERTHIS, Qgis::GeometryType geometryType = Qgis::GeometryType::Line );
+    ~QgsRubberBand() override;
 
     /**
      * Sets the color for the rubberband.
@@ -234,7 +243,7 @@ class GUI_EXPORT QgsRubberBand : public QgsMapCanvasItem
      * Sets the representation type according to geometryType.
      *  \param geometryType Defines how the data should be drawn onto the screen. (Use Qgis::Line, Qgis::Polygon or Qgis::Point)
      */
-    void reset( QgsWkbTypes::GeometryType geometryType = QgsWkbTypes::LineGeometry );
+    void reset( Qgis::GeometryType geometryType = Qgis::GeometryType::Line );
 
     /**
      * Adds a vertex to the rubberband and update canvas.
@@ -317,6 +326,13 @@ class GUI_EXPORT QgsRubberBand : public QgsMapCanvasItem
      */
     void setToCanvasRectangle( QRect rect );
 
+    /**
+     * Copies the points from another rubber band.
+     *
+     * \since QGIS 3.22
+     */
+    void copyPointsFrom( const QgsRubberBand *other );
+
     /**
      * Adds the geometry of an existing feature to a rubberband
      * This is useful for multi feature highlighting.
@@ -328,11 +344,10 @@ class GUI_EXPORT QgsRubberBand : public QgsMapCanvasItem
      * After adding the final geometry updatePosition() should be called.
      *
      *  \param geometry the geometry object. Will be treated as a collection of vertices.
-     *  \param layer the layer containing the feature, used for coord transformation to map
-     *               crs. If \a layer is NULLPTR, the coordinates are not going to be transformed.
+     *  \param layer the layer associated with the geometry. This is used for transforming the geometry from the layer's CRS to the map crs. If \a layer is NULLPTR no coordinate transformation will occur.
      *  \param doUpdate set to FALSE to defer updates of the rubber band.
      */
-    void addGeometry( const QgsGeometry &geometry, QgsVectorLayer *layer, bool doUpdate = true );
+    void addGeometry( const QgsGeometry &geometry, QgsMapLayer *layer, bool doUpdate = true );
 
     /**
      * Adds a \a geometry to the rubberband.
@@ -385,6 +400,29 @@ class GUI_EXPORT QgsRubberBand : public QgsMapCanvasItem
 
     void updatePosition() override;
 
+    /**
+     * Returns the symbol used for rendering the rubberband, if set.
+     *
+     * \see setSymbol()
+     * \since QGIS 3.20
+     */
+    QgsSymbol *symbol() const;
+
+    /**
+     * Sets the \a symbol used for rendering the rubberband.
+     *
+     * Ownership of \a symbol is transferred to the rubberband.
+     *
+     * \warning Only line and fill symbols are currently supported.
+     *
+     * \note Setting a symbol for the rubberband overrides any other appearance setting,
+     * such as the strokeColor() or width().
+     *
+     * \see setSymbol()
+     * \since QGIS 3.20
+     */
+    void setSymbol( QgsSymbol *symbol SIP_TRANSFER );
+
   protected:
 
     /**
@@ -423,11 +461,13 @@ class GUI_EXPORT QgsRubberBand : public QgsMapCanvasItem
     std::unique_ptr<QSvgRenderer> mSvgRenderer;
     QPoint mSvgOffset;
 
+    std::unique_ptr< QgsSymbol > mSymbol;
+
     /**
      * Nested lists used for multitypes
      */
     QVector< QVector< QVector <QgsPointXY> > > mPoints;
-    QgsWkbTypes::GeometryType mGeometryType = QgsWkbTypes::PolygonGeometry;
+    Qgis::GeometryType mGeometryType = Qgis::GeometryType::Polygon;
     double mTranslationOffsetX = 0.0;
     double mTranslationOffsetY = 0.0;
 
