@@ -23,6 +23,7 @@
 #include "qgis_core.h"
 #include "qgis_sip.h"
 #include "qgslogger.h"
+#include "qgssettingstreenode.h"
 
 /**
  * \ingroup core
@@ -79,35 +80,6 @@ class CORE_EXPORT QgsSettings : public QObject
       Gps, //!< GPS section, since QGIS 3.22
     };
 
-    /**
-     * \ingroup core
-     * \brief Prefixes for the settings keys
-     * \since QGIS 3.26
-     */
-    class Prefix SIP_SKIP
-    {
-      public:
-        static const inline char *APP_GEOREFERENCER = "app/georeferencer";
-        static const inline char *CORE = "core";
-        static const inline char *CORE_LAYOUT = "core/Layout";
-        static const inline char *GEOMETRYVALIDATION = "geometry_validation";
-        static const inline char *GPS = "gps";
-        static const inline char *GUI_LOCATORFILTERS = "gui/locator_filters";
-        static const inline char *GUI_QGIS = "gui/qgis";
-        static const inline char *LOCALE = "locale";
-        static const inline char *MAP = "Map";
-        static const inline char *PLUGINS = "plugins";
-        static const inline char *PROCESSING_CONFIGURATION = "Processing/Configuration";
-        static const inline char *QGIS = "qgis";
-        static const inline char *QGIS_DIGITIZING = "qgis/digitizing";
-        static const inline char *QGIS_DIGITIZING_SHAPEMAPTOOLS = "qgis/digitizing/shape-map-tools";
-        static const inline char *QGIS_NETWORKANDPROXY = "qgis/networkAndProxy";
-        static const inline char *SVG = "svg";
-        static const inline char *ELEVATION_PROFILE = "elevation-profile";
-        static const inline char *CORE_LAYERTREE = "core/layer-tree";
-        static const inline char *STYLE_MANAGER = "app/style-manager";
-    };
-
     /**
      * Constructs a QgsSettings object for accessing settings of the application
      * called application from the organization called organization, and with parent parent.
@@ -206,7 +178,7 @@ class CORE_EXPORT QgsSettings : public QObject
     //! Returns a list of all top-level keys that can be read using the QSettings object.
     QStringList childKeys() const;
     //! Returns a list of all key top-level groups that contain keys that can be read using the QSettings object.
-    QStringList childGroups() const;
+    QStringList childGroups( Qgis::SettingsOrigin origin = Qgis::SettingsOrigin::Any ) const;
     //! Returns a list of all key top-level groups (same as childGroups) but only for groups defined in global settings.
     QStringList globalChildGroups() const;
     //! Returns the path to the Global Settings QSettings storage file
@@ -231,6 +203,13 @@ class CORE_EXPORT QgsSettings : public QObject
      */
     void setArrayIndex( int i );
 
+    /**
+     * Returns the origin of the setting if it exists at the given \a key
+     * \note it will return Qgis::SettingsOrigin::Any if the key doesn't exist
+     * \since QGIS 3.30
+     */
+    Qgis::SettingsOrigin origin( const QString &key ) const;
+
     /**
      * Sets the value of setting key to value. If the key already exists, the previous value is overwritten.
      * An optional Section argument can be used to set a value to a specific Section.
@@ -266,6 +245,7 @@ class CORE_EXPORT QgsSettings : public QObject
     % End
 #endif
 
+
 #ifndef SIP_RUN
 
     /**
