@@ -239,9 +239,6 @@ class CORE_EXPORT QgsStyleProxyModel: public QSortFilterProxyModel
      */
     explicit QgsStyleProxyModel( QgsStyleModel *model, QObject *parent SIP_TRANSFERTHIS = nullptr );
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 13, 0)
-    SIP_IF_FEATURE( CONCATENATED_TABLES_MODEL )
-
     /**
      * Constructor for QgsStyleProxyModel, using the specified source combined \a model and \a parent object.
      *
@@ -252,8 +249,6 @@ class CORE_EXPORT QgsStyleProxyModel: public QSortFilterProxyModel
      * \since QGIS 3.26
      */
     explicit QgsStyleProxyModel( QgsCombinedStyleModel *model, QObject *parent SIP_TRANSFERTHIS = nullptr );
-    SIP_END
-#endif
 
     /**
      * Returns the current filter string, if set.
@@ -356,7 +351,7 @@ class CORE_EXPORT QgsStyleProxyModel: public QSortFilterProxyModel
      *
      * \see setLayerType()
      */
-    QgsWkbTypes::GeometryType layerType() const;
+    Qgis::GeometryType layerType() const;
 
     /**
      * Sets the layer \a type filter. Set \a type to QgsWkbTypes::UnknownGeometry if no
@@ -364,7 +359,7 @@ class CORE_EXPORT QgsStyleProxyModel: public QSortFilterProxyModel
      *
      * \see layerType()
      */
-    void setLayerType( QgsWkbTypes::GeometryType type );
+    void setLayerType( Qgis::GeometryType type );
 
     /**
      * Sets a tag \a id to filter style entities by. Only entities with the given
@@ -473,9 +468,7 @@ class CORE_EXPORT QgsStyleProxyModel: public QSortFilterProxyModel
     void initialize();
 
     QgsStyleModel *mModel = nullptr;
-#if QT_VERSION >= QT_VERSION_CHECK(5, 13, 0)
     QgsCombinedStyleModel *mCombinedModel = nullptr;
-#endif
     QgsStyle *mStyle = nullptr;
 
     QString mFilterString;
@@ -496,7 +489,7 @@ class CORE_EXPORT QgsStyleProxyModel: public QSortFilterProxyModel
     bool mSymbolTypeFilterEnabled = false;
     Qgis::SymbolType mSymbolType = Qgis::SymbolType::Marker;
 
-    QgsWkbTypes::GeometryType mLayerType = QgsWkbTypes::UnknownGeometry;
+    Qgis::GeometryType mLayerType = Qgis::GeometryType::Unknown;
 
 };
 
