@@ -238,7 +238,7 @@ class CORE_EXPORT QgsSymbolLayerUtils
      * \returns encoded string
      * \see decodeSldUom()
      */
-    static QString encodeSldUom( QgsUnitTypes::RenderUnit unit, double *scaleFactor );
+    static QString encodeSldUom( Qgis::RenderUnit unit, double *scaleFactor );
 
     /**
      * Decodes a SLD unit of measure string to a render unit.
@@ -247,7 +247,7 @@ class CORE_EXPORT QgsSymbolLayerUtils
      * \returns matching render unit
      * \see encodeSldUom()
      */
-    static QgsUnitTypes::RenderUnit decodeSldUom( const QString &str, double *scaleFactor = nullptr );
+    static Qgis::RenderUnit decodeSldUom( const QString &str, double *scaleFactor = nullptr );
 
     /**
      * Returns the size scaled in pixels according to the uom attribute.
@@ -314,7 +314,7 @@ class CORE_EXPORT QgsSymbolLayerUtils
      * \see symbolLayerPreviewIcon()
      * \since QGIS 2.9
      */
-    static QPicture symbolLayerPreviewPicture( const QgsSymbolLayer *layer, QgsUnitTypes::RenderUnit units, QSize size, const QgsMapUnitScale &scale = QgsMapUnitScale(), Qgis::SymbolType parentSymbolType = Qgis::SymbolType::Hybrid );
+    static QPicture symbolLayerPreviewPicture( const QgsSymbolLayer *layer, Qgis::RenderUnit units, QSize size, const QgsMapUnitScale &scale = QgsMapUnitScale(), Qgis::SymbolType parentSymbolType = Qgis::SymbolType::Hybrid );
 
     /**
      * Draws a symbol layer preview to an icon.
@@ -323,10 +323,11 @@ class CORE_EXPORT QgsSymbolLayerUtils
      * \param size target size of preview icon
      * \param scale map unit scale for preview
      * \param parentSymbolType since QGIS 3.22, can be used to specify the parent symbol type so that geometry generator preview icons are correctly calculated
+     * \param mapLayer since QGIS 3.28, can be used to specify the associated map layer so that layer related expressions are correctly calculated
      * \returns icon containing symbol layer preview
      * \see symbolLayerPreviewPicture()
      */
-    static QIcon symbolLayerPreviewIcon( const QgsSymbolLayer *layer, QgsUnitTypes::RenderUnit u, QSize size, const QgsMapUnitScale &scale = QgsMapUnitScale(), Qgis::SymbolType parentSymbolType = Qgis::SymbolType::Hybrid );
+    static QIcon symbolLayerPreviewIcon( const QgsSymbolLayer *layer, Qgis::RenderUnit u, QSize size, const QgsMapUnitScale &scale = QgsMapUnitScale(), Qgis::SymbolType parentSymbolType = Qgis::SymbolType::Hybrid, QgsMapLayer *mapLayer = nullptr );
 
     /**
      * Returns an icon preview for a color ramp.
@@ -411,7 +412,7 @@ class CORE_EXPORT QgsSymbolLayerUtils
     /**
      * Creates a symbol layer list from a DOM \a element.
      */
-    static bool createSymbolLayerListFromSld( QDomElement &element, QgsWkbTypes::GeometryType geomType, QList<QgsSymbolLayer *> &layers );
+    static bool createSymbolLayerListFromSld( QDomElement &element, Qgis::GeometryType geomType, QList<QgsSymbolLayer *> &layers );
 
     static QgsSymbolLayer *createFillLayerFromSld( QDomElement &element ) SIP_FACTORY;
     static QgsSymbolLayer *createLineLayerFromSld( QDomElement &element ) SIP_FACTORY;
@@ -421,7 +422,20 @@ class CORE_EXPORT QgsSymbolLayerUtils
      * Converts a polygon symbolizer \a element to a list of marker symbol layers.
      */
     static bool convertPolygonSymbolizerToPointMarker( QDomElement &element, QList<QgsSymbolLayer *> &layerList );
+
+    /**
+     * Checks if \a element contains an ExternalGraphic element with format "image/svg+xml"
+     * @return TRUE if the ExternalGraphic with format "image/svg+xml" is found .
+     */
     static bool hasExternalGraphic( QDomElement &element );
+
+    /**
+     * Checks if \a element contains an ExternalGraphic element, if the optional \a format is specified it will also be checked.
+     * @return TRUE if the ExternalGraphic element is found and the optionally specified format matches.
+     * \since QGIS 3.30
+     */
+    static bool hasExternalGraphicV2( QDomElement &element, const QString format = QString() );
+
     static bool hasWellKnownMark( QDomElement &element );
 
     static bool needFontMarker( QDomElement &element );
@@ -432,6 +446,13 @@ class CORE_EXPORT QgsSymbolLayerUtils
     static bool needPointPatternFill( QDomElement &element );
     static bool needSvgFill( QDomElement &element );
 
+    /**
+     * Checks if \a element contains a graphic fill with a raster image of type PNG, JPEG or GIF.
+     * @return TRUE if element contains a graphic fill with a raster image.
+     * \since QGIS 3.30
+     */
+    static bool needRasterImageFill( QDomElement &element );
+
     static void fillToSld( QDomDocument &doc, QDomElement &element,
                            Qt::BrushStyle brushStyle, const QColor &color = QColor() );
     static bool fillFromSld( QDomElement &element,
@@ -815,21 +836,21 @@ class CORE_EXPORT QgsSymbolLayerUtils
      *  returns the value un-modified
      * \since QGIS 3.0
      */
-    static double rescaleUom( double size, QgsUnitTypes::RenderUnit unit, const QVariantMap &props );
+    static double rescaleUom( double size, Qgis::RenderUnit unit, const QVariantMap &props );
 
     /**
      * Rescales the given point based on the uomScale found in the props, if any is found, otherwise
      *  returns a copy of the original point
      * \since QGIS 3.0
      */
-    static QPointF rescaleUom( QPointF point, QgsUnitTypes::RenderUnit unit, const QVariantMap &props ) SIP_PYNAME( rescalePointUom );
+    static QPointF rescaleUom( QPointF point, Qgis::RenderUnit unit, const QVariantMap &props ) SIP_PYNAME( rescalePointUom );
 
     /**
      * Rescales the given array based on the uomScale found in the props, if any is found, otherwise
      *  returns a copy of the original point
      * \since QGIS 3.0
      */
-    static QVector<qreal> rescaleUom( const QVector<qreal> &array, QgsUnitTypes::RenderUnit unit, const QVariantMap &props ) SIP_PYNAME( rescaleArrayUom );
+    static QVector<qreal> rescaleUom( const QVector<qreal> &array, Qgis::RenderUnit unit, const QVariantMap &props ) SIP_PYNAME( rescaleArrayUom );
 
     /**
      * Checks if the properties contain scaleMinDenom and scaleMaxDenom, if available, they are added into the SE Rule element
@@ -862,8 +883,9 @@ class CORE_EXPORT QgsSymbolLayerUtils
     /**
      * Converts a set of symbol layer id to a set of pointers to actual symbol layers carried by the feature renderer.
      * \since QGIS 3.12
+     * \deprecated since QGIS 3.30 because it was related to old QgsSymbolLayerReference system
      */
-    static QSet<const QgsSymbolLayer *> toSymbolLayerPointers( QgsFeatureRenderer *renderer, const QSet<QgsSymbolLayerId> &symbolLayerIds );
+    Q_DECL_DEPRECATED static QSet<const QgsSymbolLayer *> toSymbolLayerPointers( const QgsFeatureRenderer *renderer, const QSet<QgsSymbolLayerId> &symbolLayerIds );
 
     /**
      * Calculates the frame rate (in frames per second) at which the given \a renderer must be redrawn.
@@ -881,10 +903,14 @@ class CORE_EXPORT QgsSymbolLayerUtils
      * \param maxSize the maximum size in mm
      * \param context the render context
      * \param width expected width, can be changed by the function
-     * \param height expected height, can be changed by this function
-     * \return 0 if size is within minSize/maxSize range. New symbol if size was out of min/max range. Caller takes ownership
+     * \param height expected height, can be changed by the function
+     * \param ok if not nullptr, ok is set to false if it's not possible to compute a restricted symbol (if geometry generators
+     * are involved for instance)
+     * \return nullptr if size is within minSize/maxSize range or if it's not possible to compute a
+     * restricted size symbol. New symbol if size was out of min/max range.
+     * Caller takes ownership
      */
-    static QgsSymbol *restrictedSizeSymbol( const QgsSymbol *s, double minSize, double maxSize, QgsRenderContext *context, double &width, double &height );
+    static QgsSymbol *restrictedSizeSymbol( const QgsSymbol *s, double minSize, double maxSize, QgsRenderContext *context, double &width, double &height, bool *ok = nullptr );
 
     /**
      * Evaluates a map of properties using the given \a context and returns a variant map with evaluated expressions from the properties.
@@ -892,6 +918,41 @@ class CORE_EXPORT QgsSymbolLayerUtils
      */
     static QgsStringMap evaluatePropertiesMap( const QMap<QString, QgsProperty> &propertiesMap, const QgsExpressionContext &context );
 
+    /**
+     * Calculate the minimum size in pixels of a symbol tile given the symbol \a width and \a height and the symbol layer rotation \a angleRad in radians (counter clockwise).
+     * The method makes approximations and can modify \a angle in order to generate the smallest possible tile.
+     * \param width marker width, including margins
+     * \param height marker height, including margins
+     * \param angleRad symbol layer rotation angle in radians (counter clockwise), it may be approximated by the method to minimize the tile size.
+     * \return the size of the tile
+     * \since QGIS 3.30
+     */
+    static QSize tileSize( int width, int height, double &angleRad SIP_INOUT );
+
+    /**
+     * Remove recursively unique id from all \a symbol symbol layers and set an empty string instead
+     * \since QGIS 3.30
+     */
+    static void clearSymbolLayerIds( QgsSymbol *symbol );
+
+    /**
+     * Remove recursively unique id from \a symbolLayer and its children and set an empty string instead
+     * \since QGIS 3.30
+     */
+    static void clearSymbolLayerIds( QgsSymbolLayer *symbolLayer );
+
+    /**
+     * Regenerate recursively unique id from all \a symbol symbol layers
+     * \since QGIS 3.30
+     */
+    static void resetSymbolLayerIds( QgsSymbol *symbol );
+
+    /**
+     * Regenerate recursively unique id from \a symbolLayer and its children
+     * \since QGIS 3.30
+     */
+    static void resetSymbolLayerIds( QgsSymbolLayer *symbolLayer );
+
     ///@cond PRIVATE
 #ifndef SIP_RUN
     static QgsProperty rotateWholeSymbol( double additionalRotation, const QgsProperty &property )
@@ -922,8 +983,6 @@ class CORE_EXPORT QgsSymbolLayerUtils
 class QPolygonF;
 
 //! calculate geometry shifted by a specified distance
-QList<QPolygonF> offsetLine( QPolygonF polyline, double dist, QgsWkbTypes::GeometryType geometryType ) SIP_SKIP;
+QList<QPolygonF> offsetLine( QPolygonF polyline, double dist, Qgis::GeometryType geometryType ) SIP_SKIP;
 
 #endif
-
-
