@@ -17,20 +17,34 @@
 #define QGSTESSELLATEDPOLYGONGEOMETRY_H
 
 #include "qgsfeatureid.h"
-#include "qgspolygon.h"
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 #include <Qt3DRender/QGeometry>
+#else
+#include <Qt3DCore/QGeometry>
+#endif
 
+class Qgs3DSceneExporter;
+class QgsPolygon;
+class QgsPointXY;
+
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 namespace Qt3DRender
 {
   class QBuffer;
 }
+#else
+namespace Qt3DCore
+{
+  class QBuffer;
+}
+#endif
 
 #define SIP_NO_FILE
 
 /**
  * \ingroup 3d
- * Class derived from Qt3DRender::QGeometry that represents polygons tessellated into 3D geometry.
+ * \brief Class derived from Qt3DRender::QGeometry that represents polygons tessellated into 3D geometry.
  *
  * Takes a list of polygons as input, internally it does tessellation and writes output to the internal
  * vertex buffer. Optionally it can add "walls" if the extrusion height is non-zero.
@@ -39,12 +53,16 @@ namespace Qt3DRender
  *
  * \since QGIS 3.0
  */
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 class QgsTessellatedPolygonGeometry : public Qt3DRender::QGeometry
+#else
+class QgsTessellatedPolygonGeometry : public Qt3DCore::QGeometry
+#endif
 {
     Q_OBJECT
   public:
     //! Constructor
-    QgsTessellatedPolygonGeometry( QNode *parent = nullptr );
+    QgsTessellatedPolygonGeometry( bool _withNormals = true, bool invertNormals = false, bool addBackFaces = false, bool addTextureCoords = false, QNode *parent = nullptr );
 
     //! Returns whether the normals of triangles will be inverted (useful for fixing clockwise / counter-clockwise face vertex orders)
     bool invertNormals() const { return mInvertNormals; }
@@ -63,6 +81,12 @@ class QgsTessellatedPolygonGeometry : public Qt3DRender::QGeometry
      */
     void setAddBackFaces( bool add ) { mAddBackFaces = add; }
 
+    /**
+     * Sets whether the texture coordinates will be generated
+     * \since QGIS 3.16
+     */
+    void setAddTextureCoords( bool add ) { mAddTextureCoords = add; }
+
     //! Initializes vertex buffer from given polygons. Takes ownership of passed polygon geometries
     void setPolygons( const QList<QgsPolygon *> &polygons, const QList<QgsFeatureId> &featureIds, const QgsPointXY &origin, float extrusionHeight, const QList<float> &extrusionHeightPerPolygon = QList<float>() );
 
@@ -79,11 +103,20 @@ class QgsTessellatedPolygonGeometry : public Qt3DRender::QGeometry
      */
     QgsFeatureId triangleIndexToFeatureId( uint triangleIndex ) const;
 
+    friend class Qgs3DSceneExporter;
   private:
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     Qt3DRender::QAttribute *mPositionAttribute = nullptr;
     Qt3DRender::QAttribute *mNormalAttribute = nullptr;
+    Qt3DRender::QAttribute *mTextureCoordsAttribute = nullptr;
     Qt3DRender::QBuffer *mVertexBuffer = nullptr;
+#else
+    Qt3DCore::QAttribute *mPositionAttribute = nullptr;
+    Qt3DCore::QAttribute *mNormalAttribute = nullptr;
+    Qt3DCore::QAttribute *mTextureCoordsAttribute = nullptr;
+    Qt3DCore::QBuffer *mVertexBuffer = nullptr;
+#endif
 
     QVector<QgsFeatureId> mTriangleIndexFids;
     QVector<uint> mTriangleIndexStartingIndices;
@@ -91,6 +124,7 @@ class QgsTessellatedPolygonGeometry : public Qt3DRender::QGeometry
     bool mWithNormals = true;
     bool mInvertNormals = false;
     bool mAddBackFaces = false;
+    bool mAddTextureCoords = false;
 };
 
 #endif // QGSTESSELLATEDPOLYGONGEOMETRY_H
