@@ -45,6 +45,8 @@
 #include "qgsrange.h"
 #include "qgsinterval.h"
 #include "qgsrenderchecker.h"
+#include "qgsmultirenderchecker.h"
+#include "qgsunittypes.h"
 #include "qgis_test.h"
 
 #define QGSTEST_MAIN(TestObject) \
@@ -92,11 +94,22 @@
     QGSCOMPARENEAR( rectangle1.yMaximum(), rectangle2.yMaximum(), epsilon ); \
   }(void)(0)
 
+#define QGSCOMPARENEARVECTOR3D(v1,v2,epsilon) { \
+    QGSCOMPARENEAR( v1.x(), v2.x(), epsilon ); \
+    QGSCOMPARENEAR( v1.y(), v2.y(), epsilon ); \
+    QGSCOMPARENEAR( v1.z(), v2.z(), epsilon ); \
+  }(void)(0)
+
 //sometimes GML attributes are in a different order - but that's ok
 #define QGSCOMPAREGML(result,expected) { \
     QCOMPARE( result.replace( QStringLiteral("ts=\" \" cs=\",\""), QStringLiteral("cs=\",\" ts=\" \"") ), expected ); \
   }(void)(0)
 
+// Start your PostgreSQL-backend connection requiring test with this macro
+#define QGSTEST_NEED_PGTEST_DB() \
+  if ( getenv( "QGIS_PGTEST_DB_SKIP" ) ) \
+    QSKIP( "Test disabled due to QGIS_PGTEST_DB_SKIP env variable being set" );
+
 /**
  * Base class for tests.
  *
@@ -119,8 +132,9 @@ class TEST_EXPORT QgsTest : public QObject
       return qgetenv( "RUN_FLAKY_TESTS" ) == QStringLiteral( "true" );
     }
 
-    QgsTest( const QString &name )
+    QgsTest( const QString &name, const QString &controlPathPrefix = QString() )
       : mName( name )
+      , mControlPathPrefix( controlPathPrefix )
     {}
 
     ~QgsTest() override
@@ -129,6 +143,64 @@ class TEST_EXPORT QgsTest : public QObject
         writeLocalHtmlReport( mReport );
     }
 
+  protected:
+
+    QString mName;
+    QString mReport;
+    QString mControlPathPrefix;
+
+    bool renderMapSettingsCheck( const QString &name, const QString &referenceImage, const QgsMapSettings &mapSettings )
+    {
+      //use the QgsRenderChecker test utility class to
+      //ensure the rendered output matches our control image
+      QgsMultiRenderChecker checker;
+      checker.setControlPathPrefix( mControlPathPrefix );
+      checker.setControlName( "expected_" + referenceImage );
+      checker.setMapSettings( mapSettings );
+      const bool result = checker.runTest( name );
+      if ( !result )
+      {
+        appendToReport( name, checker.report() );
+
+      }
+      return result;
+    }
+
+    bool imageCheck( const QString &name, const QString &referenceImage, const QImage &image, const QString &controlName = QString(), int allowedMismatch = 20, const QSize &sizeTolerance = QSize( 0, 0 ) )
+    {
+      const QString renderedFileName = QDir::tempPath() + '/' + name + ".png";
+      image.save( renderedFileName );
+
+      QgsMultiRenderChecker checker;
+      checker.setControlPathPrefix( mControlPathPrefix );
+      checker.setControlName( controlName.isEmpty() ? "expected_" + referenceImage : controlName );
+      checker.setRenderedImage( renderedFileName );
+      checker.setSizeTolerance( sizeTolerance.width(), sizeTolerance.height() );
+
+      const bool result = checker.runTest( name, allowedMismatch );
+      if ( !result )
+      {
+        appendToReport( name, checker.report() );
+      }
+      return result;
+    }
+
+    /**
+     * Appends some \a content to the test report.
+     *
+     * This should be used only for appending useful information when a test fails.
+     */
+    void appendToReport( const QString &testName, const QString &content )
+    {
+      QString testIdentifier;
+      if ( QTest::currentDataTag() )
+        testIdentifier = QStringLiteral( "%1 (%2: %3)" ).arg( testName, QTest::currentTestFunction(), QTest::currentDataTag() );
+      else
+        testIdentifier = QStringLiteral( "%1 (%2)" ).arg( testName, QTest::currentTestFunction() );
+      mReport += QStringLiteral( "<h2>%1</h2>\n" ).arg( testIdentifier );
+      mReport += content;
+    }
+
   private:
 
     /**
@@ -143,7 +215,15 @@ class TEST_EXPORT QgsTest : public QObject
 
       const QString reportFile = reportDir.filePath( "index.html" );
       QFile file( reportFile );
-      if ( file.open( QIODevice::WriteOnly | QIODevice::Append ) )
+
+      QFile::OpenMode mode = QIODevice::WriteOnly;
+      if ( qgetenv( "QGIS_CONTINUOUS_INTEGRATION_RUN" ) == QStringLiteral( "true" )
+           || qgetenv( "QGIS_APPEND_TO_TEST_REPORT" ) == QStringLiteral( "true" ) )
+        mode |= QIODevice::Append;
+      else
+        mode |= QIODevice::Truncate;
+
+      if ( file.open( mode ) )
       {
         QTextStream stream( &file );
         stream << QStringLiteral( "<h1>%1</h1>\n" ).arg( mName );
@@ -155,10 +235,6 @@ class TEST_EXPORT QgsTest : public QObject
       }
     }
 
-  protected:
-
-    QString mName;
-    QString mReport;
 };
 
 /**
