@@ -18,7 +18,7 @@
 
 #include "qgis_sip.h"
 #include "qgis_core.h"
-#include "qgsunittypes.h"
+#include "qgis.h"
 #include "qgstextbuffersettings.h"
 #include "qgstextbackgroundsettings.h"
 #include "qgstextshadowsettings.h"
@@ -41,14 +41,6 @@ class CORE_EXPORT QgsTextFormat
 {
   public:
 
-    //! Text orientation
-    enum TextOrientation
-    {
-      HorizontalOrientation, //!< Vertically oriented text
-      VerticalOrientation, //!< Horizontally oriented text
-      RotationBasedOrientation, //!< Horizontally or vertically oriented text based on rotation (only available for map labeling)
-    };
-
     /**
      * Default constructor for QgsTextFormat. Creates a text format initially
      * set to an invalid state (see isValid()).
@@ -189,11 +181,12 @@ class CORE_EXPORT QgsTextFormat
      * QgsTextRenderer::FONT_WORKAROUND_SCALE and then manually scale painter devices or calculations
      * based on the resultant font metrics. Failure to do so will result in poor quality text rendering
      * at small font sizes.
+     * \param isZeroSize will be set to true if the font is scaled down to a near 0 size, and nothing should be rendered. Not available in Python bindings.
      * \returns font with scaled size
      * \see font()
      * \see size()
      */
-    QFont scaledFont( const QgsRenderContext &context, double scaleFactor = 1.0 ) const;
+    QFont scaledFont( const QgsRenderContext &context, double scaleFactor = 1.0, bool *isZeroSize SIP_PYARGREMOVE = nullptr ) const;
 
     /**
      * Sets the font used for rendering text. Note that the size of the font
@@ -221,6 +214,58 @@ class CORE_EXPORT QgsTextFormat
      */
     void setNamedStyle( const QString &style );
 
+    /**
+     * Returns TRUE if the format is set to force a bold style.
+     *
+     * \warning Unlike setting a font's style via setNamedStyle(), this will ensure that a font is
+     * always rendered in bold regardless of whether the font family actually has a bold variant. A
+     * "faux bold" effect will be emulated, which may result in poor quality font rendering. For this
+     * reason it is greatly preferred to call setNamedStyle() instead.
+     *
+     * \see setForcedBold()
+     * \since QGIS 3.26
+     */
+    bool forcedBold() const;
+
+    /**
+     * Sets whether the format is set to force a bold style.
+     *
+     * \warning Unlike setting a font's style via setNamedStyle(), this will ensure that a font is
+     * always rendered in bold regardless of whether the font family actually has a bold variant. A
+     * "faux bold" effect will be emulated, which may result in poor quality font rendering. For this
+     * reason it is greatly preferred to call setNamedStyle() instead.
+     *
+     * \see forcedBold()
+     * \since QGIS 3.26
+     */
+    void setForcedBold( bool forced );
+
+    /**
+     * Returns TRUE if the format is set to force an italic style.
+     *
+     * \warning Unlike setting a font's style via setNamedStyle(), this will ensure that a font is
+     * always rendered in italic regardless of whether the font family actually has an italic variant. A
+     * "faux italic" slanted text effect will be emulated, which may result in poor quality font rendering. For this
+     * reason it is greatly preferred to call setNamedStyle() instead.
+     *
+     * \see setForcedItalic()
+     * \since QGIS 3.26
+     */
+    bool forcedItalic() const;
+
+    /**
+     * Sets whether the format is set to force an italic style.
+     *
+     * \warning Unlike setting a font's style via setNamedStyle(), this will ensure that a font is
+     * always rendered in italic regardless of whether the font family actually has an italic variant. A
+     * "faux italic" slanted text effect will be emulated, which may result in poor quality font rendering. For this
+     * reason it is greatly preferred to call setNamedStyle() instead.
+     *
+     * \see forcedItalic()
+     * \since QGIS 3.26
+     */
+    void setForcedItalic( bool forced );
+
     /**
      * Returns the list of font families to use when restoring the text format, in order of precedence.
      *
@@ -268,7 +313,7 @@ class CORE_EXPORT QgsTextFormat
      * \see setSizeUnit()
      * \see sizeMapUnitScale()
      */
-    QgsUnitTypes::RenderUnit sizeUnit() const;
+    Qgis::RenderUnit sizeUnit() const;
 
     /**
      * Sets the units for the size of rendered text.
@@ -277,7 +322,7 @@ class CORE_EXPORT QgsTextFormat
      * \see sizeUnit()
      * \see setSizeMapUnitScale()
      */
-    void setSizeUnit( QgsUnitTypes::RenderUnit unit );
+    void setSizeUnit( Qgis::RenderUnit unit );
 
     /**
      * Returns the map unit scale object for the size. This is only used if the
@@ -323,6 +368,34 @@ class CORE_EXPORT QgsTextFormat
      */
     void setOpacity( double opacity );
 
+    /**
+     * Returns the text's stretch factor.
+     *
+     * The stretch factor matches a condensed or expanded version of the font or applies a stretch
+     * transform that changes the width of all characters in the font by factor percent.
+     *
+     * For example, a factor of 150 results in all characters in the font being 1.5 times
+     * (ie. 150%) wider. The minimum stretch factor is 1, and the maximum stretch factor is 4000.
+     *
+     * \see setStretchFactor()
+     * \since QGIS 3.24
+     */
+    int stretchFactor() const;
+
+    /**
+     * Sets the text's stretch \a factor.
+     *
+     * The stretch factor matches a condensed or expanded version of the font or applies a stretch
+     * transform that changes the width of all characters in the font by factor percent.
+     *
+     * For example, setting \a factor to 150 results in all characters in the font being 1.5 times
+     * (ie. 150%) wider. The minimum stretch factor is 1, and the maximum stretch factor is 4000.
+     *
+     * \see stretchFactor()
+     * \since QGIS 3.24
+     */
+    void setStretchFactor( int factor );
+
     /**
      * Returns the blending mode used for drawing the text.
      * \see setBlendMode()
@@ -337,35 +410,62 @@ class CORE_EXPORT QgsTextFormat
     void setBlendMode( QPainter::CompositionMode mode );
 
     /**
-     * Returns the line height for text. This is a number between
-     * 0.0 and 10.0 representing the leading between lines as a
-     * multiplier of line height.
+     * Returns the line height for text.
+     *
+     * If lineHeightUnit() is QgsUnitTypes::RenderPercentage (the default), then this is a number representing
+     * the leading between lines as a multiplier of line height (where 0 - 1.0 represents 0 to 100% of text line height).
+     * Otherwise the line height is an absolute measurement in lineHeightUnit().
+     *
      * \see setLineHeight()
+     * \see lineHeightUnit()
      */
     double lineHeight() const;
 
     /**
      * Sets the line height for text.
-     * \param height a number between
-     * 0.0 and 10.0 representing the leading between lines as a
-     * multiplier of line height.
+     *
+     * If lineHeightUnit() is QgsUnitTypes::RenderPercentage (the default), then \a height is a number representing
+     * the leading between lines as a multiplier of line height (where 0 - 1.0 represents 0 to 100% of text line height).
+     * Otherwise \a height is an absolute measurement in lineHeightUnit().
+     *
      * \see lineHeight()
+     * \see setLineHeightUnit()
      */
     void setLineHeight( double height );
 
+    /**
+     * Returns the units for the line height for text.
+     *
+     * \see setLineHeightUnit()
+     * \see lineHeight()
+     *
+     * \since QGIS 3.28
+     */
+    Qgis::RenderUnit lineHeightUnit() const;
+
+    /**
+     * Sets the \a unit for the line height for text.
+     *
+     * \see lineHeightUnit()
+     * \see setLineHeight()
+     *
+     * \since QGIS 3.28
+     */
+    void setLineHeightUnit( Qgis::RenderUnit unit );
+
     /**
      * Returns the orientation of the text.
      * \see setOrientation()
      * \since QGIS 3.10
      */
-    TextOrientation orientation() const;
+    Qgis::TextOrientation orientation() const;
 
     /**
      * Sets the \a orientation for the text.
      * \see orientation()
      * \since QGIS 3.10
      */
-    void setOrientation( TextOrientation orientation );
+    void setOrientation( Qgis::TextOrientation orientation );
 
     /**
      * Returns the text capitalization style.
@@ -373,7 +473,7 @@ class CORE_EXPORT QgsTextFormat
      * \see setCapitalization()
      * \since QGIS 3.16
      */
-    QgsStringUtils::Capitalization capitalization() const;
+    Qgis::Capitalization capitalization() const;
 
     /**
      * Sets the text \a capitalization style.
@@ -381,7 +481,7 @@ class CORE_EXPORT QgsTextFormat
      * \see capitalization()
      * \since QGIS 3.16
      */
-    void setCapitalization( QgsStringUtils::Capitalization capitalization );
+    void setCapitalization( Qgis::Capitalization capitalization );
 
     /**
      * Returns TRUE if text should be treated as a HTML document and HTML tags should be used for formatting
