@@ -19,13 +19,18 @@
 #include "qgis_sip.h"
 #include "qgis_core.h"
 #include "qgstextblock.h"
-#include "qgstextformat.h"
+#include "qgsmapunitscale.h"
+#include "qgis.h"
 
 #include <QPicture>
 
 class QgsTextDocument;
+class QgsTextDocumentMetrics;
+class QgsTextFormat;
 class QgsRenderContext;
 
+class QFontMetricsF;
+
 /**
  * \class QgsTextRenderer
   * \ingroup core
@@ -37,58 +42,21 @@ class CORE_EXPORT QgsTextRenderer
 {
   public:
 
-    //! Draw mode to calculate width and height
-    enum DrawMode
-    {
-      Rect = 0, //!< Text within rectangle draw mode
-      Point, //!< Text at point of origin draw mode
-      Label, //!< Label-specific draw mode
-    };
-
-    //! Components of text
-    enum TextPart
-    {
-      Text = 0, //!< Text component
-      Buffer, //!< Buffer component
-      Background, //!< Background shape
-      Shadow, //!< Drop shadow
-    };
-
-    //! Horizontal alignment
-    enum HAlignment
-    {
-      AlignLeft = 0, //!< Left align
-      AlignCenter, //!< Center align
-      AlignRight, //!< Right align
-      AlignJustify, //!< Justify align
-    };
-
     /**
-     * Converts a Qt horizontal \a alignment flag to a QgsTextRenderer::HAlignment value.
+     * Converts a Qt horizontal \a alignment flag to a Qgis::TextHorizontalAlignment value.
      *
      * \see convertQtVAlignment()
      * \since QGIS 3.16
      */
-    static HAlignment convertQtHAlignment( Qt::Alignment alignment );
+    static Qgis::TextHorizontalAlignment convertQtHAlignment( Qt::Alignment alignment );
 
     /**
-     * Vertical alignment
-     * \since QGIS 3.16
-     */
-    enum VAlignment
-    {
-      AlignTop = 0, //!< Align to top
-      AlignVCenter, //!< Center align
-      AlignBottom, //!< Align to bottom
-    };
-
-    /**
-     * Converts a Qt vertical \a alignment flag to a QgsTextRenderer::VAlignment value.
+     * Converts a Qt vertical \a alignment flag to a Qgis::TextVerticalAlignment value.
      *
      * \see convertQtHAlignment()
      * \since QGIS 3.16
      */
-    static VAlignment convertQtVAlignment( Qt::Alignment alignment );
+    static Qgis::TextVerticalAlignment convertQtVAlignment( Qt::Alignment alignment );
 
     /**
      * Calculates pixel size (considering output size should be in pixel or map units, scale factors and optionally oversampling)
@@ -98,7 +66,7 @@ class CORE_EXPORT QgsTextRenderer
      * \param mapUnitScale a mapUnitScale clamper
      * \returns font pixel size
      */
-    static int sizeToPixel( double size, const QgsRenderContext &c, QgsUnitTypes::RenderUnit unit, const QgsMapUnitScale &mapUnitScale = QgsMapUnitScale() );
+    static int sizeToPixel( double size, const QgsRenderContext &c, Qgis::RenderUnit unit, const QgsMapUnitScale &mapUnitScale = QgsMapUnitScale() );
 
     // TODO QGIS 4.0 -- remove drawAsOutlines from below methods!
 
@@ -116,11 +84,48 @@ class CORE_EXPORT QgsTextRenderer
      * as of QGIS 3.4.3 and the text format should be set using QgsRenderContext::setTextRenderFormat() instead.
      * \param vAlignment vertical alignment (since QGIS 3.16)
      * \param flags text rendering flags (since QGIS 3.24)
+     * \param mode text layout mode. Only Qgis::TextLayoutMode::Rectangle, Qgis::TextLayoutMode::RectangleCapHeightBased and Qgis::TextLayoutMode::RectangleAscentBased are accepted (since QGIS 3.30)
+     *
+     * \see drawDocument(), which is more efficient if the text document and metrics have already been calculated.
      */
-    static void drawText( const QRectF &rect, double rotation, HAlignment alignment, const QStringList &textLines,
+    static void drawText( const QRectF &rect, double rotation, Qgis::TextHorizontalAlignment alignment, const QStringList &textLines,
                           QgsRenderContext &context, const QgsTextFormat &format,
-                          bool drawAsOutlines = true, VAlignment vAlignment = AlignTop,
-                          Qgis::TextRendererFlags flags = Qgis::TextRendererFlags() );
+                          bool drawAsOutlines = true, Qgis::TextVerticalAlignment vAlignment = Qgis::TextVerticalAlignment::Top,
+                          Qgis::TextRendererFlags flags = Qgis::TextRendererFlags(),
+                          Qgis::TextLayoutMode mode = Qgis::TextLayoutMode::Rectangle );
+
+    /**
+     * Draws a text document within a rectangle using the specified settings.
+     *
+     * Calling this method is more efficient than calling drawText() if the text document and metrics have already
+     * been calculated.
+     *
+     * \warning Unlike drawText(), this method does not automatically update data defined properties in the text \a format. This
+     * is the caller's responsibility to do, and must be done prior to generating the text \a document and \a metrics.
+     *
+     * \param rect destination rectangle for text
+     * \param format base text format
+     * \param document text document to draw
+     * \param metrics precalculated text metrics
+     * \param context destination render context
+     * \param horizontalAlignment horizontal alignment
+     * \param verticalAlignment vertical alignment
+     * \param rotation text rotation
+     * \param mode text layout mode. Only Qgis::TextLayoutMode::Rectangle, Qgis::TextLayoutMode::RectangleCapHeightBased and Qgis::TextLayoutMode::RectangleAscentBased are accepted.
+     * \param flags text rendering flags
+     *
+     * \since QGIS 3.30
+     */
+    static void drawDocument( const QRectF &rect,
+                              const QgsTextFormat &format,
+                              const QgsTextDocument &document,
+                              const QgsTextDocumentMetrics &metrics,
+                              QgsRenderContext &context,
+                              Qgis::TextHorizontalAlignment horizontalAlignment = Qgis::TextHorizontalAlignment::Left,
+                              Qgis::TextVerticalAlignment verticalAlignment = Qgis::TextVerticalAlignment::Top,
+                              double rotation = 0,
+                              Qgis::TextLayoutMode mode = Qgis::TextLayoutMode::Rectangle,
+                              Qgis::TextRendererFlags flags = Qgis::TextRendererFlags() );
 
     /**
      * Draws text at a point origin using the specified settings.
@@ -135,7 +140,7 @@ class CORE_EXPORT QgsTextRenderer
      * rendering and may result in side effects like misaligned text buffers. This setting is deprecated and has no effect
      * as of QGIS 3.4.3 and the text format should be set using QgsRenderContext::setTextRenderFormat() instead.
      */
-    static void drawText( QPointF point, double rotation, HAlignment alignment, const QStringList &textLines,
+    static void drawText( QPointF point, double rotation, Qgis::TextHorizontalAlignment alignment, const QStringList &textLines,
                           QgsRenderContext &context, const QgsTextFormat &format,
                           bool drawAsOutlines = true );
 
@@ -157,9 +162,9 @@ class CORE_EXPORT QgsTextRenderer
      *
      * \deprecated Private API only, will be removed in 4.0
      */
-    Q_DECL_DEPRECATED static void drawPart( const QRectF &rect, double rotation, HAlignment alignment, const QStringList &textLines,
+    Q_DECL_DEPRECATED static void drawPart( const QRectF &rect, double rotation, Qgis::TextHorizontalAlignment alignment, const QStringList &textLines,
                                             QgsRenderContext &context, const QgsTextFormat &format,
-                                            TextPart part, bool drawAsOutlines = true ) SIP_DEPRECATED;
+                                            Qgis::TextComponent part, bool drawAsOutlines = true ) SIP_DEPRECATED;
 
     /**
      * Draws a single component of rendered text using the specified settings.
@@ -179,9 +184,9 @@ class CORE_EXPORT QgsTextRenderer
      *
      * \deprecated Private API only, will be removed in 4.0
      */
-    Q_DECL_DEPRECATED static void drawPart( QPointF origin, double rotation, HAlignment alignment, const QStringList &textLines,
+    Q_DECL_DEPRECATED static void drawPart( QPointF origin, double rotation, Qgis::TextHorizontalAlignment alignment, const QStringList &textLines,
                                             QgsRenderContext &context, const QgsTextFormat &format,
-                                            TextPart part, bool drawAsOutlines = true ) SIP_DEPRECATED;
+                                            Qgis::TextComponent part, bool drawAsOutlines = true ) SIP_DEPRECATED;
 
     /**
      * Returns the font metrics for the given text \a format, when rendered
@@ -217,7 +222,7 @@ class CORE_EXPORT QgsTextRenderer
      * \param flags text renderer flags (since QGIS 3.24)
      * \param maxLineWidth maximum line width, in painter units. Used when the Qgis::TextRendererFlag::WrapLines flag is used (since QGIS 3.24)
      */
-    static double textHeight( const QgsRenderContext &context, const QgsTextFormat &format, const QStringList &textLines, DrawMode mode = Point,
+    static double textHeight( const QgsRenderContext &context, const QgsTextFormat &format, const QStringList &textLines, Qgis::TextLayoutMode mode = Qgis::TextLayoutMode::Point,
                               QFontMetricsF *fontMetrics = nullptr, Qgis::TextRendererFlags flags = Qgis::TextRendererFlags(), double maxLineWidth = 0 );
 
     /**
@@ -265,6 +270,13 @@ class CORE_EXPORT QgsTextRenderer
     {
       //! Block to render
       QgsTextBlock block;
+
+      //! Index of block
+      int blockIndex = 0;
+
+      //! Index of first fragment in block
+      int firstFragmentIndex = 0;
+
       //! Current origin point for painting (generally current painter rotation point)
       QPointF origin;
       //! Whether to translate the painter to supplied origin
@@ -290,7 +302,7 @@ class CORE_EXPORT QgsTextRenderer
       //! A ratio of native painter dpi and that of rendering context's painter
       double dpiRatio = 1.0;
       //! Horizontal alignment
-      HAlignment hAlign = AlignLeft;
+      Qgis::TextHorizontalAlignment hAlign = Qgis::TextHorizontalAlignment::Left;
 
       //! Any additional word spacing to apply while rendering component
       double extraWordSpacing = 0;
@@ -299,7 +311,7 @@ class CORE_EXPORT QgsTextRenderer
     };
 
     static double textWidth( const QgsRenderContext &context, const QgsTextFormat &format, const QgsTextDocument &document );
-    static double textHeight( const QgsRenderContext &context, const QgsTextFormat &format, const QgsTextDocument &document, DrawMode mode = Point );
+    static double textHeight( const QgsRenderContext &context, const QgsTextFormat &format, const QgsTextDocument &document, Qgis::TextLayoutMode mode = Qgis::TextLayoutMode::Point );
 
     /**
      * Draws a single component of rendered text using the specified settings.
@@ -308,17 +320,19 @@ class CORE_EXPORT QgsTextRenderer
      * \param alignment horizontal alignment
      * \param vAlignment vertical alignment
      * \param document text document to draw
+     * \param metrics document metrics
      * \param context render context
      * \param format text format
      * \param part component of text to draw. Note that Shadow parts cannot be drawn
      * individually and instead are drawn with their associated part (e.g., drawn together
      * with the text or background parts)
+     * \param mode layout mode
      * \note Not available in Python bindings
      * \since QGIS 3.14
      */
-    static void drawPart( const QRectF &rect, double rotation, HAlignment alignment, VAlignment vAlignment, const QgsTextDocument &document,
+    static void drawPart( const QRectF &rect, double rotation, Qgis::TextHorizontalAlignment alignment, Qgis::TextVerticalAlignment vAlignment, const QgsTextDocument &document, const QgsTextDocumentMetrics &metrics,
                           QgsRenderContext &context, const QgsTextFormat &format,
-                          TextPart part );
+                          Qgis::TextComponent part, Qgis::TextLayoutMode mode );
 
     /**
      * Draws a single component of rendered text using the specified settings.
@@ -326,28 +340,33 @@ class CORE_EXPORT QgsTextRenderer
      * \param rotation text rotation
      * \param alignment horizontal alignment
      * \param document document to draw
+     * \param metrics precalculated document metrics
      * \param context render context
      * \param format text format
      * \param part component of text to draw. Note that Shadow parts cannot be drawn
      * individually and instead are drawn with their associated part (e.g., drawn together
      * with the text or background parts)
+     * \param mode layout mode
      * \note Not available in Python bindings
      * \since QGIS 3.14
      */
-    static void drawPart( QPointF origin, double rotation, HAlignment alignment, const QgsTextDocument &document,
+    static void drawPart( QPointF origin, double rotation, Qgis::TextHorizontalAlignment alignment, const QgsTextDocument &document,
+                          const QgsTextDocumentMetrics &metrics,
                           QgsRenderContext &context, const QgsTextFormat &format,
-                          TextPart part );
+                          Qgis::TextComponent part,
+                          Qgis::TextLayoutMode mode );
 
     static double drawBuffer( QgsRenderContext &context,
                               const Component &component,
                               const QgsTextFormat &format,
-                              DrawMode mode );
+                              const QgsTextDocumentMetrics &metrics,
+                              Qgis::TextLayoutMode mode );
 
     static void drawBackground( QgsRenderContext &context,
                                 Component component,
                                 const QgsTextFormat &format,
-                                const QgsTextDocument &document,
-                                DrawMode mode = Rect );
+                                const QgsTextDocumentMetrics &metrics,
+                                Qgis::TextLayoutMode mode = Qgis::TextLayoutMode::Rectangle );
 
     static void drawShadow( QgsRenderContext &context,
                             const Component &component,
@@ -356,49 +375,50 @@ class CORE_EXPORT QgsTextRenderer
     static void drawMask( QgsRenderContext &context,
                           const Component &component,
                           const QgsTextFormat &format,
-                          DrawMode mode );
+                          const QgsTextDocumentMetrics &metrics,
+                          Qgis::TextLayoutMode mode );
 
     static void drawText( QgsRenderContext &context,
                           const Component &component,
                           const QgsTextFormat &format );
 
-    static void drawTextInternal( TextPart drawType,
+    static void drawTextInternal( Qgis::TextComponent drawType,
                                   QgsRenderContext &context,
                                   const QgsTextFormat &format,
                                   const Component &component,
                                   const QgsTextDocument &document,
-                                  const QFontMetricsF *fontMetrics,
-                                  HAlignment alignment,
-                                  VAlignment vAlignment,
-                                  DrawMode mode = Rect );
+                                  const QgsTextDocumentMetrics &metrics,
+                                  Qgis::TextHorizontalAlignment alignment,
+                                  Qgis::TextVerticalAlignment vAlignment,
+                                  Qgis::TextLayoutMode mode = Qgis::TextLayoutMode::Rectangle );
 
-    static QgsTextFormat::TextOrientation calculateRotationAndOrientationForComponent( const QgsTextFormat &format, const Component &component, double &rotation );
+    static Qgis::TextOrientation calculateRotationAndOrientationForComponent( const QgsTextFormat &format, const Component &component, double &rotation );
 
     static void calculateExtraSpacingForLineJustification( double spaceToDistribute, const QgsTextBlock &block, double &extraWordSpace, double &extraLetterSpace );
     static void applyExtraSpacingForLineJustification( QFont &font, double extraWordSpace, double extraLetterSpace );
 
     static void drawTextInternalHorizontal( QgsRenderContext &context,
                                             const QgsTextFormat &format,
-                                            TextPart drawType,
-                                            DrawMode mode,
+                                            Qgis::TextComponent drawType,
+                                            Qgis::TextLayoutMode mode,
                                             const Component &component,
                                             const QgsTextDocument &document,
+                                            const QgsTextDocumentMetrics &metrics,
                                             double fontScale,
-                                            const QFontMetricsF *fontMetrics,
-                                            HAlignment hAlignment,
-                                            VAlignment vAlignment,
+                                            Qgis::TextHorizontalAlignment hAlignment,
+                                            Qgis::TextVerticalAlignment vAlignment,
                                             double rotation );
 
     static void drawTextInternalVertical( QgsRenderContext &context,
                                           const QgsTextFormat &format,
-                                          TextPart drawType,
-                                          DrawMode mode,
+                                          Qgis::TextComponent drawType,
+                                          Qgis::TextLayoutMode mode,
                                           const Component &component,
                                           const QgsTextDocument &document,
+                                          const QgsTextDocumentMetrics &metrics,
                                           double fontScale,
-                                          const QFontMetricsF *fontMetrics,
-                                          HAlignment hAlignment,
-                                          VAlignment vAlignment,
+                                          Qgis::TextHorizontalAlignment hAlignment,
+                                          Qgis::TextVerticalAlignment vAlignment,
                                           double rotation );
 
     static double calculateScaleFactorForFormat( const QgsRenderContext &context, const QgsTextFormat &format );
