@@ -33,6 +33,7 @@ class QTextCodec;
 #include "qgsfeaturesink.h"
 #include "qgsfeaturesource.h"
 #include "qgsfeaturerequest.h"
+#include "qgsvectordataprovidertemporalcapabilities.h"
 
 typedef QList<int> QgsAttributeList SIP_SKIP;
 typedef QSet<int> QgsAttributeIds SIP_SKIP;
@@ -47,7 +48,7 @@ class QgsAbstractVectorLayerLabeling;
 
 /**
  * \ingroup core
- * This is the base class for vector data providers.
+ * \brief This is the base class for vector data providers.
  *
  * Data providers abstract the retrieval and writing (where supported)
  * of feature and attribute information from a spatial datasource.
@@ -85,9 +86,7 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider, public QgsFeat
       SimplifyGeometriesWithTopologicalValidation = 1 << 15, //!< Supports topological simplification of geometries on provider side according to a distance tolerance
       TransactionSupport = 1 << 16, //!< Supports transactions
       CircularGeometries = 1 << 17, //!< Supports circular geometry types (circularstring, compoundcurve, curvepolygon)
-      ChangeFeatures = 1 << 18, /**  Supports joint updates for attributes and geometry
-                                                               *  Providers supporting this should still define
-                                                               *  ChangeGeometries | ChangeAttributeValues */
+      ChangeFeatures = 1 << 18, //!< Supports joint updates for attributes and geometry. Providers supporting this should still define ChangeGeometries | ChangeAttributeValues.
       RenameAttributes = 1 << 19, //!< Supports renaming attributes (fields). Since QGIS 2.16
       FastTruncate = 1 << 20, //!< Supports fast truncation of the layer (removing all features). Since QGIS 3.0
       ReadLayerMetadata = 1 << 21, //!< Provider can read layer metadata from data store. Since QGIS 3.0. See QgsDataProvider::layerMetadata()
@@ -95,6 +94,8 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider, public QgsFeat
       CancelSupport = 1 << 23, //!< Supports interruption of pending queries from a separated thread. Since QGIS 3.2
       CreateRenderer = 1 << 24, //!< Provider can create feature renderers using backend-specific formatting information. Since QGIS 3.2. See QgsVectorDataProvider::createRenderer().
       CreateLabeling = 1 << 25, //!< Provider can set labeling settings using backend-specific formatting information. Since QGIS 3.6. See QgsVectorDataProvider::createLabeling().
+      ReloadData = 1 << 26, //!< Provider is able to force reload data
+      FeatureSymbology = 1 << 27, //!< Provider is able retrieve embedded symbology associated with individual features. Since QGIS 3.20.
     };
 
     Q_DECLARE_FLAGS( Capabilities, Capability )
@@ -104,25 +105,16 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider, public QgsFeat
                                            ChangeAttributeValues | ChangeGeometries | AddAttributes | DeleteAttributes |
                                            RenameAttributes;
 
-    /**
-     * Enumeration of feature count states
-     */
-    enum FeatureCountState
-    {
-      //! Feature count not yet computed
-      Uncounted = -2,
-      //! Provider returned an unknown feature count
-      UnknownCount = -1,
-    };
-
     /**
      * Constructor for a vector data provider.
      *
      * The \a uri argument specifies the uniform resource locator (URI) for the associated dataset.
      *
-     * Additional creation options are specified within the \a options value.
+     * Additional creation options are specified within the \a options value and since QGIS 3.16 creation flags are specified within the \a flags value.
      */
-    QgsVectorDataProvider( const QString &uri = QString(), const QgsDataProvider::ProviderOptions &providerOptions = QgsDataProvider::ProviderOptions() );
+    QgsVectorDataProvider( const QString &uri = QString(),
+                           const QgsDataProvider::ProviderOptions &providerOptions = QgsDataProvider::ProviderOptions(),
+                           QgsDataProvider::ReadFlags flags = QgsDataProvider::ReadFlags() );
 
     /**
      * Returns feature source object that can be used for querying provider's data. The returned feature source
@@ -157,13 +149,13 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider, public QgsFeat
     /**
      * Returns the geometry type which is returned by this layer
      */
-    QgsWkbTypes::Type wkbType() const override = 0;
+    Qgis::WkbType wkbType() const override = 0;
 
     /**
      * Number of features in the layer
-     * \returns long containing number of features
+     * \returns number of features
      */
-    long featureCount() const override = 0;
+    long long featureCount() const override = 0;
 
     /**
      * Returns TRUE if the layer does not contain any feature.
@@ -172,6 +164,25 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider, public QgsFeat
      */
     virtual bool empty() const;
 
+    /**
+     * Returns TRUE if the layer is a query (SQL) layer.
+     *
+     * \note this is simply a shortcut to check if the SqlQuery flag
+     *       is set.
+     *
+     *\see vectorLayerTypeFlags()
+     * \since QGIS 3.24
+     */
+    virtual bool isSqlQuery() const;
+
+    /**
+     * Returns the vector layer type flags.
+     *
+     * \see isSqlQuery()
+     * \since QGIS 3.24
+     */
+    virtual Qgis::VectorLayerTypeFlags vectorLayerTypeFlags() const;
+
     /**
      * Will always return FeatureAvailability::FeaturesAvailable or
      * FeatureAvailability::NoFeaturesAvailable.
@@ -197,7 +208,7 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider, public QgsFeat
      * Returns a short comment for the data that this provider is
      * providing access to (e.g. the comment for postgres table).
      */
-    virtual QString dataComment() const;
+    virtual QString dataComment() const override;
 
     /**
      * Returns the minimum value of an attribute
@@ -258,7 +269,8 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider, public QgsFeat
      */
     virtual void enumValues( int index, QStringList &enumList SIP_OUT ) const { Q_UNUSED( index ) enumList.clear(); }
 
-    bool addFeatures( QgsFeatureList &flist SIP_INOUT, QgsFeatureSink::Flags flags = nullptr ) override;
+    bool addFeatures( QgsFeatureList &flist SIP_INOUT, QgsFeatureSink::Flags flags = QgsFeatureSink::Flags() ) override;
+    QString lastError() const override;
 
     /**
      * Deletes one or more features from the provider. This requires the DeleteFeatures capability.
@@ -314,6 +326,8 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider, public QgsFeat
     /**
      * Changes attribute values of existing features. This should
      * succeed if the provider reports the ChangeAttributeValues capability.
+     * The method returns FALSE if the provider does not have ChangeAttributeValues
+     * capability or if any of the changes could not be successfully applied.
      * \param attr_map a map containing changed attributes
      * \returns TRUE in case of success and FALSE in case of failure
      */
@@ -392,11 +406,11 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider, public QgsFeat
 
     /**
      * Returns flags containing the supported capabilities
-        \note, some capabilities may change depending on whether
-        a spatial filter is active on this provider, so it may
-        be prudent to check this value per intended operation.
+     * \note, some capabilities may change depending on whether
+     * a spatial filter is active on this provider, so it may
+     * be prudent to check this value per intended operation.
      */
-    virtual QgsVectorDataProvider::Capabilities capabilities() const;
+    Q_INVOKABLE virtual QgsVectorDataProvider::Capabilities capabilities() const;
 
     /**
      *  Returns the above in friendly format.
@@ -571,12 +585,12 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider, public QgsFeat
 
     /**
      * Discover the available relations with the given layers.
-     * \param self the layer using this data provider.
+     * \param target the layer using this data provider.
      * \param layers the other layers.
      * \returns the list of N-1 relations from this provider.
      * \since QGIS 3.0
      */
-    virtual QList<QgsRelation> discoverRelations( const QgsVectorLayer *self, const QList<QgsVectorLayer *> &layers ) const;
+    virtual QList<QgsRelation> discoverRelations( const QgsVectorLayer *target, const QList<QgsVectorLayer *> &layers ) const;
 
     /**
      * Gets metadata, dependent on the provider type, that will be display in the metadata tab of the layer properties.
@@ -616,6 +630,9 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider, public QgsFeat
      */
     virtual void handlePostCloneOperations( QgsVectorDataProvider *source );
 
+    QgsVectorDataProviderTemporalCapabilities *temporalCapabilities() override;
+    const QgsVectorDataProviderTemporalCapabilities *temporalCapabilities() const override SIP_SKIP;
+
   signals:
 
     /**
@@ -686,6 +703,8 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider, public QgsFeat
     //! List of errors
     mutable QStringList mErrors;
 
+    std::unique_ptr< QgsVectorDataProviderTemporalCapabilities > mTemporalCapabilities;
+
     static QStringList sEncodings;
 
     /**
