@@ -23,11 +23,11 @@
 #include "qgis_sip.h"
 #include "qgsfields.h"
 #include "qgsfeedback.h"
-#include "qgstaskmanager.h"
 #include "qgsogrutils.h"
 #include "qgsrenderer.h"
 #include "qgsgeometryengine.h"
 #include "qgsfeaturesink.h"
+#include "qgsrendercontext.h"
 #include <ogr_api.h>
 
 class QgsSymbolLayer;
@@ -77,7 +77,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
       public:
         SetOption( const QString &docString, const QStringList &values, const QString &defaultValue, bool allowNone = false )
           : Option( docString, Set )
-          , values( qgis::listToSet( values ) )
+          , values( values.begin(), values.end() )
           , defaultValue( defaultValue )
           , allowNone( allowNone )
         {}
@@ -355,7 +355,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
         QgsVectorFileWriter::SymbologyExport symbologyExport = QgsVectorFileWriter::NoSymbology,
         double symbologyScale = 1.0,
         const QgsRectangle *filterExtent = nullptr,
-        QgsWkbTypes::Type overrideGeometryType = QgsWkbTypes::Unknown,
+        Qgis::WkbType overrideGeometryType = Qgis::WkbType::Unknown,
         bool forceMulti = false,
         bool includeZ = false,
         const QgsAttributeList &attributes = QgsAttributeList(),
@@ -438,7 +438,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
         QgsVectorFileWriter::SymbologyExport symbologyExport = QgsVectorFileWriter::NoSymbology,
         double symbologyScale = 1.0,
         const QgsRectangle *filterExtent = nullptr,
-        QgsWkbTypes::Type overrideGeometryType = QgsWkbTypes::Unknown,
+        Qgis::WkbType overrideGeometryType = Qgis::WkbType::Unknown,
         bool forceMulti = false,
         bool includeZ = false,
         const QgsAttributeList &attributes = QgsAttributeList(),
@@ -495,6 +495,9 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
         //! Attributes to export (empty means all unless skipAttributeCreation is set)
         QgsAttributeList attributes;
 
+        //! Attributes export names
+        QStringList attributesExportNames;
+
         //! Symbology to export
         QgsVectorFileWriter::SymbologyExport symbologyExport = NoSymbology;
 
@@ -508,7 +511,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
          * Set to a valid geometry type to override the default geometry type for the layer. This parameter
          * allows for conversion of geometryless tables to null geometries, etc.
         */
-        QgsWkbTypes::Type overrideGeometryType = QgsWkbTypes::Unknown;
+        Qgis::WkbType overrideGeometryType = Qgis::WkbType::Unknown;
 
         //! Sets to TRUE to force creation of multi* geometries
         bool forceMulti = false;
@@ -595,7 +598,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
     Q_DECL_DEPRECATED QgsVectorFileWriter( const QString &vectorFileName,
                                            const QString &fileEncoding,
                                            const QgsFields &fields,
-                                           QgsWkbTypes::Type geometryType,
+                                           Qgis::WkbType geometryType,
                                            const QgsCoordinateReferenceSystem &srs = QgsCoordinateReferenceSystem(),
                                            const QString &driverName = "GPKG",
                                            const QStringList &datasourceOptions = QStringList(),
@@ -635,7 +638,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
     Q_DECL_DEPRECATED QgsVectorFileWriter( const QString &vectorFileName,
                                            const QString &fileEncoding,
                                            const QgsFields &fields,
-                                           QgsWkbTypes::Type geometryType,
+                                           Qgis::WkbType geometryType,
                                            const QgsCoordinateReferenceSystem &srs,
                                            const QString &driverName,
                                            const QStringList &datasourceOptions,
@@ -671,7 +674,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
      */
     static QgsVectorFileWriter *create( const QString &fileName,
                                         const QgsFields &fields,
-                                        QgsWkbTypes::Type geometryType,
+                                        Qgis::WkbType geometryType,
                                         const QgsCoordinateReferenceSystem &srs,
                                         const QgsCoordinateTransformContext &transformContext,
                                         const QgsVectorFileWriter::SaveVectorOptions &options,
@@ -817,10 +820,10 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
     static QString convertCodecNameForEncodingOption( const QString &codecName );
 
     //! Checks whether there were any errors in constructor
-    QgsVectorFileWriter::WriterError hasError();
+    QgsVectorFileWriter::WriterError hasError() const;
 
     //! Retrieves error message
-    QString errorMessage();
+    QString errorMessage() const;
 
     bool addFeature( QgsFeature &feature, QgsFeatureSink::Flags flags = QgsFeatureSink::Flags() ) override;
     bool addFeatures( QgsFeatureList &features, QgsFeatureSink::Flags flags = QgsFeatureSink::Flags() ) override;
@@ -830,10 +833,10 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
      * Adds a \a feature to the currently opened data source, using the style from a specified \a renderer.
      * \since QGIS 3.0
      */
-    bool addFeatureWithStyle( QgsFeature &feature, QgsFeatureRenderer *renderer, QgsUnitTypes::DistanceUnit outputUnit = QgsUnitTypes::DistanceMeters );
+    bool addFeatureWithStyle( QgsFeature &feature, QgsFeatureRenderer *renderer, Qgis::DistanceUnit outputUnit = Qgis::DistanceUnit::Meters );
 
     //! \note not available in Python bindings
-    QMap<int, int> attrIdxToOgrIdx() { return mAttrIdxToOgrIdx; } SIP_SKIP
+    QMap<int, int> attrIdxToOgrIdx() const { return mAttrIdxToOgrIdx; } SIP_SKIP
 
     //! Close opened shapefile for writing
     ~QgsVectorFileWriter() override;
@@ -888,7 +891,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
      * Will drop M values and convert Z to 2.5D where required.
      * \note not available in Python bindings
      */
-    static OGRwkbGeometryType ogrTypeFromWkbType( QgsWkbTypes::Type type ) SIP_SKIP;
+    static OGRwkbGeometryType ogrTypeFromWkbType( Qgis::WkbType type ) SIP_SKIP;
 
     /**
      * Returns edition capabilities for an existing dataset name.
@@ -914,7 +917,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
 
   protected:
     //! \note not available in Python bindings
-    OGRGeometryH createEmptyGeometry( QgsWkbTypes::Type wkbType ) SIP_SKIP;
+    OGRGeometryH createEmptyGeometry( Qgis::WkbType wkbType ) SIP_SKIP;
 
     gdal::ogr_datasource_unique_ptr mDS;
     OGRLayerH mLayer = nullptr;
@@ -929,7 +932,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
     QTextCodec *mCodec = nullptr;
 
     //! Geometry type which is being used
-    QgsWkbTypes::Type mWkbType;
+    Qgis::WkbType mWkbType;
 
     //! Map attribute indizes to OGR field indexes
     QMap<int, int> mAttrIdxToOgrIdx;
@@ -955,7 +958,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
     {
       std::unique_ptr< QgsFeatureRenderer > renderer;
       QgsCoordinateReferenceSystem sourceCrs;
-      QgsWkbTypes::Type sourceWkbType = QgsWkbTypes::Unknown;
+      Qgis::WkbType sourceWkbType = Qgis::WkbType::Unknown;
       QgsFields sourceFields;
       QString providerType;
       long long featureCount = 0;
@@ -968,7 +971,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
       QgsRenderContext renderContext;
       bool shallTransform = false;
       QgsCoordinateReferenceSystem outputCrs;
-      QgsWkbTypes::Type destWkbType = QgsWkbTypes::Unknown;
+      Qgis::WkbType destWkbType = Qgis::WkbType::Unknown;
       QgsAttributeList attributes;
       QgsFields outputFields;
       QgsFeatureIterator sourceFeatureIterator;
@@ -1019,7 +1022,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
         QString *newLayer = nullptr ) SIP_DEPRECATED;
 
     void init( QString vectorFileName, QString fileEncoding, const QgsFields &fields,
-               QgsWkbTypes::Type geometryType, QgsCoordinateReferenceSystem srs,
+               Qgis::WkbType geometryType, QgsCoordinateReferenceSystem srs,
                const QString &driverName, QStringList datasourceOptions,
                QStringList layerOptions, QString *newFilename,
                QgsVectorFileWriter::FieldValueConverter *fieldValueConverter,
@@ -1044,8 +1047,8 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
 
     //! Writes features considering symbol level order
     QgsVectorFileWriter::WriterError exportFeaturesSymbolLevels( const PreparedWriterDetails &details, QgsFeatureIterator &fit, const QgsCoordinateTransform &ct, QString *errorMessage = nullptr );
-    double mmScaleFactor( double scale, QgsUnitTypes::RenderUnit symbolUnits, QgsUnitTypes::DistanceUnit mapUnits );
-    double mapUnitScaleFactor( double scale, QgsUnitTypes::RenderUnit symbolUnits, QgsUnitTypes::DistanceUnit mapUnits );
+    double mmScaleFactor( double scale, Qgis::RenderUnit symbolUnits, Qgis::DistanceUnit mapUnits );
+    double mapUnitScaleFactor( double scale, Qgis::RenderUnit symbolUnits, Qgis::DistanceUnit mapUnits );
 
     void startRender( QgsFeatureRenderer *sourceRenderer, const QgsFields &fields );
     void stopRender();
