@@ -20,8 +20,6 @@
 
 #include "ui_qgsvectortilebasicrendererwidget.h"
 
-#include "qgswkbtypes.h"
-
 #include <memory>
 #include <QSortFilterProxyModel>
 
@@ -56,7 +54,7 @@ class GUI_EXPORT QgsVectorTileBasicRendererWidget : public QgsMapLayerConfigWidg
     void apply() override;
 
   private slots:
-    void addStyle( QgsWkbTypes::GeometryType geomType );
+    void addStyle( Qgis::GeometryType geomType );
     void editStyle();
     void editStyleAtIndex( const QModelIndex &index );
     void removeStyle();
