@@ -37,7 +37,7 @@ from qgis.PyQt.QtWidgets import (QDialog, QComboBox,
                                  QTreeWidgetItem, QWidget)
 from qgis.PyQt.QtGui import QColor
 
-from qgis.core import (QgsApplication, QgsCoordinateReferenceSystem,
+from qgis.core import (Qgis, QgsApplication, QgsCoordinateReferenceSystem,
                        QgsCoordinateTransform, QgsGeometry, QgsPointXY,
                        QgsProviderRegistry, QgsSettings, QgsProject,
                        QgsRectangle)
@@ -82,7 +82,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         self.setWindowTitle(self.tr('MetaSearch'))
 
-        self.rubber_band = QgsRubberBand(self.map, True)  # True = a polygon
+        self.rubber_band = QgsRubberBand(self.map, Qgis.GeometryType.Polygon)
         self.rubber_band.setColor(QColor(255, 0, 0, 75))
         self.rubber_band.setWidth(5)
 
@@ -863,8 +863,8 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
                 cat = get_catalog_service(self.catalog_url,  # spellok
                                           catalog_type=self.catalog_type,
                                           timeout=self.timeout,
-                                          username=self.catalog_username,
-                                          password=self.catalog_password,
+                                          username=self.catalog_username or None,
+                                          password=self.catalog_password or None,
                                           auth=auth)
                 record = cat.get_record(identifier)
                 if cat.type == 'OGC API - Records':
@@ -957,8 +957,8 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             try:
                 self.catalog = get_catalog_service(
                     self.catalog_url, catalog_type=self.catalog_type,
-                    timeout=self.timeout, username=self.catalog_username,
-                    password=self.catalog_password, auth=auth)
+                    timeout=self.timeout, username=self.catalog_username or None,
+                    password=self.catalog_password or None, auth=auth)
                 return True
             except Exception as err:
                 msg = self.tr('Error connecting to service: {0}').format(err)
