@@ -160,7 +160,7 @@ class DBPlugin(QObject):
             md.deleteConnection(self.connectionName())
         except (AttributeError, QgsProviderConnectionException):
             settings = QgsSettings()
-            settings.beginGroup(u"/%s/%s" % (self.connectionSettingsKey(), self.connectionName()))
+            settings.beginGroup("/%s/%s" % (self.connectionSettingsKey(), self.connectionName()))
             settings.remove("")
 
         self.deleted.emit()
@@ -329,11 +329,11 @@ class Database(DbItemObject):
                     q = 1
                     while "_subq_%d_" % q in sql:
                         q += 1
-                    sql = u"SELECT %s AS _uid_,* FROM (%s\n) AS _subq_%d_" % (uniqueFct, sql, q)
+                    sql = "SELECT %s AS _uid_,* FROM (%s\n) AS _subq_%d_" % (uniqueFct, sql, q)
                     uniqueCol = "_uid_"
 
         uri = self.uri()
-        uri.setDataSource("", u"(%s\n)" % sql, geomCol, filter, uniqueCol)
+        uri.setDataSource("", "(%s\n)" % sql, geomCol, filter, uniqueCol)
         if avoidSelectById:
             uri.disableSelectAtId(True)
         provider = self.dbplugin().providerName()
@@ -348,7 +348,7 @@ class Database(DbItemObject):
     def registerSubPluginActions(self, mainWindow):
         # load plugins!
         try:
-            exec(u"from .%s.plugins import load" % self.dbplugin().typeName(), globals())
+            exec("from .%s.plugins import load" % self.dbplugin().typeName(), globals())
         except ImportError:
             pass
         else:
@@ -571,11 +571,10 @@ class Database(DbItemObject):
     def tables(self, schema=None, sys_tables=False):
         tables = self.connector.getTables(schema.name if schema else None, sys_tables)
         if tables is not None:
-            ret = []
-            for t in tables:
-                table = self.tablesFactory(t, self, schema)
-                ret.append(table)
-
+            ret = [
+                self.tablesFactory(t, self, schema)
+                for t in tables
+            ]
         return ret
 
     def createTable(self, table, fields, schema=None):
@@ -585,6 +584,13 @@ class Database(DbItemObject):
 
         ret = self.connector.createTable((schema, table), field_defs, pk_name)
         if ret is not False:
+            # Add comments if any, because definition does not include
+            # the comment
+            for f in fields:
+                if f.comment:
+                    self.connector.updateTableColumn(
+                        (schema, table), f.name, comment=f.comment
+                    )
             self.refresh()
         return ret
 
@@ -754,7 +760,7 @@ class Table(DbItemObject):
 
     def mimeUri(self):
         layerType = "raster" if self.type == Table.RasterType else "vector"
-        return u"%s:%s:%s:%s" % (layerType, self.database().dbplugin().providerName(), self.name, self.uri().uri(False))
+        return "%s:%s:%s:%s" % (layerType, self.database().dbplugin().providerName(), self.name, self.uri().uri(False))
 
     def toMapLayer(self, geometryType=None, crs=None):
         provider = self.database().dbplugin().providerName()
@@ -770,6 +776,9 @@ class Table(DbItemObject):
             return QgsRasterLayer(uri, self.name, provider)
         return QgsVectorLayer(uri, self.name, provider)
 
+    def geometryType(self):
+        pass
+
     def getValidQgisUniqueFields(self, onlyOne=False):
         """ list of fields valid to load the table as layer in QGIS canvas.
                 QGIS automatically search for a valid unique field, so it's
@@ -1225,8 +1234,8 @@ class TableField(TableSubItemObject):
 
     def type2String(self):
         if self.modifier is None or self.modifier == -1:
-            return u"%s" % self.dataType
-        return u"%s (%s)" % (self.dataType, self.modifier)
+            return "%s" % self.dataType
+        return "%s (%s)" % (self.dataType, self.modifier)
 
     def default2String(self):
         if not self.hasDefault:
@@ -1241,9 +1250,9 @@ class TableField(TableSubItemObject):
         name = quoteIdFunc(self.name)
         not_null = "NOT NULL" if self.notNull else ""
 
-        txt = u"%s %s %s" % (name, self.type2String(), not_null)
+        txt = "%s %s %s" % (name, self.type2String(), not_null)
         if self.hasDefault:
-            txt += u" DEFAULT %s" % self.default2String()
+            txt += " DEFAULT %s" % self.default2String()
         return txt
 
     def getComment(self):
@@ -1361,7 +1370,7 @@ class TableTrigger(TableSubItemObject):
         self.name = self.function = None
 
     def type2String(self):
-        trig_type = u''
+        trig_type = ''
         trig_type += "Before " if self.type & TableTrigger.TypeBefore else "After "
         if self.type & TableTrigger.TypeInsert:
             trig_type += "INSERT "
