@@ -20,9 +20,12 @@ email                : brush.tyler@gmail.com
  ***************************************************************************/
 """
 
-from PyQt4.QtCore import SIGNAL
-
-from ..data_model import TableDataModel, SqlResultModel
+from qgis.core import QgsMessageLog
+from ..plugin import BaseError
+from ..data_model import (TableDataModel,
+                          SqlResultModel,
+                          SqlResultModelAsync,
+                          SqlResultModelTask)
 
 
 class PGTableDataModel(TableDataModel):
@@ -36,33 +39,33 @@ class PGTableDataModel(TableDataModel):
             if self.table.rowCount is None:
                 return
 
-        self.connect(self.table, SIGNAL("aboutToChange"), self._deleteCursor)
+        self.table.aboutToChange.connect(self._deleteCursor)
         self._createCursor()
 
     def _createCursor(self):
-        fields_txt = u", ".join(self.fields)
+        fields_txt = ", ".join(self.fields)
         table_txt = self.db.quoteId((self.table.schemaName(), self.table.name))
 
         self.cursor = self.db._get_cursor()
-        sql = u"SELECT %s FROM %s" % (fields_txt, table_txt)
+        sql = "SELECT %s FROM %s" % (fields_txt, table_txt)
         self.db._execute(self.cursor, sql)
 
     def _sanitizeTableField(self, field):
         # get fields, ignore geometry columns
         if field.dataType.lower() == "geometry":
-            return u"CASE WHEN %(fld)s IS NULL THEN NULL ELSE GeometryType(%(fld)s) END AS %(fld)s" % {
+            return "CASE WHEN %(fld)s IS NULL THEN NULL ELSE GeometryType(%(fld)s) END AS %(fld)s" % {
                 'fld': self.db.quoteId(field.name)}
         elif field.dataType.lower() == "raster":
-            return u"CASE WHEN %(fld)s IS NULL THEN NULL ELSE 'RASTER' END AS %(fld)s" % {
+            return "CASE WHEN %(fld)s IS NULL THEN NULL ELSE 'RASTER' END AS %(fld)s" % {
                 'fld': self.db.quoteId(field.name)}
-        return u"%s::text" % self.db.quoteId(field.name)
+        return "%s::text" % self.db.quoteId(field.name)
 
     def _deleteCursor(self):
         self.db._close_cursor(self.cursor)
         self.cursor = None
 
     def __del__(self):
-        self.disconnect(self.table, SIGNAL("aboutToChange"), self._deleteCursor)
+        self.table.aboutToChange.disconnect(self._deleteCursor)
         self._deleteCursor()
         pass  # print "PGTableModel.__del__"
 
@@ -80,5 +83,34 @@ class PGTableDataModel(TableDataModel):
         self.fetchedFrom = row_start
 
 
+class PGSqlResultModelTask(SqlResultModelTask):
+
+    def __init__(self, db, sql, parent):
+        super().__init__(db, sql, parent)
+
+    def run(self):
+        try:
+            self.model = PGSqlResultModel(self.db, self.sql, None)
+        except BaseError as e:
+            self.error = e
+            QgsMessageLog.logMessage(e.msg)
+            return False
+        return True
+
+    def cancel(self):
+        self.db.connector.cancel()
+        SqlResultModelTask.cancel(self)
+
+
+class PGSqlResultModelAsync(SqlResultModelAsync):
+
+    def __init__(self, db, sql, parent):
+        super().__init__()
+
+        self.task = PGSqlResultModelTask(db, sql, parent)
+        self.task.taskCompleted.connect(self.modelDone)
+        self.task.taskTerminated.connect(self.modelDone)
+
+
 class PGSqlResultModel(SqlResultModel):
     pass
