@@ -43,7 +43,7 @@ current_path = os.path.dirname(__file__)
 def load(db, mainwindow):
     # check whether the selected database supports topology
     # (search for topology.topology)
-    sql = u"""SELECT count(*)
+    sql = """SELECT count(*)
                 FROM pg_class AS cls JOIN pg_namespace AS nsp ON nsp.oid = cls.relnamespace
                 WHERE cls.relname = 'topology' AND nsp.nspname = 'topology'"""
     res = db.executeSql(sql)
@@ -72,24 +72,24 @@ def run(item, action, mainwindow):
     isTopoSchema = False
 
     if not hasattr(item, 'schema'):
-        mainwindow.infoBar.pushMessage("Invalid topology", u'Select a topology schema to continue.', Qgis.Info,
+        mainwindow.infoBar.pushMessage("Invalid topology", 'Select a topology schema to continue.', Qgis.Info,
                                        mainwindow.iface.messageTimeout())
         return False
 
     if item.schema() is not None:
-        sql = u"SELECT srid FROM topology.topology WHERE name = %s" % quoteStr(item.schema().name)
+        sql = "SELECT srid FROM topology.topology WHERE name = %s" % quoteStr(item.schema().name)
         res = db.executeSql(sql)
         isTopoSchema = len(res) > 0
 
     if not isTopoSchema:
         mainwindow.infoBar.pushMessage("Invalid topology",
-                                       u'Schema "{0}" is not registered in topology.topology.'.format(
+                                       'Schema "{0}" is not registered in topology.topology.'.format(
                                            item.schema().name), Qgis.Warning,
                                        mainwindow.iface.messageTimeout())
         return False
 
     if (res[0][0] < 0):
-        mainwindow.infoBar.pushMessage("WARNING", u'Topology "{0}" is registered as having a srid of {1} in topology.topology, we will assume 0 (for unknown)'.format(item.schema().name, res[0]), Qgis.Warning, mainwindow.iface.messageTimeout())
+        mainwindow.infoBar.pushMessage("WARNING", 'Topology "{0}" is registered as having a srid of {1} in topology.topology, we will assume 0 (for unknown)'.format(item.schema().name, res[0]), Qgis.Warning, mainwindow.iface.messageTimeout())
         toposrid = '0'
     else:
         toposrid = str(res[0][0])
@@ -114,37 +114,37 @@ def run(item, action, mainwindow):
         uri.setDataSource(toponame, 'face', 'mbr', '', 'face_id')
         uri.setSrid(toposrid)
         uri.setWkbType(QgsWkbTypes.Polygon)
-        layerFaceMbr = QgsVectorLayer(uri.uri(False), u'%s.face_mbr' % toponame, provider)
+        layerFaceMbr = QgsVectorLayer(uri.uri(False), '%s.face_mbr' % toponame, provider)
         layerFaceMbr.loadNamedStyle(os.path.join(template_dir, 'face_mbr.qml'))
 
         face_extent = layerFaceMbr.extent()
 
         # face geometry
-        sql = u'SELECT face_id, mbr, topology.ST_GetFaceGeometry(%s,' \
+        sql = 'SELECT face_id, mbr, topology.ST_GetFaceGeometry(%s,' \
               'face_id)::geometry(polygon, %s) as geom ' \
               'FROM %s.face WHERE face_id > 0' % \
               (quoteStr(toponame), toposrid, quoteId(toponame))
-        uri.setDataSource('', u'(%s\n)' % sql, 'geom', '', 'face_id')
+        uri.setDataSource('', '(%s\n)' % sql, 'geom', '', 'face_id')
         uri.setParam('bbox', 'mbr')
         uri.setParam('checkPrimaryKeyUnicity', '0')
         uri.setSrid(toposrid)
         uri.setWkbType(QgsWkbTypes.Polygon)
-        layerFaceGeom = QgsVectorLayer(uri.uri(False), u'%s.face' % toponame, provider)
+        layerFaceGeom = QgsVectorLayer(uri.uri(False), '%s.face' % toponame, provider)
         layerFaceGeom.setExtent(face_extent)
         layerFaceGeom.loadNamedStyle(os.path.join(template_dir, 'face.qml'))
 
         # face_seed
-        sql = u'SELECT face_id, mbr, ST_PointOnSurface(' \
+        sql = 'SELECT face_id, mbr, ST_PointOnSurface(' \
               'topology.ST_GetFaceGeometry(%s,' \
               'face_id))::geometry(point, %s) as geom ' \
               'FROM %s.face WHERE face_id > 0' % \
               (quoteStr(toponame), toposrid, quoteId(toponame))
-        uri.setDataSource('', u'(%s)' % sql, 'geom', '', 'face_id')
+        uri.setDataSource('', '(%s)' % sql, 'geom', '', 'face_id')
         uri.setParam('bbox', 'mbr')
         uri.setParam('checkPrimaryKeyUnicity', '0')
         uri.setSrid(toposrid)
         uri.setWkbType(QgsWkbTypes.Point)
-        layerFaceSeed = QgsVectorLayer(uri.uri(False), u'%s.face_seed' % toponame, provider)
+        layerFaceSeed = QgsVectorLayer(uri.uri(False), '%s.face_seed' % toponame, provider)
         layerFaceSeed.setExtent(face_extent)
         layerFaceSeed.loadNamedStyle(os.path.join(template_dir, 'face_seed.qml'))
 
@@ -157,7 +157,7 @@ def run(item, action, mainwindow):
         uri.removeParam('bbox')
         uri.setSrid(toposrid)
         uri.setWkbType(QgsWkbTypes.Point)
-        layerNode = QgsVectorLayer(uri.uri(False), u'%s.node' % toponame, provider)
+        layerNode = QgsVectorLayer(uri.uri(False), '%s.node' % toponame, provider)
         layerNode.loadNamedStyle(os.path.join(template_dir, 'node.qml'))
         node_extent = layerNode.extent()
 
@@ -166,7 +166,7 @@ def run(item, action, mainwindow):
         uri.setSrid(toposrid)
         uri.setWkbType(QgsWkbTypes.Point)
         uri.removeParam('bbox')
-        layerNodeLabel = QgsVectorLayer(uri.uri(False), u'%s.node_id' % toponame, provider)
+        layerNodeLabel = QgsVectorLayer(uri.uri(False), '%s.node_id' % toponame, provider)
         layerNodeLabel.setExtent(node_extent)
         layerNodeLabel.loadNamedStyle(os.path.join(template_dir, 'node_label.qml'))
 
@@ -177,7 +177,7 @@ def run(item, action, mainwindow):
         uri.setSrid(toposrid)
         uri.setWkbType(QgsWkbTypes.LineString)
         uri.removeParam('bbox')
-        layerEdge = QgsVectorLayer(uri.uri(False), u'%s.edge' % toponame, provider)
+        layerEdge = QgsVectorLayer(uri.uri(False), '%s.edge' % toponame, provider)
         edge_extent = layerEdge.extent()
 
         # directed edge
@@ -185,7 +185,7 @@ def run(item, action, mainwindow):
         uri.setSrid(toposrid)
         uri.setWkbType(QgsWkbTypes.LineString)
         uri.removeParam('bbox')
-        layerDirectedEdge = QgsVectorLayer(uri.uri(False), u'%s.directed_edge' % toponame, provider)
+        layerDirectedEdge = QgsVectorLayer(uri.uri(False), '%s.directed_edge' % toponame, provider)
         layerDirectedEdge.setExtent(edge_extent)
         layerDirectedEdge.loadNamedStyle(os.path.join(template_dir, 'edge.qml'))
 
@@ -194,7 +194,7 @@ def run(item, action, mainwindow):
         uri.setSrid(toposrid)
         uri.setWkbType(QgsWkbTypes.LineString)
         uri.removeParam('bbox')
-        layerEdgeLabel = QgsVectorLayer(uri.uri(False), u'%s.edge_id' % toponame, provider)
+        layerEdgeLabel = QgsVectorLayer(uri.uri(False), '%s.edge_id' % toponame, provider)
         layerEdgeLabel.setExtent(edge_extent)
         layerEdgeLabel.loadNamedStyle(os.path.join(template_dir, 'edge_label.qml'))
 
@@ -203,7 +203,7 @@ def run(item, action, mainwindow):
         uri.setSrid(toposrid)
         uri.setWkbType(QgsWkbTypes.LineString)
         uri.removeParam('bbox')
-        layerFaceLeft = QgsVectorLayer(uri.uri(False), u'%s.face_left' % toponame, provider)
+        layerFaceLeft = QgsVectorLayer(uri.uri(False), '%s.face_left' % toponame, provider)
         layerFaceLeft.setExtent(edge_extent)
         layerFaceLeft.loadNamedStyle(os.path.join(template_dir, 'face_left.qml'))
 
@@ -212,7 +212,7 @@ def run(item, action, mainwindow):
         uri.setSrid(toposrid)
         uri.setWkbType(QgsWkbTypes.LineString)
         uri.removeParam('bbox')
-        layerFaceRight = QgsVectorLayer(uri.uri(False), u'%s.face_right' % toponame, provider)
+        layerFaceRight = QgsVectorLayer(uri.uri(False), '%s.face_right' % toponame, provider)
         layerFaceRight.setExtent(edge_extent)
         layerFaceRight.loadNamedStyle(os.path.join(template_dir, 'face_right.qml'))
 
@@ -221,7 +221,7 @@ def run(item, action, mainwindow):
         uri.setSrid(toposrid)
         uri.setWkbType(QgsWkbTypes.LineString)
         uri.removeParam('bbox')
-        layerNextLeft = QgsVectorLayer(uri.uri(False), u'%s.next_left' % toponame, provider)
+        layerNextLeft = QgsVectorLayer(uri.uri(False), '%s.next_left' % toponame, provider)
         layerNextLeft.setExtent(edge_extent)
         layerNextLeft.loadNamedStyle(os.path.join(template_dir, 'next_left.qml'))
 
@@ -230,7 +230,7 @@ def run(item, action, mainwindow):
         uri.setSrid(toposrid)
         uri.setWkbType(QgsWkbTypes.LineString)
         uri.removeParam('bbox')
-        layerNextRight = QgsVectorLayer(uri.uri(False), u'%s.next_right' % toponame, provider)
+        layerNextRight = QgsVectorLayer(uri.uri(False), '%s.next_right' % toponame, provider)
         layerNextRight.setExtent(edge_extent)
         layerNextRight.loadNamedStyle(os.path.join(template_dir, 'next_right.qml'))
 
@@ -246,25 +246,25 @@ def run(item, action, mainwindow):
 
         # Organize layers in groups
 
-        groupFaces = QgsLayerTreeGroup(u'Faces')
+        groupFaces = QgsLayerTreeGroup('Faces')
         for layer in faceLayers:
             nodeLayer = groupFaces.addLayer(layer)
             nodeLayer.setItemVisibilityChecked(False)
             nodeLayer.setExpanded(False)
 
-        groupNodes = QgsLayerTreeGroup(u'Nodes')
+        groupNodes = QgsLayerTreeGroup('Nodes')
         for layer in nodeLayers:
             nodeLayer = groupNodes.addLayer(layer)
             nodeLayer.setItemVisibilityChecked(False)
             nodeLayer.setExpanded(False)
 
-        groupEdges = QgsLayerTreeGroup(u'Edges')
+        groupEdges = QgsLayerTreeGroup('Edges')
         for layer in edgeLayers:
             nodeLayer = groupEdges.addLayer(layer)
             nodeLayer.setItemVisibilityChecked(False)
             nodeLayer.setExpanded(False)
 
-        supergroup = QgsLayerTreeGroup(u'Topology "%s"' % toponame)
+        supergroup = QgsLayerTreeGroup('Topology "%s"' % toponame)
         supergroup.insertChildNodes(-1, [groupFaces, groupNodes, groupEdges])
 
         layerTree = QgsProject.instance().layerTreeRoot()
