@@ -22,8 +22,9 @@ The content of this file is based on
  ***************************************************************************/
 """
 
-from PyQt4.QtCore import Qt, SIGNAL
-from PyQt4.QtGui import QDialog, QMessageBox, QApplication
+from qgis.PyQt.QtCore import Qt
+from qgis.PyQt.QtWidgets import QDialog, QMessageBox, QApplication
+from qgis.utils import OverrideCursor
 
 from .db_plugins.plugin import DbError
 from .dlg_db_error import DlgDbError
@@ -40,9 +41,9 @@ class DlgCreateIndex(QDialog, Ui_Dialog):
         self.db = self.table.database() if self.table and self.table.database() else db
         self.setupUi(self)
 
-        self.connect(self.buttonBox, SIGNAL("accepted()"), self.createIndex)
+        self.buttonBox.accepted.connect(self.createIndex)
 
-        self.connect(self.cboColumn, SIGNAL("currentIndexChanged(int)"), self.columnChanged)
+        self.cboColumn.currentIndexChanged.connect(self.columnChanged)
         self.populateColumns()
 
     def populateColumns(self):
@@ -51,23 +52,21 @@ class DlgCreateIndex(QDialog, Ui_Dialog):
             self.cboColumn.addItem(fld.name)
 
     def columnChanged(self):
-        self.editName.setText(u"idx_%s_%s" % (self.table.name, self.cboColumn.currentText()))
+        self.editName.setText("idx_%s_%s" % (self.table.name, self.cboColumn.currentText()))
 
     def createIndex(self):
         idx = self.getIndex()
         if idx.name == "":
-            QMessageBox.critical(self, self.tr("Error"), self.tr("Please enter some name for the index"))
+            QMessageBox.critical(self, self.tr("Error"), self.tr("Please enter a name for the index."))
             return
 
         # now create the index
-        QApplication.setOverrideCursor(Qt.WaitCursor)
-        try:
-            self.table.addIndex(idx)
-        except DbError as e:
-            DlgDbError.showError(e, self)
-            return
-        finally:
-            QApplication.restoreOverrideCursor()
+        with OverrideCursor(Qt.WaitCursor):
+            try:
+                self.table.addIndex(idx)
+            except DbError as e:
+                DlgDbError.showError(e, self)
+                return
 
         self.accept()
 
