@@ -19,9 +19,11 @@ email                : brush.tyler@gmail.com
  *                                                                         *
  ***************************************************************************/
 """
+from builtins import str
 
-from PyQt4.QtCore import Qt, SIGNAL
-from PyQt4.QtGui import QTextBrowser, QApplication
+from qgis.PyQt.QtCore import Qt
+from qgis.PyQt.QtWidgets import QTextBrowser, QApplication
+from qgis.utils import OverrideCursor
 
 from .db_plugins.plugin import BaseError, DbError, DBPlugin, Schema, Table
 from .dlg_db_error import DlgDbError
@@ -39,22 +41,19 @@ class InfoViewer(QTextBrowser):
         self._clear()
         self._showPluginInfo()
 
-        self.connect(self, SIGNAL("anchorClicked(const QUrl&)"), self._linkClicked)
+        self.anchorClicked.connect(self._linkClicked)
 
     def _linkClicked(self, url):
         if self.item is None:
             return
 
         if url.scheme() == "action":
-            QApplication.setOverrideCursor(Qt.WaitCursor)
-            try:
-                if self.item.runAction(url.path()):
-                    self.refresh()
-            except BaseError as e:
-                DlgDbError.showError(e, self)
-                return
-            finally:
-                QApplication.restoreOverrideCursor()
+            with OverrideCursor(Qt.WaitCursor):
+                try:
+                    if self.item.runAction(url.path()):
+                        self.refresh()
+                except BaseError as e:
+                    DlgDbError.showError(e, self)
 
     def refresh(self):
         self.setDirty(True)
@@ -77,16 +76,16 @@ class InfoViewer(QTextBrowser):
             return
 
         self.item = item
-        self.connect(self.item, SIGNAL('aboutToChange'), self.setDirty)
+        item.aboutToChange.connect(self.setDirty)
 
     def setDirty(self, val=True):
         self.dirty = val
 
     def _clear(self):
         if self.item is not None:
-            ## skip exception on RuntimeError fixes #6892
+            # skip exception on RuntimeError fixes #6892
             try:
-                self.disconnect(self.item, SIGNAL('aboutToChange'), self.setDirty)
+                self.item.aboutToChange.disconnect(self.setDirty)
             except RuntimeError:
                 pass
         self.item = None
@@ -98,14 +97,14 @@ class InfoViewer(QTextBrowser):
     def _showPluginInfo(self):
         from .db_plugins import getDbPluginErrors
 
-        html = u'<div style="background-color:#ffffcc;"><h1>&nbsp;' + self.tr("DB Manager") + '</h1></div>'
+        html = '<div style="background-color:#ffffcc;"><h1>&nbsp;' + self.tr("DB Manager") + '</h1></div>'
         html += '<div style="margin-left:8px;">'
         for msg in getDbPluginErrors():
-            html += u"<p>%s" % msg
+            html += "<p>%s" % msg
         self.setHtml(html)
 
     def _showDatabaseInfo(self, connection):
-        html = u'<div style="background-color:#ccffcc;"><h1>&nbsp;%s</h1></div>' % connection.connectionName()
+        html = '<div style="background-color:#ccffcc;"><h1>&nbsp;%s</h1></div>' % connection.connectionName()
         html += '<div style="margin-left:8px;">'
         try:
             if connection.database() is None:
@@ -113,37 +112,37 @@ class InfoViewer(QTextBrowser):
             else:
                 html += connection.database().info().toHtml()
         except DbError as e:
-            html += u'<p style="color:red">%s</p>' % unicode(e).replace('\n', '<br>')
+            html += '<p style="color:red">%s</p>' % str(e).replace('\n', '<br>')
         html += '</div>'
         self.setHtml(html)
 
     def _showSchemaInfo(self, schema):
-        html = u'<div style="background-color:#ffcccc;"><h1>&nbsp;%s</h1></div>' % schema.name
+        html = '<div style="background-color:#ffcccc;"><h1>&nbsp;%s</h1></div>' % schema.name
         html += '<div style="margin-left:8px;">'
         try:
             html += schema.info().toHtml()
         except DbError as e:
-            html += u'<p style="color:red">%s</p>' % unicode(e).replace('\n', '<br>')
+            html += '<p style="color:red">%s</p>' % str(e).replace('\n', '<br>')
         html += "</div>"
         self.setHtml(html)
 
     def _showTableInfo(self, table):
-        html = u'<div style="background-color:#ccccff"><h1>&nbsp;%s</h1></div>' % table.name
+        html = '<div style="background-color:#ccccff"><h1>&nbsp;%s</h1></div>' % table.name
         html += '<div style="margin-left:8px;">'
         try:
             html += table.info().toHtml()
         except DbError as e:
-            html += u'<p style="color:red">%s</p>' % unicode(e).replace('\n', '<br>')
+            html += '<p style="color:red">%s</p>' % str(e).replace('\n', '<br>')
         html += '</div>'
         self.setHtml(html)
         return True
 
     def setHtml(self, html):
         # convert special tags :)
-        html = unicode(html).replace('<warning>', '<img src=":/db_manager/warning">&nbsp;&nbsp; ')
+        html = str(html).replace('<warning>', '<img src=":/db_manager/warning">&nbsp;&nbsp; ')
 
         # add default style
-        html = u"""
+        html = """
 <!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
 <html>
 <head>
