@@ -20,11 +20,13 @@ email                : brush.tyler@gmail.com
  ***************************************************************************/
 """
 
-from PyQt4.QtCore import Qt, QSettings, QTimer, SIGNAL
-from PyQt4.QtGui import QColor, QApplication, QCursor
+from qgis.PyQt.QtCore import Qt, QTimer
+from qgis.PyQt.QtGui import QColor, QCursor
+from qgis.PyQt.QtWidgets import QApplication
 
-from qgis.gui import QgsMapCanvas, QgsMapCanvasLayer, QgsMessageBar
-from qgis.core import QgsVectorLayer, QgsMapLayerRegistry
+from qgis.gui import QgsMapCanvas, QgsMessageBar
+from qgis.core import Qgis, QgsVectorLayer, QgsProject, QgsSettings
+from qgis.utils import OverrideCursor
 
 from .db_plugins.plugin import Table
 
@@ -32,20 +34,19 @@ from .db_plugins.plugin import Table
 class LayerPreview(QgsMapCanvas):
 
     def __init__(self, parent=None):
-        QgsMapCanvas.__init__(self, parent)
+        super(LayerPreview, self).__init__(parent)
         self.parent = parent
         self.setCanvasColor(QColor(255, 255, 255))
 
         self.item = None
         self.dirty = False
-        self.currentLayer = None
+        self.current_layer = None
 
         # reuse settings from QGIS
-        settings = QSettings()
+        settings = QgsSettings()
         self.enableAntiAliasing(settings.value("/qgis/enable_anti_aliasing", False, type=bool))
-        action = settings.value("/qgis/wheel_action", 0, type=float)
         zoomFactor = settings.value("/qgis/zoom_factor", 2, type=float)
-        self.setWheelAction(QgsMapCanvas.WheelAction(action), zoomFactor)
+        self.setWheelFactor(zoomFactor)
 
     def refresh(self):
         self.setDirty(True)
@@ -62,13 +63,15 @@ class LayerPreview(QgsMapCanvas):
 
         if isinstance(item, Table) and item.type in [Table.VectorType, Table.RasterType]:
             # update the preview, but first let the manager chance to show the canvas
-            runPrev = lambda: self._loadTablePreview(item)
+            def runPrev():
+                return self._loadTablePreview(item)
+
             QTimer.singleShot(50, runPrev)
         else:
             return
 
         self.item = item
-        self.connect(self.item, SIGNAL('aboutToChange'), self.setDirty)
+        self.item.aboutToChange.connect(self.setDirty)
 
     def setDirty(self, val=True):
         self.dirty = val
@@ -76,9 +79,9 @@ class LayerPreview(QgsMapCanvas):
     def _clear(self):
         """ remove any layers from preview canvas """
         if self.item is not None:
-            ## skip exception on RuntimeError fixes #6892
+            # skip exception on RuntimeError fixes #6892
             try:
-                self.disconnect(self.item, SIGNAL('aboutToChange'), self.setDirty)
+                self.item.aboutToChange.disconnect(self.setDirty)
             except RuntimeError:
                 pass
 
@@ -88,45 +91,40 @@ class LayerPreview(QgsMapCanvas):
 
     def _loadTablePreview(self, table, limit=False):
         """ if has geometry column load to map canvas """
-        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
-        self.setRenderFlag(False)
-        vl = None
+        with OverrideCursor(Qt.WaitCursor):
+            self.freeze()
+            vl = None
 
-        if table and table.geomType:
-            # limit the query result if required
-            if limit and table.rowCount > 1000:
-                uniqueField = table.getValidQGisUniqueFields(True)
-                if uniqueField is None:
-                    self.parent.tabs.setCurrentWidget(self.parent.info)
-                    self.parent.infoBar.pushMessage(
-                        QApplication.translate("DBManagerPlugin", "Unable to find a valid unique field"),
-                        QgsMessageBar.WARNING, self.parent.iface.messageTimeout())
-                    return
+            if table and table.geomType:
+                # limit the query result if required
+                if limit and table.rowCount > 1000:
+                    uniqueField = table.getValidQgisUniqueFields(True)
+                    if uniqueField is None:
+                        self.parent.tabs.setCurrentWidget(self.parent.info)
+                        self.parent.infoBar.pushMessage(
+                            QApplication.translate("DBManagerPlugin", "Unable to find a valid unique field"),
+                            Qgis.Warning, self.parent.iface.messageTimeout())
+                        return
 
-                uri = table.database().uri()
-                uri.setDataSource("", u"(SELECT * FROM %s LIMIT 1000)" % table.quotedName(), table.geomColumn, "",
-                                  uniqueField.name)
-                provider = table.database().dbplugin().providerName()
-                vl = QgsVectorLayer(uri.uri(False), table.name, provider)
+                    uri = table.database().uri()
+                    uri.setDataSource("", "(SELECT * FROM %s LIMIT 1000)" % table.quotedName(), table.geomColumn, "",
+                                      uniqueField.name)
+                    provider = table.database().dbplugin().providerName()
+                    vl = QgsVectorLayer(uri.uri(False), table.name, provider)
+                else:
+                    vl = table.toMapLayer()
+
+                if vl and not vl.isValid():
+                    vl.deleteLater()
+                    vl = None
+
+            if vl and vl.isValid():
+                self.current_layer = vl
+                self.setLayers([self.current_layer])
+                self.zoomToFullExtent()
             else:
-                vl = table.toMapLayer()
+                self.setLayers([])
+                self.current_layer = None
 
-            if not vl.isValid():
-                vl.deleteLater()
-                vl = None
-
-        # remove old layer (if any) and set new
-        if self.currentLayer:
-            QgsMapLayerRegistry.instance().removeMapLayers([self.currentLayer.id()])
-
-        if vl:
-            self.setLayerSet([QgsMapCanvasLayer(vl)])
-            QgsMapLayerRegistry.instance().addMapLayers([vl], False)
-            self.zoomToFullExtent()
-        else:
-            self.setLayerSet([])
-
-        self.currentLayer = vl
-
-        self.setRenderFlag(True)
-        QApplication.restoreOverrideCursor()
+            self.freeze(False)
+            super().refresh()
