@@ -21,10 +21,6 @@ __author__ = 'Alexander Bruy'
 __date__ = 'July 2013'
 __copyright__ = '(C) 2013, Alexander Bruy'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '58734527abdbaab95719a48a4a64ea05abd70b44'
-
 import math
 from qgis.core import (QgsFeatureSink,
                        QgsGeometry,
@@ -41,7 +37,6 @@ from processing.algs.qgis.QgisAlgorithm import QgisAlgorithm
 
 
 class PointsDisplacement(QgisAlgorithm):
-
     INPUT = 'INPUT'
     DISTANCE = 'DISTANCE'
     PROXIMITY = 'PROXIMITY'
@@ -60,12 +55,18 @@ class PointsDisplacement(QgisAlgorithm):
     def initAlgorithm(self, config=None):
         self.addParameter(QgsProcessingParameterFeatureSource(self.INPUT,
                                                               self.tr('Input layer'), [QgsProcessing.TypeVectorPoint]))
-        self.addParameter(QgsProcessingParameterDistance(self.PROXIMITY,
-                                                         self.tr('Minimum distance to other points'), parentParameterName='INPUT',
-                                                         minValue=0.00001, defaultValue=1.0))
-        self.addParameter(QgsProcessingParameterDistance(self.DISTANCE,
-                                                         self.tr('Displacement distance'), parentParameterName='INPUT',
-                                                         minValue=0.00001, defaultValue=1.0))
+        param = QgsProcessingParameterDistance(self.PROXIMITY,
+                                               self.tr('Minimum distance to other points'), parentParameterName='INPUT',
+                                               minValue=0.00001, defaultValue=1.0)
+        param.setMetadata({'widget_wrapper': {'decimals': 5}})
+        self.addParameter(param)
+
+        param = QgsProcessingParameterDistance(self.DISTANCE,
+                                               self.tr('Displacement distance'), parentParameterName='INPUT',
+                                               minValue=0.00001, defaultValue=1.0)
+        param.setMetadata({'widget_wrapper': {'decimals': 5}})
+        self.addParameter(param)
+
         self.addParameter(QgsProcessingParameterBoolean(self.HORIZONTAL,
                                                         self.tr('Horizontal distribution for two point case')))
         self.addParameter(QgsProcessingParameterFeatureSink(self.OUTPUT, self.tr('Displaced'), QgsProcessing.TypeVectorPoint))
@@ -83,7 +84,7 @@ class PointsDisplacement(QgisAlgorithm):
 
         proximity = self.parameterAsDouble(parameters, self.PROXIMITY, context)
         radius = self.parameterAsDouble(parameters, self.DISTANCE, context)
-        horizontal = self.parameterAsBool(parameters, self.HORIZONTAL, context)
+        horizontal = self.parameterAsBoolean(parameters, self.HORIZONTAL, context)
 
         (sink, dest_id) = self.parameterAsSink(parameters, self.OUTPUT, context,
                                                source.fields(), source.wkbType(), source.sourceCrs())
