@@ -26,7 +26,8 @@ import os
 from qgis.core import (Qgis,
                        QgsMessageLog,
                        QgsApplication,
-                       QgsProcessingProvider)
+                       QgsProcessingProvider,
+                       QgsRuntimeProfiler)
 
 from processing.core.ProcessingConfig import ProcessingConfig, Setting
 
@@ -48,7 +49,7 @@ class ScriptAlgorithmProvider(QgsProcessingProvider):
     def __init__(self):
         super().__init__()
         self.algs = []
-        self.folder_algorithms = []
+        self.additional_algorithm_classes = []
         self.actions = [CreateNewScriptAction(),
                         AddScriptFromTemplateAction(),
                         OpenScriptFromFileAction(),
@@ -58,18 +59,19 @@ class ScriptAlgorithmProvider(QgsProcessingProvider):
                                    DeleteScriptAction()]
 
     def load(self):
-        ProcessingConfig.settingIcons[self.name()] = self.icon()
-        ProcessingConfig.addSetting(Setting(self.name(),
-                                            ScriptUtils.SCRIPTS_FOLDERS,
-                                            self.tr("Scripts folder(s)"),
-                                            ScriptUtils.defaultScriptsFolder(),
-                                            valuetype=Setting.MULTIPLE_FOLDERS))
+        with QgsRuntimeProfiler.profile('Script Provider'):
+            ProcessingConfig.settingIcons[self.name()] = self.icon()
+            ProcessingConfig.addSetting(Setting(self.name(),
+                                                ScriptUtils.SCRIPTS_FOLDERS,
+                                                self.tr("Scripts folder(s)"),
+                                                ScriptUtils.defaultScriptsFolder(),
+                                                valuetype=Setting.MULTIPLE_FOLDERS))
 
-        ProviderActions.registerProviderActions(self, self.actions)
-        ProviderContextMenuActions.registerProviderContextMenuActions(self.contextMenuActions)
+            ProviderActions.registerProviderActions(self, self.actions)
+            ProviderContextMenuActions.registerProviderContextMenuActions(self.contextMenuActions)
 
-        ProcessingConfig.readSettings()
-        self.refreshAlgorithms()
+            ProcessingConfig.readSettings()
+            self.refreshAlgorithms()
 
         return True
 
@@ -98,6 +100,13 @@ class ScriptAlgorithmProvider(QgsProcessingProvider):
         # they'll get an error if they use them with incompatible outputs...
         return True
 
+    def add_algorithm_class(self, algorithm_class):
+        """
+        Adds an algorithm class to the provider
+        """
+        self.additional_algorithm_classes.append(algorithm_class)
+        self.refreshAlgorithms()
+
     def loadAlgorithms(self):
         self.algs = []
         folders = ScriptUtils.scriptsFolders()
@@ -120,5 +129,8 @@ class ScriptAlgorithmProvider(QgsProcessingProvider):
                         if alg is not None:
                             self.algs.append(alg)
 
+        for alg_class in self.additional_algorithm_classes:
+            self.algs.append(alg_class())
+
         for a in self.algs:
             self.addAlgorithm(a)
