@@ -26,18 +26,29 @@
 import os
 import json
 import zipfile
+from functools import partial
 
+from qgis.PyQt import sip
 from qgis.PyQt.QtCore import Qt, QObject, QDir, QUrl, QFileInfo, QFile
-from qgis.PyQt.QtWidgets import QApplication, QDialog, QDialogButtonBox, QFrame, QMessageBox, QLabel, QVBoxLayout
+from qgis.PyQt.QtWidgets import (
+    QApplication,
+    QDialog,
+    QDialogButtonBox,
+    QFrame,
+    QMessageBox,
+    QLabel,
+    QVBoxLayout,
+    QPushButton
+)
 from qgis.PyQt.QtNetwork import QNetworkRequest
 
 import qgis
-from qgis.core import Qgis, QgsApplication, QgsNetworkAccessManager, QgsSettings, QgsNetworkRequestParameters
+from qgis.core import Qgis, QgsApplication, QgsNetworkAccessManager, QgsSettings, QgsSettingsTree, QgsNetworkRequestParameters
 from qgis.gui import QgsMessageBar, QgsPasswordLineEdit, QgsHelp
 from qgis.utils import (iface, startPlugin, unloadPlugin, loadPlugin, OverrideCursor,
                         reloadPlugin, updateAvailablePlugins, plugins_metadata_parser)
 from .installer_data import (repositories, plugins, officialRepo,
-                             settingsGroup, reposGroup, removeDir)
+                             reposGroup, removeDir)
 from .qgsplugininstallerinstallingdialog import QgsPluginInstallerInstallingDialog
 from .qgsplugininstallerpluginerrordialog import QgsPluginInstallerPluginErrorDialog
 from .qgsplugininstallerfetchingdialog import QgsPluginInstallerFetchingDialog
@@ -61,8 +72,6 @@ class QgsPluginInstaller(QObject):
 
     """ The main class for managing the plugin installer stuff"""
 
-    statusLabel = None
-
     # ----------------------------------------- #
     def __init__(self):
         """ Initialize data objects, starts fetching if appropriate, and warn about/removes obsolete plugins """
@@ -71,11 +80,10 @@ class QgsPluginInstaller(QObject):
         repositories.load()
         plugins.getAllInstalled()
 
+        self.message_bar_widget = None
+
         if repositories.checkingOnStart() and repositories.timeForChecking() and repositories.allEnabled():
             # start fetching repositories
-            self.statusLabel = QLabel(iface.mainWindow().statusBar())
-            iface.mainWindow().statusBar().addPermanentWidget(self.statusLabel)
-            self.statusLabel.linkActivated.connect(self.showPluginManagerWhenReady)
             repositories.checkingDone.connect(self.checkingDone)
             for key in repositories.allEnabled():
                 repositories.requestFetching(key)
@@ -142,35 +150,36 @@ class QgsPluginInstaller(QObject):
 
     # ----------------------------------------- #
     def checkingDone(self):
-        """ Remove the "Looking for new plugins..." label and display a notification instead if any updates or news available """
-        if not self.statusLabel:
-            # only proceed if the label is present
-            return
+        """ Remove the "Looking for new plugins..." label and display a notification instead if any updates available """
+
         # rebuild plugins cache
         plugins.rebuild()
         # look for news in the repositories
         plugins.markNews()
-        status = ""
-        icon = ""
-        # first check for news
-        for key in plugins.all():
-            if plugins.all()[key]["status"] == "new":
-                status = self.tr("There is a new plugin available")
-                icon = "pluginNew.svg"
-                tabIndex = 4  # PLUGMAN_TAB_NEW
+        status = None
         # then check for updates (and eventually overwrite status)
-        for key in plugins.all():
-            if plugins.all()[key]["status"] == "upgradeable":
-                status = self.tr("There is a plugin update available")
-                icon = "pluginUpgrade.svg"
-                tabIndex = 3  # PLUGMAN_TAB_UPGRADEABLE
+        updatable_count = 0
+        updatable_plugin_name = None
+        for _, properties in plugins.all().items():
+            if properties["status"] == "upgradeable":
+                updatable_count += 1
+                updatable_plugin_name = properties["name"]
+
+        if updatable_count:
+            if updatable_count > 1:
+                status = self.tr("Multiple plugin updates are available")
+            else:
+                status = self.tr("An update to the {} plugin is available").format(updatable_plugin_name)
+            tabIndex = 3  # PLUGMAN_TAB_UPGRADEABLE
+
         # finally set the notify label
         if status:
-            self.statusLabel.setText(u'<a href="%d"><img src="qrc:/images/themes/default/%s"></a>' % (tabIndex, icon))
-            self.statusLabel.setToolTip(status)
-        else:
-            iface.mainWindow().statusBar().removeWidget(self.statusLabel)
-            self.statusLabel = None
+            bar = iface.messageBar()
+            self.message_bar_widget = bar.createMessage('', status)
+            update_button = QPushButton("Install Updates…")
+            update_button.pressed.connect(partial(self.showPluginManagerWhenReady, tabIndex))
+            self.message_bar_widget.layout().addWidget(update_button)
+            bar.pushWidget(self.message_bar_widget, Qgis.Info)
 
     # ----------------------------------------- #
     def exportRepositoriesToManager(self):
@@ -258,9 +267,10 @@ class QgsPluginInstaller(QObject):
     def showPluginManagerWhenReady(self, * params):
         """ Open the plugin manager window. If fetching is still in progress, it shows the progress window first """
         """ Optionally pass the index of tab to be opened in params """
-        if self.statusLabel:
-            iface.mainWindow().statusBar().removeWidget(self.statusLabel)
-            self.statusLabel = None
+        if self.message_bar_widget:
+            if not sip.isdeleted(self.message_bar_widget):
+                iface.messageBar().popWidget(self.message_bar_widget)
+            self.message_bar_widget = None
 
         self.fetchAvailablePlugins(reloadMode=False)
         self.exportRepositoriesToManager()
@@ -283,7 +293,8 @@ class QgsPluginInstaller(QObject):
     # ----------------------------------------- #
     def exportSettingsGroup(self):
         """ Return QgsSettings settingsGroup value """
-        return settingsGroup
+        # todo QGIS 4 remove
+        return "plugin-manager"
 
     # ----------------------------------------- #
     def upgradeAllUpgradeable(self):
@@ -448,7 +459,7 @@ class QgsPluginInstaller(QObject):
             plugins.rebuild()
             self.exportPluginsToManager()
             QApplication.restoreOverrideCursor()
-            iface.pluginManagerInterface().pushMessage(self.tr("Plugin uninstalled successfully"), Qgis.Info)
+            iface.pluginManagerInterface().pushMessage(self.tr("Plugin uninstalled successfully"), Qgis.Success)
 
             settings = QgsSettings()
             settings.remove("/PythonPlugins/" + key)
@@ -567,9 +578,7 @@ class QgsPluginInstaller(QObject):
         if not os.path.isfile(filePath):
             return
 
-        settings = QgsSettings()
-        settings.setValue(settingsGroup + '/lastZipDirectory',
-                          QFileInfo(filePath).absoluteDir().absolutePath())
+        QgsSettingsTree.node("plugin-manager").childSetting("last-zip-directory").setValue(QFileInfo(filePath).absoluteDir().absolutePath())
 
         pluginName = None
         with zipfile.ZipFile(filePath, 'r') as zf:
@@ -652,6 +661,7 @@ class QgsPluginInstaller(QObject):
                 plugins.getAllInstalled()
                 plugins.rebuild()
 
+                settings = QgsSettings()
                 if settings.contains('/PythonPlugins/' + pluginName):  # Plugin was available?
                     if settings.value('/PythonPlugins/' + pluginName, False, bool):  # Plugin was also active?
                         reloadPlugin(pluginName)  # unloadPlugin + loadPlugin + startPlugin
@@ -667,7 +677,7 @@ class QgsPluginInstaller(QObject):
         else:
             msg = "<b>%s:</b> %s" % (self.tr("Plugin installation failed"), infoString)
 
-        level = Qgis.Info if success else Qgis.Critical
+        level = Qgis.Success if success else Qgis.Critical
         iface.pluginManagerInterface().pushMessage(msg, level)
 
     def processDependencies(self, plugin_id):
@@ -687,10 +697,10 @@ class QgsPluginInstaller(QObject):
                         self.installPlugin(dependency_plugin_id, stable=action_data['use_stable_version'])
                         if action_data['action'] == 'install':
                             iface.pluginManagerInterface().pushMessage(self.tr("Plugin dependency <b>%s</b> successfully installed") %
-                                                                       dependency_plugin_id, Qgis.Info)
+                                                                       dependency_plugin_id, Qgis.Success)
                         else:
                             iface.pluginManagerInterface().pushMessage(self.tr("Plugin dependency <b>%s</b> successfully upgraded") %
-                                                                       dependency_plugin_id, Qgis.Info)
+                                                                       dependency_plugin_id, Qgis.Success)
                     except Exception as ex:
                         if action_data['action'] == 'install':
                             iface.pluginManagerInterface().pushMessage(self.tr("Error installing plugin dependency <b>%s</b>: %s") %
